# -*- coding: utf-8 -*-
"""
Program:
    Cute Thermowell
    (qt-thermowell)
    FlangeData.py

Author:
    Alex Borisov <>

Copyright (c) 2011 Alex Borisov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from xml.etree.ElementTree import ElementTree, Element, SubElement
import sys
import resources

class FlangeDataDlg(QDialog):
    def __init__(self, flangedata_file,  parent = None):
        QDialog.__init__(self)

        self.parent = parent
        self.flangedata_file = flangedata_file
        self.script_path = sys.path[0]

        self.tree = ElementTree()

        self.MatDataUI()

    def MatDataUI(self):
        self.win_icon = QIcon(":/myresources/icons/flange.png")
        self.setWindowIcon(self.win_icon)
        self.win_size = QSize(500, 200)
        self.setWindowTitle("Edit Flange Data - Cute Thermowell")
        self.resize(self.win_size)

        self.icon_size = QSize(20, 20)
        self.btn_size = QSize(100, 25)
        self.l_size = QSize(160, 20)
        self.txt_size = QSize(130, 20)
        self.cb_size = QSize(130, 20)
        self.chk_size = QSize(150, 20)
        self.radio_size = QSize(80, 20)
        self.txt_max_length = 10

        self.dbl_validator = QDoubleValidator()
        self.dbl_validator.setNotation(QDoubleValidator.StandardNotation)

        self.h_spacer = QSpacerItem(2, 20, QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        self.v_spacer = QSpacerItem(20, 2, QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)

        self.icon_save = QIcon(":/myresources/icons/save.svg")
        self.icon_cancel = QIcon(":/myresources/icons/cancel.svg")

        self.gb_flangedata = QGroupBox("Flange Data", self)
        self.gb_flangedata.setStyleSheet(QString.fromUtf8("font-size: 8pt;"))
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_new_val = QLabel("New / Edit Values",  self.gb_flangedata)
        size_policy.setHeightForWidth(self.lb_new_val.sizePolicy().hasHeightForWidth())
        self.lb_new_val.setAlignment(Qt.AlignHCenter)
        self.lb_new_val.setSizePolicy(size_policy)
        self.lb_new_val.setMinimumSize(self.l_size)
        self.lb_material = QLabel("Material", self.gb_flangedata)
        size_policy.setHeightForWidth(self.lb_material.sizePolicy().hasHeightForWidth())
        self.lb_material.setSizePolicy(size_policy)
        self.lb_material.setMinimumSize(self.l_size)
        self.lb_temp = QLabel(self.gb_flangedata)
        self.lb_temp.setTextFormat(Qt.RichText)
        self.lb_temp.setText("Temperature (&#x00B0; C)")
        size_policy.setHeightForWidth(self.lb_temp.sizePolicy().hasHeightForWidth())
        self.lb_temp.setSizePolicy(size_policy)
        self.lb_temp.setMinimumSize(self.l_size)
        self.lb_class = QLabel("Class",  self.gb_flangedata)
        size_policy.setHeightForWidth(self.lb_class.sizePolicy().hasHeightForWidth())
        self.lb_class.setSizePolicy(size_policy)
        self.lb_class.setMinimumSize(self.l_size)
        self.lb_press = QLabel("Pressure (kPag)", self.gb_flangedata)
        size_policy.setHeightForWidth(self.lb_press.sizePolicy().hasHeightForWidth())
        self.lb_press.setSizePolicy(size_policy)
        self.lb_press.setMinimumSize(self.l_size)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.txt_material = QLineEdit("", self.gb_flangedata)
        size_policy.setHeightForWidth(self.txt_material.sizePolicy().hasHeightForWidth())
        self.txt_material.setSizePolicy(size_policy)
        self.txt_material.setMinimumSize(self.txt_size)
        self.txt_material.setMaxLength(20)
        self.txt_temp = QLineEdit("", self.gb_flangedata)
        size_policy.setHeightForWidth(self.txt_temp.sizePolicy().hasHeightForWidth())
        self.txt_temp.setSizePolicy(size_policy)
        self.txt_temp.setMinimumSize(self.txt_size)
        self.txt_temp.setMaxLength(3)
        self.txt_temp.setValidator(self.dbl_validator)
        self.txt_class = QLineEdit("", self.gb_flangedata)
        size_policy.setHeightForWidth(self.txt_class.sizePolicy().hasHeightForWidth())
        self.txt_class.setSizePolicy(size_policy)
        self.txt_class.setMinimumSize(self.txt_size)
        self.txt_class.setMaxLength(4)
        self.txt_class.setValidator(self.dbl_validator)
        self.txt_press = QLineEdit("", self.gb_flangedata)
        size_policy.setHeightForWidth(self.txt_press.sizePolicy().hasHeightForWidth())
        self.txt_press.setSizePolicy(size_policy)
        self.txt_press.setMinimumSize(self.txt_size)
        self.txt_press.setMaxLength(self.txt_max_length)
        self.txt_press.setValidator(self.dbl_validator)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.cb_material = QComboBox(self.gb_flangedata)
        size_policy.setHeightForWidth(self.cb_material.sizePolicy().hasHeightForWidth())
        self.cb_material.setSizePolicy(size_policy)
        self.cb_material.setMinimumSize(self.cb_size)
        self.cb_temp = QComboBox(self.gb_flangedata)
        size_policy.setHeightForWidth(self.cb_temp.sizePolicy().hasHeightForWidth())
        self.cb_temp.setSizePolicy(size_policy)
        self.cb_temp.setMinimumSize(self.cb_size)
        self.cb_class = QComboBox(self.gb_flangedata)
        size_policy.setHeightForWidth(self.cb_class.sizePolicy().hasHeightForWidth())
        self.cb_class.setSizePolicy(size_policy)
        self.cb_class.setMinimumSize(self.cb_size)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.radio_exist_material = QRadioButton("Existing", self.gb_flangedata)
        self.radio_exist_material.setSizePolicy(size_policy)
        self.radio_exist_material.setMinimumSize(self.radio_size)
        self.radio_new_material = QRadioButton("New", self.gb_flangedata)
        self.radio_new_material.setSizePolicy(size_policy)
        self.radio_new_material.setMinimumSize(self.radio_size)
        self.radio_exist_temp = QRadioButton("Existing", self.gb_flangedata)
        self.radio_exist_temp.setSizePolicy(size_policy)
        self.radio_exist_temp.setMinimumSize(self.radio_size)
        self.radio_new_temp = QRadioButton("New", self.gb_flangedata)
        self.radio_new_temp.setSizePolicy(size_policy)
        self.radio_new_temp.setMinimumSize(self.radio_size)
        self.radio_exist_class = QRadioButton("Existing", self.gb_flangedata)
        self.radio_exist_class.setSizePolicy(size_policy)
        self.radio_exist_class.setMinimumSize(self.radio_size)
        self.radio_new_class = QRadioButton("New", self.gb_flangedata)
        self.radio_new_class.setSizePolicy(size_policy)
        self.radio_new_class.setMinimumSize(self.radio_size)
        
        self.btngr_material = QButtonGroup()
        self.btngr_material.addButton(self.radio_exist_material)
        self.btngr_material.addButton(self.radio_new_material)
        self.radio_exist_material.setChecked(True)
        self.btngr_temp = QButtonGroup()
        self.btngr_temp.addButton(self.radio_exist_temp)
        self.btngr_temp.addButton(self.radio_new_temp)
        self.radio_exist_temp.setChecked(True)
        self.btngr_class = QButtonGroup()
        self.btngr_class.addButton(self.radio_exist_class)
        self.btngr_class.addButton(self.radio_new_class)
        self.radio_exist_class.setChecked(True)
       
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.chk_del_material = QCheckBox("Delete Material", self.gb_flangedata)
        self.chk_del_material.setSizePolicy(size_policy)
        self.chk_del_material.setMinimumSize(self.chk_size)
        self.chk_del_temp = QCheckBox("Delete Temperature", self.gb_flangedata)
        self.chk_del_temp.setSizePolicy(size_policy)
        self.chk_del_temp.setMinimumSize(self.chk_size)
        self.chk_del_class = QCheckBox("Delete Class", self.gb_flangedata)
        self.chk_del_class.setSizePolicy(size_policy)
        self.chk_del_class.setMinimumSize(self.chk_size)
        
        self.gb_flangedata_layout = QGridLayout(self.gb_flangedata)
        self.gb_flangedata_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gb_flangedata_layout.addItem(self.h_spacer, 0, 0, 2, 1)
        self.gb_flangedata_layout.addWidget(self.lb_new_val, 0, 2)
        self.gb_flangedata_layout.addWidget(self.lb_material, 1, 0)
        self.gb_flangedata_layout.addWidget(self.cb_material, 1, 1)
        self.gb_flangedata_layout.addWidget(self.txt_material, 1, 2)
        self.gb_flangedata_layout.addWidget(self.radio_exist_material,  1, 3)
        self.gb_flangedata_layout.addWidget(self.radio_new_material,  1, 4)
        self.gb_flangedata_layout.addWidget(self.chk_del_material,  1, 5)
        self.gb_flangedata_layout.addWidget(self.lb_temp, 2, 0)
        self.gb_flangedata_layout.addWidget(self.cb_temp, 2, 1)
        self.gb_flangedata_layout.addWidget(self.txt_temp, 2, 2)
        self.gb_flangedata_layout.addWidget(self.radio_exist_temp,  2, 3)
        self.gb_flangedata_layout.addWidget(self.radio_new_temp,  2, 4)
        self.gb_flangedata_layout.addWidget(self.chk_del_temp,  2, 5)
        self.gb_flangedata_layout.addWidget(self.lb_class, 3, 0)
        self.gb_flangedata_layout.addWidget(self.cb_class, 3, 1)
        self.gb_flangedata_layout.addWidget(self.txt_class, 3, 2)
        self.gb_flangedata_layout.addWidget(self.radio_exist_class,  3, 3)
        self.gb_flangedata_layout.addWidget(self.radio_new_class,  3, 4)
        self.gb_flangedata_layout.addWidget(self.chk_del_class,  3, 5)
        self.gb_flangedata_layout.addWidget(self.lb_press, 4, 0)
        self.gb_flangedata_layout.addItem(self.h_spacer, 4, 1)
        self.gb_flangedata_layout.addWidget(self.txt_press, 4, 2)
        self.gb_flangedata_layout.addItem(self.h_spacer, 4, 3)
        self.gb_flangedata_layout.addItem(self.h_spacer, 0, 6, 5, 1)
        self.gb_flangedata_layout.addItem(self.v_spacer, 5, 0)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_status = QLabel("", self)
        self.lb_status.setTextFormat(Qt.RichText)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.btn_save = QPushButton(self.icon_save, "Save", self)
        size_policy.setHeightForWidth(self.btn_save.sizePolicy().hasHeightForWidth())
        self.btn_save.setSizePolicy(size_policy)
        self.btn_save.setMinimumSize(self.btn_size)
        self.btn_cancel = QPushButton(self.icon_cancel, "Cancel", self)
        size_policy.setHeightForWidth(self.btn_cancel.sizePolicy().hasHeightForWidth())
        self.btn_cancel.setSizePolicy(size_policy)
        self.btn_cancel.setMinimumSize(self.btn_size)

        self.hb_button_layout = QHBoxLayout()
        self.hb_button_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.hb_button_layout.addWidget(self.lb_status)
        self.hb_button_layout.addItem(self.h_spacer)
        self.hb_button_layout.addWidget(self.btn_save)
        self.hb_button_layout.addWidget(self.btn_cancel)
        
        self.main_layout = QVBoxLayout(self)
        self.main_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.main_layout.addWidget(self.gb_flangedata)
        self.main_layout.addLayout(self.hb_button_layout)
        
        self.cb_material.addItems(self.GetMaterials())

        self.timer = QTimer(self)
        self.timer.setInterval(5000)
        
        self.connect(self.timer, SIGNAL("timeout()") , self.TimerHit)
        self.connect(self.btn_save, SIGNAL("clicked()"), self.OnBtnSaveClicked)
        self.connect(self.btn_cancel, SIGNAL("clicked()"), self.Close)
        self.connect(self.cb_material, SIGNAL("currentIndexChanged(int)"), self.OnMaterialCurrentIndexChanged)
        self.connect(self.cb_temp, SIGNAL("currentIndexChanged(int)"), self.OnTempCurrentIndexChanged)
        self.connect(self.cb_class, SIGNAL("currentIndexChanged(int)"), self.OnClassCurrentIndexChanged)
        self.connect(self.radio_exist_material, SIGNAL("toggled(bool)"), self.OnExistMaterialToggled)
        self.connect(self.radio_new_material, SIGNAL("toggled(bool)"), self.OnNewMaterialToggled)
        self.connect(self.radio_exist_temp, SIGNAL("toggled(bool)"), self.OnExistTempToggled)
        self.connect(self.radio_new_temp, SIGNAL("toggled(bool)"), self.OnNewTempToggled)
        self.connect(self.radio_exist_class, SIGNAL("toggled(bool)"), self.OnExistClassToggled)
        self.connect(self.radio_new_class, SIGNAL("toggled(bool)"), self.OnNewClassToggled)

    def GetMaterials(self):
        self.flanges_xml = self.tree.parse(self.flangedata_file)
        self.materials = self.flanges_xml.getiterator("material")
        matlist = QStringList("")
        for material in self.materials:
            matlist.append(material.attrib["name"])
        return matlist

    def OnMaterialCurrentIndexChanged(self):
        self.cb_temp.clear()
        self.ClearInputs()
        material = self.cb_material.currentText()
        if material != "":
            self.txt_material.setText(material)
            templist = self.GetTemp(material)
            self.cb_temp.addItems(templist)
        else:
            self.ClearInputs()

    def GetTemp(self,  matvar):
        templist = QStringList("")
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    templist.append(temp.text)
                return templist

    def OnTempCurrentIndexChanged(self):
        self.cb_class.clear()
        material = self.cb_material.currentText()
        temp = self.cb_temp.currentText()
        self.txt_temp.setText(temp)
        if material != "" and temp != "":
            classlist = self.GetClass(material,  temp)
            self.cb_class.addItems(classlist)
        else:
            self.cb_class.clear()

    def GetClass(self,  matvar,  tempvar):
        classlist = QStringList("")
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        classes = temp.getiterator("class")
                        for fclass in classes:
                            classlist.append(fclass.text)
                        return classlist

    def OnClassCurrentIndexChanged(self):
        material = self.cb_material.currentText()
        temp = self.cb_temp.currentText()
        fclass = self.cb_class.currentText()
        self.txt_class.setText(fclass)
        if fclass != "":
            press = self.GetFlangeData(material,  temp,  fclass,  "pressure")
            press = str(float(press) / 1000)
            self.txt_press.setText(press)
        else:
            self.txt_press.setText("")

    def GetFlangeData(self,  matvar,  tempvar,  classvar,  tag):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        classes = temp.getiterator("class")
                        for fclass in classes:
                            if fclass.text == classvar:
                                press = fclass.find(tag)
                                return press.text

    def OnExistMaterialToggled(self):
        self.lb_material.setText("Material")
        self.cb_material.show()
        self.cb_material.addItem("")
        self.cb_material.addItems(self.GetMaterials())
        self.cb_material.setCurrentIndex(0)
        self.chk_del_material.show()
        self.chk_del_material.setChecked(False)
        self.chk_del_temp.setChecked(False)
        self.radio_exist_temp.show()
        self.radio_exist_temp.setChecked(True)

    def OnNewMaterialToggled(self):
        self.lb_material.setText("New Material")
        self.cb_material.hide()
        self.cb_material.clear()
        self.chk_del_material.hide()
        self.chk_del_material.setChecked(False)
        self.chk_del_temp.setChecked(False)
        self.radio_exist_temp.hide()
        self.radio_new_temp.setChecked(True)
        self.ClearInputs()

    def OnExistTempToggled(self):
        self.lb_temp.setText("Temperature (&#x00B0; C)")
        self.cb_temp.show()
        self.OnMaterialCurrentIndexChanged()
        self.chk_del_temp.show()
        self.chk_del_temp.setChecked(False)
        self.chk_del_class.setChecked(False)
        self.radio_exist_class.show()
        self.radio_exist_class.setChecked(True)

    def OnNewTempToggled(self):
        self.lb_temp.setText("New Temperature (&#x00B0; C)")
        self.cb_temp.clear()
        self.cb_temp.hide()
        self.chk_del_temp.hide()
        self.chk_del_temp.setChecked(False)
        self.chk_del_class.setChecked(False)
        self.radio_exist_class.hide()
        self.radio_new_class.setChecked(True)

    def OnExistClassToggled(self):
        self.lb_class.setText("Class")
        self.cb_class.show()
        self.OnTempCurrentIndexChanged()
        self.chk_del_class.show()
        self.chk_del_class.setChecked(False)

    def OnNewClassToggled(self):
        self.lb_class.setText("New Class")
        self.cb_class.clear()
        self.cb_class.hide()
        self.chk_del_class.hide()
        self.chk_del_class.setChecked(False)

    def OnBtnSaveClicked(self):
        try:
            if self.SaveChangesDlg():
                if not self.MakeDataBackup():
                        self.ShowStatus("Could not back up data file", "red")
                material = self.cb_material.currentText()
                new_material = self.txt_material.text()
                temp = self.cb_temp.currentText()
                new_temp = self.txt_temp.text()
                fclass = self.cb_class.currentText()
                new_class = self.txt_class.text()
                press = self.txt_press.text()
                if self.chk_del_material.isChecked() and material != "":
                    if self.DeleteMaterial(material):
                        self.Close()
                    else:
                        self.ShowStatus("Could not delete material and associated data", "red")
                elif self.chk_del_temp.isChecked() and temp != "":
                    if self.DeleteTemp(material,  temp):
                        self.Close()
                    else:
                        self.ShowStatus("Could not delete temperature and associated data", "red")
                elif self.chk_del_class.isChecked() and fclass != "":
                    if self.DeleteClass(material,  temp,  fclass):
                        self.Close()
                    else:
                        self.ShowStatus("Could not delete class and associated data", "red")
                elif new_material != "" and new_temp != "" and new_class != "" and press != "":
                    press = str(float(press) * 1000)
                    if self.SaveData(material, new_material, temp, new_temp, fclass, new_class, press):
                        self.Close()
                    else:
                        self.ShowStatus("Could not save data", "red")
                else:
                    self.ShowStatus("Could not save changes, some fields are empty", "red")
        except Exception, inst:
            QMessageBox.warning(self.parent, "Error - Cute Thermowell", "Unexpected error:\n %s" % (inst))

    def SaveChangesDlg(self):
        msg = "You are about to make changes to material data file.\n"
        msg += "A backup copy of the data file will be created.\n"
        msg += "Press OK to save changes or Cancel to return."
        dlg = QMessageBox(self)
        dlg.setIcon(QMessageBox.Warning)
        dlg.setWindowTitle("Save Changes - Cute Thermowell")
        dlg.setText(msg)
        dlg.addButton(QMessageBox.Ok)
        dlg.addButton(QMessageBox.Cancel)
        dlg.setDefaultButton(QMessageBox.Cancel)
        button = dlg.exec_()
        if (button == QMessageBox.Ok):
            return True
        else:
            return False        

    def MakeDataBackup(self):
        data_file_info = QFileInfo(self.flangedata_file)
        data_file_dir = self.script_path + "/data"
        data_file_base = data_file_info.completeBaseName()
        data_file_ext = data_file_info.completeSuffix()
        backup_file_ext = data_file_info.completeSuffix() + "~"
        data_file_name = data_file_dir + "/" + data_file_base + "." + data_file_ext
        backup_file_name = data_file_dir + "/" + data_file_base + "." + backup_file_ext
        data_file = QFile(data_file_name)
        backup_file = QFile(backup_file_name)
        if self.FileExists(data_file_name):
            if self.FileExists(backup_file_name):
                backup_file.remove()
                data_file.copy(backup_file_name)
                return True
            else:
                data_file.copy(backup_file_name)
                return True
        else:
            return False
        
    def FileExists(self, file_name):
        file = QFile(file_name)
        if (file.exists()):
            return True
        else:
            return False

    def ClearInputs(self):
        self.txt_material.setText("")
        self.txt_temp.setText("")
        self.txt_class.setText("")
        self.txt_press.setText("")
        self.lb_status.setText("<font color=\"blue\"></font>")

    def Close(self):
        self.close()

    def DeleteMaterial(self,  matvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                self.flanges_xml.remove(material)
                self.SaveXML()
                return True

    def DeleteTemp(self,  matvar,  tempvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        material.remove(temp)
                        self.SaveXML()
                        return True

    def DeleteClass(self,  matvar,  tempvar,  classvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        classes = temp.getiterator("class")
                        for fclass in classes:
                            if fclass.text == classvar:
                                temp.remove(fclass)
                                self.SaveXML()
                                return True

    def SaveData(self, matvar, new_matvar, tempvar, new_tempvar, classvar, new_classvar, pressvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        classes = temp.getiterator("class")
                        for fclass in classes:
                            if fclass.text == classvar:
                                fclass.text = unicode(new_classvar)
                                items = fclass.getchildren()
                                items[0].text = unicode(pressvar)
                                break
                            elif classvar == "" and fclass.text == new_classvar:
                                fclass.text = unicode(new_classvar)
                                items = fclass.getchildren()
                                items[0].text = unicode(pressvar)
                                break
                            elif classes.index(fclass) == len(classes) - 1 and fclass.text != new_classvar:
                                new_class = SubElement(temp, "class")
                                new_class.text = unicode(new_classvar)
                                new_press = SubElement(new_class, "pressure")
                                new_press.text = unicode(pressvar)
                        temp.text = unicode(new_tempvar)
                        material.attrib["name"] = unicode(new_matvar)
                        self.SaveXML()
                        return True
                    elif tempvar == "" and temp.text == new_tempvar:
                        classes = temp.getiterator("class")
                        for fclass in classes:
                            if fclass.text == classvar:
                                fclass.text = unicode(new_classvar)
                                items = fclass.getchildren()
                                items[0].text = unicode(pressvar)
                                break
                            elif classvar == "" and fclass.text == new_classvar:
                                fclass.text = unicode(new_classvar)
                                items = fclass.getchildren()
                                items[0].text = unicode(pressvar)
                                break
                            elif classes.index(fclass) == len(classes) - 1 and fclass.text != new_classvar:
                                new_class = SubElement(temp, "class")
                                new_class.text = unicode(new_classvar)
                                new_press = SubElement(new_class, "pressure")
                                new_press.text = unicode(pressvar)
                        temp.text = unicode(new_tempvar)
                        material.attrib["name"] = unicode(new_matvar)
                        self.SaveXML()
                        return True
                    elif temps.index(temp) == len(temps) - 1 and temp.text != new_tempvar:
                        new_temp = SubElement(material, "temp")
                        new_temp.text = unicode(new_tempvar)
                        new_class = SubElement(new_temp, "class")
                        new_class.text = unicode(new_classvar)
                        new_press = SubElement(new_class, "pressure")
                        new_press.text = unicode(pressvar)
                material.attrib["name"] = unicode(new_matvar)
                self.SaveXML()
                return True
            elif matvar == "" and material.attrib["name"] == new_matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        classes = temp.getiterator("class")
                        for fclass in classes:
                            if fclass.text == classvar:
                                fclass.text = unicode(new_classvar)
                                items = fclass.getchildren()
                                items[0].text = unicode(pressvar)
                                break
                            elif classvar == "" and fclass.text == new_classvar:
                                fclass.text = unicode(new_classvar)
                                items = fclass.getchildren()
                                items[0].text = unicode(pressvar)
                                break
                            elif classes.index(fclass) == len(classes) - 1 and fclass.text != new_classvar:
                                new_class = SubElement(temp, "class")
                                new_class.text = unicode(new_classvar)
                                new_press = SubElement(new_class, "pressure")
                                new_press.text = unicode(pressvar)
                        temp.text = unicode(new_tempvar)
                        material.attrib["name"] = unicode(new_matvar)
                        self.SaveXML()
                        return True
                    elif tempvar == "" and temp.text == new_tempvar:
                        classes = temp.getiterator("class")
                        for fclass in classes:
                            if fclass.text == classvar:
                                fclass.text = unicode(new_classvar)
                                items = fclass.getchildren()
                                items[0].text = unicode(pressvar)
                                break
                            elif classvar == "" and fclass.text == new_classvar:
                                fclass.text = unicode(new_classvar)
                                items = fclass.getchildren()
                                items[0].text = unicode(pressvar)
                                break
                            elif classes.index(fclass) == len(classes) - 1 and fclass.text != new_classvar:
                                new_class = SubElement(temp, "class")
                                new_class.text = unicode(new_classvar)
                                new_press = SubElement(new_class, "pressure")
                                new_press.text = unicode(pressvar)
                        temp.text = unicode(new_tempvar)
                        material.attrib["name"] = unicode(new_matvar)
                        self.SaveXML()
                        return True
                    elif temps.index(temp) == len(temps) - 1 and temp.text != new_tempvar:
                        new_temp = SubElement(material, "temp")
                        new_temp.text = unicode(new_tempvar)
                        new_class = SubElement(new_temp, "class")
                        new_class.text = unicode(new_classvar)
                        new_press = SubElement(new_class, "pressure")
                        new_press.text = unicode(pressvar)
                material.attrib["name"] = unicode(new_matvar)
                self.SaveXML()
                return True
            elif self.materials.index(material) == len(self.materials) - 1 and material.attrib["name"] != new_matvar:
                new_material = Element("material", name = unicode(new_matvar))
                new_temp = SubElement(new_material, "temp")
                new_temp.text = unicode(new_tempvar)
                new_class = SubElement(new_temp, "class")
                new_class.text = unicode(new_classvar)
                new_press = SubElement(new_class, "pressure")
                new_press.text = unicode(pressvar)
                self.tree.getroot().append(new_material)
                self.SaveXML()
                return True
        return False
        
    def SaveXML(self):
        self.xmlIndent(self.tree.getroot())
        self.tree.write(self.flangedata_file, encoding="utf-8")
    
    """
    def Prettify(self, elem):
        ugly_string = ElementTree.tostring(elem, 'utf-8')
        pretty_string = minidom.parseString(ugly_string)
        return pretty_string.toprettyxml(indent="  ")
    """

    def xmlIndent(self, elem, level=0):
        i = "\n" + level*"  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self.xmlIndent(elem, level+1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i

    def closeEvent(self, event):
        self.ClearInputs()
        self.chk_del_material.setChecked(False)
        self.chk_del_temp.setChecked(False)
        self.chk_del_class.setChecked(False)
        self.radio_exist_material.setChecked(True)
        self.radio_exist_temp.setChecked(True)
        self.radio_exist_class.setChecked(True)
        self.cb_material.clear()
        self.cb_material.addItems(self.GetMaterials())
        self.cb_material.setCurrentIndex(0)
        self.emit(SIGNAL("closed()"))

    def StartTimer(self):
        self.timer.start()
    
    def TimerHit(self):
        self.lb_status.setText("")

    def ShowStatus(self, msg,  color):
        self.lb_status.setText("<font color=\"" + color + "\">" + msg + "</font>")
        self.StartTimer()
