# -*- coding: utf-8 -*-
"""
Program:
    Cute Thermowell
    (qt-thermowell)
    FormWidget.py
    
Author:
    Alex Borisov <>
     
Copyright (c) 2011 Alex Borisov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
import sys
import resources
from xml.etree.ElementTree import ElementTree, Element, SubElement
from TaperedCalc import TaperedCalc
from StraightCalc import StraightCalc
from StepCalc import StepCalc
from TWImageWidget import TWImageWidget

class FormWidget(QWidget):
    def __init__(self, materialdata_file = None, flangedata_file = None, pipedata_file = None, results_file = None,  parent = None):
        QWidget.__init__(self)
        
        self.parent = parent
        self.materialdata_file = materialdata_file
        self.flangedata_file = flangedata_file
        self.pipedata_file = pipedata_file
        self.results_file = results_file
        self.main_widget = QWidget()
        
        self.tree = ElementTree()

        self.StartUI()
        
    def StartUI(self):

        self.l_size = QSize(100, 20)
        self.l_size_short = QSize(50, 20)
        self.txt_size = QSize(120, 20)
        self.cb_size = QSize(120, 20)
        self.chk_size = QSize(120, 20)
        self.fr_size = QSize(310, 180)
        self.txt_max_length = 10
        #self.main_widget.setStyleSheet(QString.fromUtf8("font-size: 8pt;"))
        
        """
        palette = QPalette()
        palette.setColor(QPalette.Text, Qt.blue)
        palette.setColor(QPalette.ButtonText, Qt.blue)
        palette.setColor(QPalette.WindowText, Qt.blue)
        self.setPalette(palette)
        """

        self.dbl_validator = QDoubleValidator()
        self.dbl_validator.setNotation(QDoubleValidator.StandardNotation)

        self.h_spacer = QSpacerItem(2, 20, QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        self.v_spacer = QSpacerItem(20, 2, QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)
        
        self.gb_TW_dim = QGroupBox("Thermowell Dimensions", self.main_widget)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_TW_out_dia = QLabel("A", self.gb_TW_dim)
        self.lb_TW_out_dia.setToolTip("Outer Diameter")
        size_policy.setHeightForWidth(self.lb_TW_out_dia.sizePolicy().hasHeightForWidth())
        self.lb_TW_out_dia.setSizePolicy(size_policy)
        self.lb_TW_out_dia.setMinimumSize(self.l_size_short)
        self.lb_TW_out_dia_unit = QLabel("mm", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.lb_TW_out_dia_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_out_dia_unit.setSizePolicy(size_policy)
        self.lb_TW_out_dia_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_tip_dia = QLabel("B", self.gb_TW_dim)
        self.lb_TW_tip_dia.setToolTip("Tip Diameter")
        size_policy.setHeightForWidth(self.lb_TW_tip_dia.sizePolicy().hasHeightForWidth())
        self.lb_TW_tip_dia.setSizePolicy(size_policy)
        self.lb_TW_tip_dia.setMinimumSize(self.l_size_short)
        self.lb_TW_tip_dia_unit = QLabel("mm", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.lb_TW_tip_dia_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_tip_dia_unit.setSizePolicy(size_policy)
        self.lb_TW_tip_dia_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_fillet_rad = QLabel("b", self.gb_TW_dim)
        self.lb_TW_fillet_rad.setToolTip("Fillet Radius at Root of Thermowell Shank")
        size_policy.setHeightForWidth(self.lb_TW_fillet_rad.sizePolicy().hasHeightForWidth())
        self.lb_TW_fillet_rad.setSizePolicy(size_policy)
        self.lb_TW_fillet_rad.setMinimumSize(self.l_size_short)
        self.lb_TW_fillet_rad_unit = QLabel("mm", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.lb_TW_fillet_rad_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_fillet_rad_unit.setSizePolicy(size_policy)
        self.lb_TW_fillet_rad_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_step_fillet_rad = QLabel(self.gb_TW_dim)
        self.lb_TW_step_fillet_rad.setTextFormat(Qt.RichText)
        self.lb_TW_step_fillet_rad.setText("b<sub>S</sub>")
        self.lb_TW_step_fillet_rad.setToolTip("Fillet Radius at Base of Reduced Diameter Thermowell Shank")
        size_policy.setHeightForWidth(self.lb_TW_step_fillet_rad.sizePolicy().hasHeightForWidth())
        self.lb_TW_step_fillet_rad.setSizePolicy(size_policy)
        self.lb_TW_step_fillet_rad.setMinimumSize(self.l_size_short)
        self.lb_TW_step_fillet_rad.hide()
        self.lb_TW_step_fillet_rad_unit = QLabel("mm", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.lb_TW_step_fillet_rad_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_step_fillet_rad_unit.setSizePolicy(size_policy)
        self.lb_TW_step_fillet_rad_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_step_fillet_rad_unit.hide()
        self.lb_TW_bore_dia = QLabel("d", self.gb_TW_dim)
        self.lb_TW_bore_dia.setToolTip("Bore Diameter")
        size_policy.setHeightForWidth(self.lb_TW_bore_dia.sizePolicy().hasHeightForWidth())
        self.lb_TW_bore_dia.setSizePolicy(size_policy)
        self.lb_TW_bore_dia.setMinimumSize(self.l_size_short)
        self.lb_TW_bore_dia_unit = QLabel("mm", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.lb_TW_bore_dia_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_bore_dia_unit.setSizePolicy(size_policy)
        self.lb_TW_bore_dia_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_length = QLabel("L", self.gb_TW_dim)
        self.lb_TW_length.setToolTip("Unsupported Length of Thermowell")
        size_policy.setHeightForWidth(self.lb_TW_length.sizePolicy().hasHeightForWidth())
        self.lb_TW_length.setSizePolicy(size_policy)
        self.lb_TW_length.setMinimumSize(self.l_size_short)
        self.lb_TW_length_unit = QLabel("mm", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.lb_TW_length_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_length_unit.setSizePolicy(size_policy)
        self.lb_TW_length_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_shield_length = QLabel(self.gb_TW_dim)
        self.lb_TW_shield_length.setTextFormat(Qt.RichText)
        self.lb_TW_shield_length.setText("L<sub>0</sub>")
        self.lb_TW_shield_length.setToolTip("Length of Thermowell Shielded from Fluid Flow")
        size_policy.setHeightForWidth(self.lb_TW_shield_length.sizePolicy().hasHeightForWidth())
        self.lb_TW_shield_length.setSizePolicy(size_policy)
        self.lb_TW_shield_length.setMinimumSize(self.l_size_short)
        self.lb_TW_shield_length_unit = QLabel("mm", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.lb_TW_shield_length_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_shield_length_unit.setSizePolicy(size_policy)
        self.lb_TW_shield_length_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_step_length = QLabel(self.gb_TW_dim)
        self.lb_TW_step_length.setTextFormat(Qt.RichText)
        self.lb_TW_step_length.setText("L<sub>S</sub>")
        self.lb_TW_step_length.setToolTip("Length of Reduced Diameter Shank")
        size_policy.setHeightForWidth(self.lb_TW_step_length.sizePolicy().hasHeightForWidth())
        self.lb_TW_step_length.setSizePolicy(size_policy)
        self.lb_TW_step_length.setMinimumSize(self.l_size_short)
        self.lb_TW_step_length.hide()
        self.lb_TW_step_length_unit = QLabel("mm", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.lb_TW_step_length_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_step_length_unit.setSizePolicy(size_policy)
        self.lb_TW_step_length_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_step_length_unit.hide()
        self.lb_TW_tip_thick = QLabel("t", self.gb_TW_dim)
        self.lb_TW_tip_thick.setToolTip("Tip Thickness")
        size_policy.setHeightForWidth(self.lb_TW_tip_thick.sizePolicy().hasHeightForWidth())
        self.lb_TW_tip_thick.setSizePolicy(size_policy)
        self.lb_TW_tip_thick.setMinimumSize(self.l_size_short)
        self.lb_TW_tip_thick_unit = QLabel("mm", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.lb_TW_tip_thick_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_tip_thick_unit.setSizePolicy(size_policy)
        self.lb_TW_tip_thick_unit.setMinimumSize(self.l_size_short)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.cb_TW_step_tip_dia = QComboBox(self.gb_TW_dim)
        size_policy.setHeightForWidth(self.cb_TW_step_tip_dia.sizePolicy().hasHeightForWidth())
        self.cb_TW_step_tip_dia.setSizePolicy(size_policy)
        self.cb_TW_step_tip_dia.setMinimumSize(self.cb_size)
        self.cb_TW_step_tip_dia.addItem("")
        self.cb_TW_step_tip_dia.addItem("12.70")
        self.cb_TW_step_tip_dia.addItem("22.23")
        self.cb_TW_step_tip_dia.hide()

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.txt_TW_out_dia = QLineEdit("", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.txt_TW_out_dia.sizePolicy().hasHeightForWidth())
        self.txt_TW_out_dia.setSizePolicy(size_policy)
        self.txt_TW_out_dia.setMinimumSize(self.txt_size)
        self.txt_TW_out_dia.setMaxLength(self.txt_max_length)
        self.txt_TW_out_dia.setValidator(self.dbl_validator)
        self.txt_TW_tip_dia = QLineEdit("", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.txt_TW_tip_dia.sizePolicy().hasHeightForWidth())
        self.txt_TW_tip_dia.setSizePolicy(size_policy)
        self.txt_TW_tip_dia.setMinimumSize(self.txt_size)
        self.txt_TW_tip_dia.setMaxLength(self.txt_max_length)
        self.txt_TW_tip_dia.setValidator(self.dbl_validator)
        self.txt_TW_bore_dia = QLineEdit("", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.txt_TW_bore_dia.sizePolicy().hasHeightForWidth())
        self.txt_TW_bore_dia.setSizePolicy(size_policy)
        self.txt_TW_bore_dia.setMinimumSize(self.txt_size)
        self.txt_TW_bore_dia.setMaxLength(self.txt_max_length)
        self.txt_TW_bore_dia.setValidator(self.dbl_validator)
        self.txt_TW_fillet_rad = QLineEdit("0", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.txt_TW_fillet_rad.sizePolicy().hasHeightForWidth())
        self.txt_TW_fillet_rad.setSizePolicy(size_policy)
        self.txt_TW_fillet_rad.setMinimumSize(self.txt_size)
        self.txt_TW_fillet_rad.setMaxLength(self.txt_max_length)
        self.txt_TW_fillet_rad.setValidator(self.dbl_validator)
        self.txt_TW_step_fillet_rad = QLineEdit("0", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.txt_TW_step_fillet_rad.sizePolicy().hasHeightForWidth())
        self.txt_TW_step_fillet_rad.setSizePolicy(size_policy)
        self.txt_TW_step_fillet_rad.setMinimumSize(self.txt_size)
        self.txt_TW_step_fillet_rad.setMaxLength(self.txt_max_length)
        self.txt_TW_step_fillet_rad.setValidator(self.dbl_validator)
        self.txt_TW_step_fillet_rad.hide()
        self.txt_TW_length = QLineEdit("", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.txt_TW_length.sizePolicy().hasHeightForWidth())
        self.txt_TW_length.setSizePolicy(size_policy)
        self.txt_TW_length.setMinimumSize(self.txt_size)
        self.txt_TW_length.setMaxLength(self.txt_max_length)
        self.txt_TW_length.setValidator(self.dbl_validator)
        self.txt_TW_shield_length = QLineEdit("", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.txt_TW_shield_length.sizePolicy().hasHeightForWidth())
        self.txt_TW_shield_length.setSizePolicy(size_policy)
        self.txt_TW_shield_length.setMinimumSize(self.txt_size)
        self.txt_TW_shield_length.setMaxLength(self.txt_max_length)
        self.txt_TW_shield_length.setValidator(self.dbl_validator)
        self.txt_TW_step_length = QLineEdit("", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.txt_TW_step_length.sizePolicy().hasHeightForWidth())
        self.txt_TW_step_length.setSizePolicy(size_policy)
        self.txt_TW_step_length.setMinimumSize(self.txt_size)
        self.txt_TW_step_length.setMaxLength(self.txt_max_length)
        self.txt_TW_step_length.setValidator(self.dbl_validator)
        self.txt_TW_step_length.hide()
        self.txt_TW_tip_thick = QLineEdit("", self.gb_TW_dim)
        size_policy.setHeightForWidth(self.txt_TW_tip_thick.sizePolicy().hasHeightForWidth())
        self.txt_TW_tip_thick.setSizePolicy(size_policy)
        self.txt_TW_tip_thick.setMinimumSize(self.txt_size)
        self.txt_TW_tip_thick.setMaxLength(self.txt_max_length)
        self.txt_TW_tip_thick.setValidator(self.dbl_validator)

        self.gb_TW_dim_layout = QGridLayout(self.gb_TW_dim)
        self.gb_TW_dim_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gb_TW_dim_layout.addWidget(self.lb_TW_out_dia, 0, 0)
        self.gb_TW_dim_layout.addWidget(self.txt_TW_out_dia, 0, 1)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_out_dia_unit, 0, 2)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_tip_dia, 1, 0)
        self.gb_TW_dim_layout.addWidget(self.txt_TW_tip_dia, 1, 1)
        self.gb_TW_dim_layout.addWidget(self.cb_TW_step_tip_dia, 1, 1)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_tip_dia_unit, 1, 2)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_bore_dia, 2, 0)
        self.gb_TW_dim_layout.addWidget(self.txt_TW_bore_dia, 2, 1)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_bore_dia_unit, 2, 2)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_fillet_rad, 3, 0)
        self.gb_TW_dim_layout.addWidget(self.txt_TW_fillet_rad, 3, 1)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_fillet_rad_unit, 3, 2)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_tip_thick, 4, 0)
        self.gb_TW_dim_layout.addWidget(self.txt_TW_tip_thick, 4, 1)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_tip_thick_unit, 4, 2)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_length, 5, 0)
        self.gb_TW_dim_layout.addWidget(self.txt_TW_length, 5, 1)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_length_unit, 5, 2)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_shield_length, 6, 0)
        self.gb_TW_dim_layout.addWidget(self.txt_TW_shield_length, 6, 1)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_shield_length_unit, 6, 2)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_step_fillet_rad, 7, 0)
        self.gb_TW_dim_layout.addWidget(self.txt_TW_step_fillet_rad, 7, 1)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_step_fillet_rad_unit, 7, 2)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_step_length, 8, 0)
        self.gb_TW_dim_layout.addWidget(self.txt_TW_step_length, 8, 1)
        self.gb_TW_dim_layout.addWidget(self.lb_TW_step_length_unit, 8, 2)
        self.gb_TW_dim_layout.addItem(self.h_spacer, 0, 3, 9, 1)
        self.gb_TW_dim_layout.addItem(self.v_spacer, 9, 0)

        self.gb_TW_select = QGroupBox("Thermowell Selection", self.main_widget)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_TW_type = QLabel("Type", self.gb_TW_select)
        self.lb_TW_type.setToolTip("Thermowell Type")
        size_policy.setHeightForWidth(self.lb_TW_type.sizePolicy().hasHeightForWidth())
        self.lb_TW_type.setSizePolicy(size_policy)
        self.lb_TW_type.setMinimumSize(self.l_size)
        self.lb_TW_connect = QLabel("Connection", self.gb_TW_select)
        self.lb_TW_connect.setToolTip("Thermowell Connection Type")
        size_policy.setHeightForWidth(self.lb_TW_connect.sizePolicy().hasHeightForWidth())
        self.lb_TW_connect.setSizePolicy(size_policy)
        self.lb_TW_connect.setMinimumSize(self.l_size)
        self.lb_TW_install = QLabel("Installation", self.gb_TW_select)
        self.lb_TW_install.setToolTip("Thermowell Installation Type")
        size_policy.setHeightForWidth(self.lb_TW_install.sizePolicy().hasHeightForWidth())
        self.lb_TW_install.setSizePolicy(size_policy)
        self.lb_TW_install.setMinimumSize(self.l_size)
        self.lb_design_press = QLabel("P<sub>Design</sub> (kPag)", self.gb_TW_select)
        self.lb_design_press.setToolTip("Design Pressure")
        size_policy.setHeightForWidth(self.lb_design_press.sizePolicy().hasHeightForWidth())
        self.lb_design_press.setSizePolicy(size_policy)
        self.lb_design_press.setMinimumSize(self.l_size)
        self.lb_design_temp = QLabel(self.gb_TW_select)
        self.lb_design_temp.setTextFormat(Qt.RichText)
        self.lb_design_temp.setText("T<sub>Design</sub> (&#x00B0; C)")
        self.lb_design_temp.setToolTip("Design Temperature")
        size_policy.setHeightForWidth(self.lb_design_temp.sizePolicy().hasHeightForWidth())
        self.lb_design_temp.setSizePolicy(size_policy)
        self.lb_design_temp.setMinimumSize(self.l_size)
        self.lb_TW_mat = QLabel("Shank Material", self.gb_TW_select)
        self.lb_TW_mat.setToolTip("Thermowell Shank Material")
        size_policy.setHeightForWidth(self.lb_TW_mat.sizePolicy().hasHeightForWidth())
        self.lb_TW_mat.setSizePolicy(size_policy)
        self.lb_TW_mat.setMinimumSize(self.l_size)
        self.lb_flange_mat = QLabel("Flange Material", self.gb_TW_select)
        self.lb_flange_mat.setToolTip("Thermowell Flange Material")
        size_policy.setHeightForWidth(self.lb_flange_mat.sizePolicy().hasHeightForWidth())
        self.lb_flange_mat.setSizePolicy(size_policy)
        self.lb_flange_mat.setMinimumSize(self.l_size)
        self.lb_flange_class = QLabel("Flange Class", self.gb_TW_select)
        self.lb_flange_class.setToolTip("Thermowell Flange Class")
        size_policy.setHeightForWidth(self.lb_flange_class.sizePolicy().hasHeightForWidth())
        self.lb_flange_class.setSizePolicy(size_policy)
        self.lb_flange_class.setMinimumSize(self.l_size)
        self.lb_NDT = QLabel("NDT Required", self.gb_TW_select)
        self.lb_NDT.setToolTip("Select if NDT is required")
        size_policy.setHeightForWidth(self.lb_NDT.sizePolicy().hasHeightForWidth())
        self.lb_NDT.setSizePolicy(size_policy)
        self.lb_NDT.setMinimumSize(self.l_size)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.cb_TW_type = QComboBox(self.gb_TW_select)
        size_policy.setHeightForWidth(self.cb_TW_type.sizePolicy().hasHeightForWidth())
        self.cb_TW_type.setSizePolicy(size_policy)
        self.cb_TW_type.setMinimumSize(self.cb_size)
        self.cb_TW_type.addItem("")
        self.cb_TW_type.addItem("Tapered")
        self.cb_TW_type.addItem("Straight")
        self.cb_TW_type.addItem("Step")
        self.cb_TW_connect = QComboBox(self.gb_TW_select)
        size_policy.setHeightForWidth(self.cb_TW_connect.sizePolicy().hasHeightForWidth())
        self.cb_TW_connect.setSizePolicy(size_policy)
        self.cb_TW_connect.setMinimumSize(self.cb_size)
        self.cb_TW_connect.addItem("")
        self.cb_TW_connect.addItem("Flanged")
        self.cb_TW_connect.addItem("Threaded")
        self.cb_TW_install = QComboBox(self.gb_TW_select)
        size_policy.setHeightForWidth(self.cb_TW_install.sizePolicy().hasHeightForWidth())
        self.cb_TW_install.setSizePolicy(size_policy)
        self.cb_TW_install.setMinimumSize(self.cb_size)
        self.cb_TW_install.addItem("")
        self.cb_TW_install.addItem("Straight")
        self.cb_TW_install.addItem("45" + u"\u00B0 " + "Angle")
        self.cb_TW_install.addItem("90" + u"\u00B0 " + "Elbow")
        self.cb_TW_mat = QComboBox(self.gb_TW_select)
        size_policy.setHeightForWidth(self.cb_TW_mat.sizePolicy().hasHeightForWidth())
        self.cb_TW_mat.setSizePolicy(size_policy)
        self.cb_TW_mat.setMinimumSize(self.cb_size)
        self.cb_flange_mat = QComboBox(self.gb_TW_select)
        size_policy.setHeightForWidth(self.cb_flange_mat.sizePolicy().hasHeightForWidth())
        self.cb_flange_mat.setSizePolicy(size_policy)
        self.cb_flange_mat.setMinimumSize(self.cb_size)
        self.cb_flange_class = QComboBox(self.gb_TW_select)
        size_policy.setHeightForWidth(self.cb_flange_class.sizePolicy().hasHeightForWidth())
        self.cb_flange_class.setSizePolicy(size_policy)
        self.cb_flange_class.setMinimumSize(self.cb_size)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.txt_design_press = QLineEdit("", self.gb_TW_select)
        size_policy.setHeightForWidth(self.txt_design_press.sizePolicy().hasHeightForWidth())
        self.txt_design_press.setSizePolicy(size_policy)
        self.txt_design_press.setMinimumSize(self.txt_size)
        self.txt_design_press.setMaxLength(self.txt_max_length)
        self.txt_design_press.setValidator(self.dbl_validator)
        self.txt_design_temp = QLineEdit("", self.gb_TW_select)
        size_policy.setHeightForWidth(self.txt_design_temp.sizePolicy().hasHeightForWidth())
        self.txt_design_temp.setSizePolicy(size_policy)
        self.txt_design_temp.setMinimumSize(self.txt_size)
        self.txt_design_temp.setMaxLength(3)
        self.txt_design_temp.setValidator(self.dbl_validator)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.chk_NDT = QCheckBox("Yes", self.gb_TW_select)
        self.chk_NDT.setSizePolicy(size_policy)
        self.chk_NDT.setMinimumSize(self.chk_size)

        self.gb_TW_type_layout = QGridLayout(self.gb_TW_select)
        self.gb_TW_type_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gb_TW_type_layout.addWidget(self.lb_TW_type, 0, 0)
        self.gb_TW_type_layout.addWidget(self.cb_TW_type, 0, 1)
        self.gb_TW_type_layout.addWidget(self.lb_TW_connect, 1, 0)
        self.gb_TW_type_layout.addWidget(self.cb_TW_connect, 1, 1)
        self.gb_TW_type_layout.addWidget(self.lb_TW_install, 2, 0)
        self.gb_TW_type_layout.addWidget(self.cb_TW_install, 2, 1)
        self.gb_TW_type_layout.addWidget(self.lb_design_press, 3, 0)
        self.gb_TW_type_layout.addWidget(self.txt_design_press, 3, 1)
        self.gb_TW_type_layout.addWidget(self.lb_design_temp, 4, 0)
        self.gb_TW_type_layout.addWidget(self.txt_design_temp, 4, 1)
        self.gb_TW_type_layout.addWidget(self.lb_TW_mat, 5, 0)
        self.gb_TW_type_layout.addWidget(self.cb_TW_mat, 5, 1)
        self.gb_TW_type_layout.addWidget(self.lb_flange_mat, 6, 0)
        self.gb_TW_type_layout.addWidget(self.cb_flange_mat, 6, 1)
        self.gb_TW_type_layout.addWidget(self.lb_flange_class, 7, 0)
        self.gb_TW_type_layout.addWidget(self.cb_flange_class, 7, 1)
        self.gb_TW_type_layout.addWidget(self.lb_NDT, 8, 0)
        self.gb_TW_type_layout.addWidget(self.chk_NDT, 8, 1)
        self.gb_TW_type_layout.addItem(self.h_spacer, 0, 3, 9, 1)
        self.gb_TW_type_layout.addItem(self.v_spacer, 9, 0)
        
        self.gb_img = QGroupBox("Thermowell Image", self.main_widget)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_img = QLabel("Click on image to enlarge", self.gb_img)
        size_policy.setHeightForWidth(self.lb_img.sizePolicy().hasHeightForWidth())
        self.lb_img.setSizePolicy(size_policy)
        self.lb_img.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        self.lb_img.hide()

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)

        self.img_TW_type = ImgLabel(self.gb_img)
        self.img_TW_type.setToolTip("Click on image to enlarge")
        size_policy.setHeightForWidth(self.img_TW_type.sizePolicy().hasHeightForWidth())
        self.img_TW_type.setSizePolicy(size_policy)
        self.img_TW_type.setMinimumSize(self.fr_size)
        self.img_TW_type.setFrameStyle(QFrame.Panel)
        self.img_TW_type.setLineWidth(1)

        self.gb_img_layout = QGridLayout(self.gb_img)
        self.gb_img_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gb_img_layout.addWidget(self.img_TW_type,  0,  0)
        self.gb_img_layout.addWidget(self.lb_img,  1,  0)
        self.gb_img_layout.addItem(self.h_spacer, 0, 1, 2, 1)
        self.gb_img_layout.addItem(self.v_spacer, 2, 0)

        self.gb_TW_mat = QGroupBox("Thermowell Data", self.main_widget)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_TW_Eref = QLabel(self.gb_TW_mat)
        self.lb_TW_Eref.setTextFormat(Qt.RichText)
        self.lb_TW_Eref.setText("E<sub>ref</sub>")
        self.lb_TW_Eref.setToolTip("Modulus of Elasticity at Normal Temperature")
        size_policy.setHeightForWidth(self.lb_TW_Eref.sizePolicy().hasHeightForWidth())
        self.lb_TW_Eref.setSizePolicy(size_policy)
        self.lb_TW_Eref.setMinimumSize(self.l_size_short)
        self.lb_TW_Eref_unit = QLabel("GPa", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.lb_TW_Eref_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_Eref_unit.setSizePolicy(size_policy)
        self.lb_TW_Eref_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_Et = QLabel(self.gb_TW_mat)
        self.lb_TW_Et.setTextFormat(Qt.RichText)
        self.lb_TW_Et.setText("E<sub>t</sub>")
        self.lb_TW_Et.setToolTip("Modulus of Elasticity at Service Temperature")
        size_policy.setHeightForWidth(self.lb_TW_Et.sizePolicy().hasHeightForWidth())
        self.lb_TW_Et.setSizePolicy(size_policy)
        self.lb_TW_Et.setMinimumSize(self.l_size_short)
        self.lb_TW_Et_unit = QLabel("GPa", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.lb_TW_Et_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_Et_unit.setSizePolicy(size_policy)
        self.lb_TW_Et_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_dens = QLabel(self.gb_TW_mat)
        self.lb_TW_dens.setTextFormat(Qt.RichText)
        self.lb_TW_dens.setText("&#x3C1;<sub>m</sub>")
        self.lb_TW_dens.setToolTip("Thermowell Density")
        size_policy.setHeightForWidth(self.lb_TW_dens.sizePolicy().hasHeightForWidth())
        self.lb_TW_dens.setSizePolicy(size_policy)
        self.lb_TW_dens.setMinimumSize(self.l_size_short)
        self.lb_TW_dens_unit = QLabel("kg/m<sup>3</sup>", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.lb_TW_dens_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_dens_unit.setSizePolicy(size_policy)
        self.lb_TW_dens_unit.setMinimumSize(self.l_size_short)
        self.lb_sensor_dens = QLabel(self.gb_TW_mat)
        self.lb_sensor_dens.setTextFormat(Qt.RichText)
        self.lb_sensor_dens.setText("&#x3C1;<sub>s</sub>")
        self.lb_sensor_dens.setToolTip("Sensor Density")
        size_policy.setHeightForWidth(self.lb_sensor_dens.sizePolicy().hasHeightForWidth())
        self.lb_sensor_dens.setSizePolicy(size_policy)
        self.lb_sensor_dens.setMinimumSize(self.l_size_short)
        self.lb_sensor_dens_unit = QLabel("kg/m<sup>3</sup>", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.lb_sensor_dens_unit.sizePolicy().hasHeightForWidth())
        self.lb_sensor_dens_unit.setSizePolicy(size_policy)
        self.lb_sensor_dens_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_stress_limit = QLabel("S", self.gb_TW_mat)
        self.lb_TW_stress_limit.setToolTip("Allowable Stress Limit")
        size_policy.setHeightForWidth(self.lb_TW_stress_limit.sizePolicy().hasHeightForWidth())
        self.lb_TW_stress_limit.setSizePolicy(size_policy)
        self.lb_TW_stress_limit.setMinimumSize(self.l_size_short)
        self.lb_TW_stress_limit_unit = QLabel("MPag", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.lb_TW_stress_limit_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_stress_limit_unit.setSizePolicy(size_policy)
        self.lb_TW_stress_limit_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_fatigue_limit = QLabel(self.gb_TW_mat)
        self.lb_TW_fatigue_limit.setTextFormat(Qt.RichText)
        self.lb_TW_fatigue_limit.setText("S<sub>f</sub>")
        self.lb_TW_fatigue_limit.setToolTip("Fatigue Endurance Limit")
        size_policy.setHeightForWidth(self.lb_TW_fatigue_limit.sizePolicy().hasHeightForWidth())
        self.lb_TW_fatigue_limit.setSizePolicy(size_policy)
        self.lb_TW_fatigue_limit.setMinimumSize(self.l_size_short)
        self.lb_TW_fatigue_limit_unit = QLabel("MPag", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.lb_TW_fatigue_limit_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_fatigue_limit_unit.setSizePolicy(size_policy)
        self.lb_TW_fatigue_limit_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_press_limit = QLabel("P<sub>f</sub>", self.gb_TW_mat)
        self.lb_TW_press_limit.setToolTip("Flange Pressure Limit")
        size_policy.setHeightForWidth(self.lb_TW_press_limit.sizePolicy().hasHeightForWidth())
        self.lb_TW_press_limit.setSizePolicy(size_policy)
        self.lb_TW_press_limit.setMinimumSize(self.l_size_short)
        self.lb_TW_press_limit_unit = QLabel("kPag", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.lb_TW_press_limit_unit.sizePolicy().hasHeightForWidth())
        self.lb_TW_press_limit_unit.setSizePolicy(size_policy)
        self.lb_TW_press_limit_unit.setMinimumSize(self.l_size_short)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.txt_TW_Eref = QLineEdit("", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.txt_TW_Eref.sizePolicy().hasHeightForWidth())
        self.txt_TW_Eref.setSizePolicy(size_policy)
        self.txt_TW_Eref.setMinimumSize(self.txt_size)
        self.txt_TW_Eref.setMaxLength(self.txt_max_length)
        self.txt_TW_Eref.setValidator(self.dbl_validator)
        self.txt_TW_Et = QLineEdit("", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.txt_TW_Et.sizePolicy().hasHeightForWidth())
        self.txt_TW_Et.setSizePolicy(size_policy)
        self.txt_TW_Et.setMinimumSize(self.txt_size)
        self.txt_TW_Et.setMaxLength(self.txt_max_length)
        self.txt_TW_Et.setValidator(self.dbl_validator)
        self.txt_TW_dens = QLineEdit("", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.txt_TW_dens.sizePolicy().hasHeightForWidth())
        self.txt_TW_dens.setSizePolicy(size_policy)
        self.txt_TW_dens.setMinimumSize(self.txt_size)
        self.txt_TW_dens.setMaxLength(self.txt_max_length)
        self.txt_TW_dens.setValidator(self.dbl_validator)
        self.txt_sensor_dens = QLineEdit("2700", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.txt_sensor_dens.sizePolicy().hasHeightForWidth())
        self.txt_sensor_dens.setSizePolicy(size_policy)
        self.txt_sensor_dens.setMinimumSize(self.txt_size)
        self.txt_sensor_dens.setMaxLength(self.txt_max_length)
        self.txt_sensor_dens.setValidator(self.dbl_validator)
        self.txt_TW_stress_limit = QLineEdit("", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.txt_TW_stress_limit.sizePolicy().hasHeightForWidth())
        self.txt_TW_stress_limit.setSizePolicy(size_policy)
        self.txt_TW_stress_limit.setMinimumSize(self.txt_size)
        self.txt_TW_stress_limit.setMaxLength(self.txt_max_length)
        self.txt_TW_stress_limit.setValidator(self.dbl_validator)
        self.txt_TW_fatigue_limit = QLineEdit("", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.txt_TW_fatigue_limit.sizePolicy().hasHeightForWidth())
        self.txt_TW_fatigue_limit.setSizePolicy(size_policy)
        self.txt_TW_fatigue_limit.setMinimumSize(self.txt_size)
        self.txt_TW_fatigue_limit.setMaxLength(self.txt_max_length)
        self.txt_TW_fatigue_limit.setValidator(self.dbl_validator)
        self.txt_TW_press_limit = QLineEdit("", self.gb_TW_mat)
        size_policy.setHeightForWidth(self.txt_TW_press_limit.sizePolicy().hasHeightForWidth())
        self.txt_TW_press_limit.setSizePolicy(size_policy)
        self.txt_TW_press_limit.setMinimumSize(self.txt_size)
        self.txt_TW_press_limit.setMaxLength(self.txt_max_length)
        self.txt_TW_press_limit.setValidator(self.dbl_validator)

        self.gb_TW_mat_layout = QGridLayout(self.gb_TW_mat)
        self.gb_TW_mat_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gb_TW_mat_layout.addWidget(self.lb_TW_Eref, 0, 0)
        self.gb_TW_mat_layout.addWidget(self.txt_TW_Eref, 0, 1)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_Eref_unit, 0, 2)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_Et, 1, 0)
        self.gb_TW_mat_layout.addWidget(self.txt_TW_Et, 1, 1)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_Et_unit, 1, 2)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_dens, 2, 0)
        self.gb_TW_mat_layout.addWidget(self.txt_TW_dens, 2, 1)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_dens_unit, 2, 2)
        self.gb_TW_mat_layout.addWidget(self.lb_sensor_dens, 3, 0)
        self.gb_TW_mat_layout.addWidget(self.txt_sensor_dens, 3, 1)
        self.gb_TW_mat_layout.addWidget(self.lb_sensor_dens_unit, 3, 2)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_stress_limit, 4, 0)
        self.gb_TW_mat_layout.addWidget(self.txt_TW_stress_limit, 4, 1)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_stress_limit_unit, 4, 2)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_fatigue_limit, 5, 0)
        self.gb_TW_mat_layout.addWidget(self.txt_TW_fatigue_limit, 5, 1)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_fatigue_limit_unit, 5, 2)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_press_limit, 6, 0)
        self.gb_TW_mat_layout.addWidget(self.txt_TW_press_limit, 6, 1)
        self.gb_TW_mat_layout.addWidget(self.lb_TW_press_limit_unit, 6, 2)
        self.gb_TW_mat_layout.addItem(self.h_spacer, 0, 3, 7, 1)
        self.gb_TW_mat_layout.addItem(self.v_spacer, 7, 0)

        self.gb_pipe_data = QGroupBox("Pipe Data", self.main_widget)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_pipe_mat = QLabel("Material", self.gb_pipe_data)
        size_policy.setHeightForWidth(self.lb_pipe_mat.sizePolicy().hasHeightForWidth())
        self.lb_pipe_mat.setSizePolicy(size_policy)
        self.lb_pipe_mat.setMinimumSize(self.l_size)
        self.lb_pipe_DN = QLabel("DN (mm)", self.gb_pipe_data)
        self.lb_pipe_DN.setToolTip("Pipe Nominal Diameter")
        size_policy.setHeightForWidth(self.lb_pipe_DN.sizePolicy().hasHeightForWidth())
        self.lb_pipe_DN.setSizePolicy(size_policy)
        self.lb_pipe_DN.setMinimumSize(self.l_size_short)
        self.lb_pipe_sch = QLabel("Schedule", self.gb_pipe_data)
        size_policy.setHeightForWidth(self.lb_pipe_sch.sizePolicy().hasHeightForWidth())
        self.lb_pipe_sch.setSizePolicy(size_policy)
        self.lb_pipe_sch.setMinimumSize(self.l_size)
        self.lb_pipe_OD = QLabel(self.gb_pipe_data)
        self.lb_pipe_OD.setTextFormat(Qt.RichText)
        self.lb_pipe_OD.setText("D<sub>o</sub>")
        self.lb_pipe_OD.setToolTip("Pipe Outside Diameter")
        size_policy.setHeightForWidth(self.lb_pipe_OD.sizePolicy().hasHeightForWidth())
        self.lb_pipe_OD.setSizePolicy(size_policy)
        self.lb_pipe_OD.setMinimumSize(self.l_size_short)
        self.lb_pipe_OD_value = QLabel("", self.gb_pipe_data)
        size_policy.setHeightForWidth(self.lb_pipe_OD_value.sizePolicy().hasHeightForWidth())
        self.lb_pipe_OD_value.setSizePolicy(size_policy)
        self.lb_pipe_OD_value.setMinimumSize(self.l_size)
        self.lb_pipe_OD_value.setFrameStyle(QFrame.Panel)
        self.lb_pipe_OD_value.setLineWidth(1)
        self.lb_pipe_OD_unit = QLabel("mm", self.gb_pipe_data)
        size_policy.setHeightForWidth(self.lb_pipe_OD_unit.sizePolicy().hasHeightForWidth())
        self.lb_pipe_OD_unit.setSizePolicy(size_policy)
        self.lb_pipe_OD_unit.setMinimumSize(self.l_size_short)
        self.lb_pipe_WT = QLabel(self.gb_pipe_data)
        self.lb_pipe_WT.setTextFormat(Qt.RichText)
        self.lb_pipe_WT.setText("t<sub>p</sub>")
        self.lb_pipe_WT.setToolTip("Pipe Wall Thickness")
        size_policy.setHeightForWidth(self.lb_pipe_WT.sizePolicy().hasHeightForWidth())
        self.lb_pipe_WT.setSizePolicy(size_policy)
        self.lb_pipe_WT.setMinimumSize(self.l_size_short)
        self.lb_pipe_WT_value = QLabel("", self.gb_pipe_data)
        size_policy.setHeightForWidth(self.lb_pipe_WT_value.sizePolicy().hasHeightForWidth())
        self.lb_pipe_WT_value.setSizePolicy(size_policy)
        self.lb_pipe_WT_value.setMinimumSize(self.l_size)
        self.lb_pipe_WT_value.setFrameStyle(QFrame.Panel)
        self.lb_pipe_WT_value.setLineWidth(1)
        self.lb_pipe_WT_unit = QLabel("mm", self.gb_pipe_data)
        size_policy.setHeightForWidth(self.lb_pipe_WT_unit.sizePolicy().hasHeightForWidth())
        self.lb_pipe_WT_unit.setSizePolicy(size_policy)
        self.lb_pipe_WT_unit.setMinimumSize(self.l_size_short)
        self.lb_pipe_ID = QLabel(self.gb_pipe_data)
        self.lb_pipe_ID.setTextFormat(Qt.RichText)
        self.lb_pipe_ID.setText("D<sub>i</sub>")
        self.lb_pipe_ID.setToolTip("Pipe Inside Diameter")
        size_policy.setHeightForWidth(self.lb_pipe_ID.sizePolicy().hasHeightForWidth())
        self.lb_pipe_ID.setSizePolicy(size_policy)
        self.lb_pipe_ID.setMinimumSize(self.l_size_short)
        self.lb_pipe_ID_unit = QLabel("mm", self.gb_pipe_data)
        size_policy.setHeightForWidth(self.lb_pipe_ID_unit.sizePolicy().hasHeightForWidth())
        self.lb_pipe_ID_unit.setSizePolicy(size_policy)
        self.lb_pipe_ID_unit.setMinimumSize(self.l_size_short)
        self.lb_TW_stress = QLabel("Corrosion", self.gb_pipe_data)
        self.lb_TW_stress.setToolTip("Select if termowell is subject to corrosion or embrittlement")
        size_policy.setHeightForWidth(self.lb_TW_stress.sizePolicy().hasHeightForWidth())
        self.lb_TW_stress.setSizePolicy(size_policy)
        self.lb_TW_stress.setMinimumSize(self.l_size)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.cb_pipe_mat = QComboBox(self.gb_pipe_data)
        size_policy.setHeightForWidth(self.cb_pipe_mat.sizePolicy().hasHeightForWidth())
        self.cb_pipe_mat.setSizePolicy(size_policy)
        self.cb_pipe_mat.setMinimumSize(self.cb_size)
        self.cb_pipe_DN = QComboBox(self.gb_pipe_data)
        size_policy.setHeightForWidth(self.cb_pipe_DN.sizePolicy().hasHeightForWidth())
        self.cb_pipe_DN.setSizePolicy(size_policy)
        self.cb_pipe_DN.setMinimumSize(self.cb_size)
        self.cb_pipe_sch = QComboBox(self.gb_pipe_data)
        size_policy.setHeightForWidth(self.cb_pipe_sch.sizePolicy().hasHeightForWidth())
        self.cb_pipe_sch.setSizePolicy(size_policy)
        self.cb_pipe_sch.setMinimumSize(self.cb_size)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.txt_pipe_ID = QLineEdit("", self.gb_pipe_data)
        size_policy.setHeightForWidth(self.txt_pipe_ID.sizePolicy().hasHeightForWidth())
        self.txt_pipe_ID.setSizePolicy(size_policy)
        self.txt_pipe_ID.setMinimumSize(self.txt_size)
        self.txt_pipe_ID.setMaxLength(self.txt_max_length)
        self.txt_pipe_ID.setValidator(self.dbl_validator)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.chk_TW_stress = QCheckBox("Yes", self.gb_pipe_data)
        self.chk_TW_stress.setSizePolicy(size_policy)
        self.chk_TW_stress.setMinimumSize(self.chk_size)

        self.gb_pipe_data_layout = QGridLayout(self.gb_pipe_data)
        self.gb_pipe_data_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_mat, 0, 0)
        self.gb_pipe_data_layout.addWidget(self.cb_pipe_mat, 0, 1)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 0, 2)
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_DN, 1, 0)
        self.gb_pipe_data_layout.addWidget(self.cb_pipe_DN, 1, 1)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 1, 2)
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_sch, 2, 0)
        self.gb_pipe_data_layout.addWidget(self.cb_pipe_sch, 2, 1)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 2, 2)
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_OD, 3, 0)
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_OD_value, 3, 1)
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_OD_unit, 3, 2)
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_WT, 4, 0)
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_WT_value, 4, 1)
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_WT_unit, 4, 2)
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_ID, 5, 0)
        self.gb_pipe_data_layout.addWidget(self.txt_pipe_ID, 5, 1)
        self.gb_pipe_data_layout.addWidget(self.lb_pipe_ID_unit, 5, 2)
        self.gb_pipe_data_layout.addWidget(self.lb_TW_stress, 6, 0)
        self.gb_pipe_data_layout.addWidget(self.chk_TW_stress, 6, 1)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 6, 2)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 0, 3, 7, 1)
        self.gb_pipe_data_layout.addItem(self.v_spacer, 7, 0)

        self.gb_proc_data = QGroupBox("Process Data", self.main_widget)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_fluid_press = QLabel("P", self.gb_proc_data)
        self.lb_fluid_press.setToolTip("Fluid Pressure")
        size_policy.setHeightForWidth(self.lb_fluid_press.sizePolicy().hasHeightForWidth())
        self.lb_fluid_press.setSizePolicy(size_policy)
        self.lb_fluid_press.setMinimumSize(self.l_size_short)
        self.lb_fluid_press_unit = QLabel("kPag", self.gb_proc_data)
        size_policy.setHeightForWidth(self.lb_fluid_press_unit.sizePolicy().hasHeightForWidth())
        self.lb_fluid_press_unit.setSizePolicy(size_policy)
        self.lb_fluid_press_unit.setMinimumSize(self.l_size_short)
        self.lb_fluid_temp = QLabel("T", self.gb_proc_data)
        self.lb_fluid_temp.setToolTip("Fluid Temperature")
        size_policy.setHeightForWidth(self.lb_fluid_temp.sizePolicy().hasHeightForWidth())
        self.lb_fluid_temp.setSizePolicy(size_policy)
        self.lb_fluid_temp.setMinimumSize(self.l_size_short)
        self.lb_fluid_temp_unit = QLabel(self.gb_proc_data)
        self.lb_fluid_temp_unit.setTextFormat(Qt.RichText)
        self.lb_fluid_temp_unit.setText("&#x00B0; C")
        size_policy.setHeightForWidth(self.lb_fluid_temp_unit.sizePolicy().hasHeightForWidth())
        self.lb_fluid_temp_unit.setSizePolicy(size_policy)
        self.lb_fluid_temp_unit.setMinimumSize(self.l_size_short)
        self.lb_fluid_rate = QLabel("F", self.gb_proc_data)
        self.lb_fluid_rate.setToolTip("Fluid Mass Flow Rate")
        size_policy.setHeightForWidth(self.lb_fluid_rate.sizePolicy().hasHeightForWidth())
        self.lb_fluid_rate.setSizePolicy(size_policy)
        self.lb_fluid_rate.setMinimumSize(self.l_size_short)
        self.lb_fluid_rate_unit = QLabel("kg/h", self.gb_proc_data)
        size_policy.setHeightForWidth(self.lb_fluid_rate_unit.sizePolicy().hasHeightForWidth())
        self.lb_fluid_rate_unit.setSizePolicy(size_policy)
        self.lb_fluid_rate_unit.setMinimumSize(self.l_size_short)
        self.lb_fluid_dens = QLabel(self.gb_proc_data)
        self.lb_fluid_dens.setTextFormat(Qt.RichText)
        self.lb_fluid_dens.setText("&#x3C1;")
        self.lb_fluid_dens.setToolTip("Fluid Density")
        size_policy.setHeightForWidth(self.lb_fluid_dens.sizePolicy().hasHeightForWidth())
        self.lb_fluid_dens.setSizePolicy(size_policy)
        self.lb_fluid_dens.setMinimumSize(self.l_size_short)
        self.lb_fluid_dens_unit = QLabel(self.gb_proc_data)
        self.lb_fluid_dens_unit.setTextFormat(Qt.RichText)
        self.lb_fluid_dens_unit.setText("kg/m<sup>3</sup>")
        size_policy.setHeightForWidth(self.lb_fluid_dens_unit.sizePolicy().hasHeightForWidth())
        self.lb_fluid_dens_unit.setSizePolicy(size_policy)
        self.lb_fluid_dens_unit.setMinimumSize(self.l_size_short)
        self.lb_fluid_spec_vol = QLabel("v", self.gb_proc_data)
        self.lb_fluid_spec_vol.setToolTip("Fluid Specific Volume")
        size_policy.setHeightForWidth(self.lb_fluid_spec_vol.sizePolicy().hasHeightForWidth())
        self.lb_fluid_spec_vol.setSizePolicy(size_policy)
        self.lb_fluid_spec_vol.setMinimumSize(self.l_size_short)
        self.lb_fluid_spec_vol_unit = QLabel(self.gb_proc_data)
        self.lb_fluid_spec_vol_unit.setTextFormat(Qt.RichText)
        self.lb_fluid_spec_vol_unit.setText("m<sup>3</sup>/kg")
        size_policy.setHeightForWidth(self.lb_fluid_spec_vol_unit.sizePolicy().hasHeightForWidth())
        self.lb_fluid_spec_vol_unit.setSizePolicy(size_policy)
        self.lb_fluid_spec_vol_unit.setMinimumSize(self.l_size_short)
        self.lb_fluid_velos = QLabel("V", self.gb_proc_data)
        self.lb_fluid_velos.setToolTip("Fluid Velocity")
        size_policy.setHeightForWidth(self.lb_fluid_velos.sizePolicy().hasHeightForWidth())
        self.lb_fluid_velos.setSizePolicy(size_policy)
        self.lb_fluid_velos.setMinimumSize(self.l_size_short)
        self.lb_fluid_velos_unit = QLabel("m/s", self.gb_proc_data)
        size_policy.setHeightForWidth(self.lb_fluid_velos_unit.sizePolicy().hasHeightForWidth())
        self.lb_fluid_velos_unit.setSizePolicy(size_policy)
        self.lb_fluid_velos_unit.setMinimumSize(self.l_size_short)
        self.lb_fluid_viscos = QLabel(self.gb_proc_data)
        self.lb_fluid_viscos.setTextFormat(Qt.RichText)
        self.lb_fluid_viscos.setText("&#x3BC;")
        self.lb_fluid_viscos.setToolTip("Fluid Dynamic Viscosity")
        size_policy.setHeightForWidth(self.lb_fluid_viscos.sizePolicy().hasHeightForWidth())
        self.lb_fluid_viscos.setSizePolicy(size_policy)
        self.lb_fluid_viscos.setMinimumSize(self.l_size_short)
        self.lb_fluid_viscos_unit = QLabel("cP", self.gb_proc_data)
        size_policy.setHeightForWidth(self.lb_fluid_viscos_unit.sizePolicy().hasHeightForWidth())
        self.lb_fluid_viscos_unit.setSizePolicy(size_policy)
        self.lb_fluid_viscos_unit.setMinimumSize(self.l_size_short)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.txt_fluid_press = QLineEdit("", self.gb_proc_data)
        size_policy.setHeightForWidth(self.txt_fluid_press.sizePolicy().hasHeightForWidth())
        self.txt_fluid_press.setSizePolicy(size_policy)
        self.txt_fluid_press.setMinimumSize(self.txt_size)
        self.txt_fluid_press.setMaxLength(self.txt_max_length)
        self.txt_fluid_press.setValidator(self.dbl_validator)
        self.txt_fluid_temp = QLineEdit("", self.gb_proc_data)
        size_policy.setHeightForWidth(self.txt_fluid_temp.sizePolicy().hasHeightForWidth())
        self.txt_fluid_temp.setSizePolicy(size_policy)
        self.txt_fluid_temp.setMinimumSize(self.txt_size)
        self.txt_fluid_temp.setMaxLength(3)
        self.txt_fluid_temp.setValidator(self.dbl_validator)
        self.txt_fluid_rate = QLineEdit("", self.gb_proc_data)
        size_policy.setHeightForWidth(self.txt_fluid_rate.sizePolicy().hasHeightForWidth())
        self.txt_fluid_rate.setSizePolicy(size_policy)
        self.txt_fluid_rate.setMinimumSize(self.txt_size)
        self.txt_fluid_rate.setMaxLength(self.txt_max_length)
        self.txt_fluid_rate.setValidator(self.dbl_validator)
        self.txt_fluid_dens = QLineEdit("", self.gb_proc_data)
        size_policy.setHeightForWidth(self.txt_fluid_dens.sizePolicy().hasHeightForWidth())
        self.txt_fluid_dens.setSizePolicy(size_policy)
        self.txt_fluid_dens.setMinimumSize(self.txt_size)
        self.txt_fluid_dens.setMaxLength(self.txt_max_length)
        self.txt_fluid_dens.setValidator(self.dbl_validator)
        self.txt_fluid_spec_vol = QLineEdit("", self.gb_proc_data)
        size_policy.setHeightForWidth(self.txt_fluid_spec_vol.sizePolicy().hasHeightForWidth())
        self.txt_fluid_spec_vol.setSizePolicy(size_policy)
        self.txt_fluid_spec_vol.setMinimumSize(self.txt_size)
        self.txt_fluid_spec_vol.setMaxLength(self.txt_max_length)
        self.txt_fluid_spec_vol.setValidator(self.dbl_validator)
        self.txt_fluid_velos = QLineEdit("", self.gb_proc_data)
        size_policy.setHeightForWidth(self.txt_fluid_velos.sizePolicy().hasHeightForWidth())
        self.txt_fluid_velos.setSizePolicy(size_policy)
        self.txt_fluid_velos.setMinimumSize(self.txt_size)
        self.txt_fluid_velos.setMaxLength(self.txt_max_length)
        self.txt_fluid_velos.setValidator(self.dbl_validator)
        self.txt_fluid_viscos = QLineEdit("", self.gb_proc_data)
        size_policy.setHeightForWidth(self.txt_fluid_viscos.sizePolicy().hasHeightForWidth())
        self.txt_fluid_viscos.setSizePolicy(size_policy)
        self.txt_fluid_viscos.setMinimumSize(self.txt_size)
        self.txt_fluid_viscos.setMaxLength(self.txt_max_length)
        self.txt_fluid_viscos.setValidator(self.dbl_validator)

        self.gb_proc_data_layout = QGridLayout(self.gb_proc_data)
        self.gb_proc_data_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gb_proc_data_layout.addWidget(self.lb_fluid_press, 0, 0)
        self.gb_proc_data_layout.addWidget(self.txt_fluid_press, 0, 1)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_press_unit, 0, 2)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_temp, 1, 0)
        self.gb_proc_data_layout.addWidget(self.txt_fluid_temp, 1, 1)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_temp_unit, 1, 2)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_rate, 2, 0)
        self.gb_proc_data_layout.addWidget(self.txt_fluid_rate, 2, 1)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_rate_unit, 2, 2)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_dens, 3, 0)
        self.gb_proc_data_layout.addWidget(self.txt_fluid_dens, 3, 1)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_dens_unit, 3, 2)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_spec_vol, 4, 0)
        self.gb_proc_data_layout.addWidget(self.txt_fluid_spec_vol, 4, 1)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_spec_vol_unit, 4, 2)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_velos, 5, 0)
        self.gb_proc_data_layout.addWidget(self.txt_fluid_velos, 5, 1)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_velos_unit, 5, 2)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_viscos, 6, 0)
        self.gb_proc_data_layout.addWidget(self.txt_fluid_viscos, 6, 1)
        self.gb_proc_data_layout.addWidget(self.lb_fluid_viscos_unit, 6, 2)
        self.gb_proc_data_layout.addItem(self.h_spacer, 0, 4, 7, 1)
        self.gb_proc_data_layout.addItem(self.v_spacer, 7, 0)

        self.page_layout = QGridLayout(self.main_widget)
        self.page_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.page_layout.addWidget(self.gb_TW_dim, 0, 0)
        self.page_layout.addWidget(self.gb_TW_select, 0, 1)
        self.page_layout.addWidget(self.gb_img, 0, 2)
        self.page_layout.addWidget(self.gb_TW_mat, 1, 0)
        self.page_layout.addWidget(self.gb_pipe_data, 1, 1)
        self.page_layout.addWidget(self.gb_proc_data, 1, 2)

        self.cb_TW_mat.addItems(self.GetTWMaterials())
        self.cb_flange_mat.addItems(self.GetFlangeMaterials())
        self.cb_pipe_mat.addItems(self.GetPipeMaterials())
        
        self.timer = QTimer(self)
        self.timer.setInterval(5000)
        
        self.connect(self.timer, SIGNAL("timeout()") , self.TimerHit)
        self.connect(self.cb_TW_type, SIGNAL("currentIndexChanged(int)"), self.OnTWTypetIndexChanged)
        self.connect(self.cb_TW_connect, SIGNAL("currentIndexChanged(int)"), self.OnTWConnectIndexChanged)
        self.connect(self.cb_TW_mat, SIGNAL("currentIndexChanged(int)"), self.OnTWMatCurrentIndexChanged)
        self.connect(self.cb_flange_mat, SIGNAL("currentIndexChanged(int)"), self.OnFlangeMatCurrentIndexChanged)
        self.connect(self.cb_flange_class, SIGNAL("currentIndexChanged(int)"), self.OnFlangeClassCurrentIndexChanged)
        self.connect(self.cb_pipe_mat, SIGNAL("currentIndexChanged(int)"), self.OnPipeMatCurrentIndexChanged)
        self.connect(self.cb_pipe_DN, SIGNAL("currentIndexChanged(int)"), self.OnPipeDNCurrentIndexChanged)
        self.connect(self.cb_pipe_sch, SIGNAL("currentIndexChanged(int)"), self.OnPipeSchCurrentIndexChanged)
        self.connect(self.txt_design_temp, SIGNAL("textChanged(QString)"), self.OnDesignTempTextChanged)
        self.connect(self.txt_design_press, SIGNAL("textChanged(QString)"), self.OnDesignPressTextChanged)
        self.connect(self.txt_TW_tip_dia, SIGNAL("textChanged(QString)"), self.OnTWTipDiaTextChanged)
        self.connect(self.chk_NDT, SIGNAL("toggled(bool)"), self.OnNDTToggled)
        self.img_TW_type.mousePressedSignal.connect(self.OnImgTWTypeMousePressed)

    def OnNDTToggled(self):
        fatig = ""
        if self.chk_NDT.isChecked() and self.cb_TW_connect.currentText() == "Flanged":
            fatig = "62.8"
        else:
            fatig = "37.2"
        self.txt_TW_fatigue_limit.setText(fatig)

    def OnTWTypetIndexChanged(self):
        str_img = ""
        if self.cb_TW_type.currentText() == "Tapered":
            if self.cb_TW_connect.currentText() == "Flanged":
                str_img = "TW_flange_tapered.png"
                self.lb_img.show()
            elif self.cb_TW_connect.currentText() == "Threaded":
                str_img = "TW_thread_tapered.png"
                self.lb_img.show()
            self.img_TW_type.setStyleSheet(QString.fromUtf8("background-image: url(:/myresources/images/" + str_img + "); background-repeat:no-repeat; background-position: center;"))
            self.txt_TW_out_dia.setText("")
            self.txt_TW_out_dia.setEnabled(True)
            self.StepInputsHide()
        elif self.cb_TW_type.currentText() == "Straight":
            if self.cb_TW_connect.currentText() == "Flanged":
                str_img = "TW_flange_straight.png"
                self.lb_img.show()
            elif self.cb_TW_connect.currentText() == "Threaded":
                str_img = "TW_thread_straight.png"
                self.lb_img.show()
            self.img_TW_type.setStyleSheet(QString.fromUtf8("background-image: url(:/myresources/images/" + str_img + "); background-repeat:no-repeat; background-position: center;"))
            self.txt_TW_out_dia.setEnabled(False)
            self.StepInputsHide()
        elif self.cb_TW_type.currentText() == "Step":
            if self.cb_TW_connect.currentText() == "Flanged":
                str_img = "TW_flange_step.png"
                self.lb_img.show()
            elif self.cb_TW_connect.currentText() == "Threaded":
                str_img = "TW_thread_step.png"
                self.lb_img.show()
            self.img_TW_type.setStyleSheet(QString.fromUtf8("background-image: url(:/myresources/images/" + str_img + "); background-repeat:no-repeat; background-position: center;"))
            self.txt_TW_out_dia.setText("")
            self.txt_TW_out_dia.setEnabled(True)
            self.StepInputsShow()
        else:
            self.txt_TW_out_dia.setText("")
            self.txt_TW_out_dia.setEnabled(True)
            self.StepInputsHide()
            self.img_TW_type.setStyleSheet(QString.fromUtf8(""))
            self.lb_img.hide()
        
    def OnTWConnectIndexChanged(self):
        fatig = ""
        str_img = ""
        if self.cb_TW_connect.currentText() == "Flanged":
            if self.cb_TW_type.currentText() == "Tapered":
                str_img = "TW_flange_tapered.png"
                self.lb_img.show()
                self.txt_TW_out_dia.setText("")
                self.txt_TW_out_dia.setEnabled(True)
                self.StepInputsHide()
            elif self.cb_TW_type.currentText() == "Straight":
                str_img = "TW_flange_straight.png"
                self.lb_img.show()
                self.txt_TW_out_dia.setEnabled(False)
                self.StepInputsHide()
            elif self.cb_TW_type.currentText() == "Step":
                str_img = "TW_flange_step.png"
                self.lb_img.show()
                self.txt_TW_out_dia.setText("")
                self.txt_TW_out_dia.setEnabled(True)
                self.StepInputsShow()
            self.img_TW_type.setStyleSheet(QString.fromUtf8("background-image: url(:/myresources/images/" + str_img + "); background-repeat:no-repeat; background-position: center;"))
            self.cb_flange_mat.clear()
            self.cb_flange_mat.setEnabled(True)
            self.cb_flange_mat.addItems(self.GetFlangeMaterials())
            self.cb_flange_class.clear()
            self.cb_flange_class.setEnabled(True)
            self.cb_flange_class.addItems(self.GetFlangeMaterials())
            self.txt_TW_press_limit.setEnabled(True)
            if self.chk_NDT.isChecked() and self.cb_TW_connect.currentText() == "Flanged":
                fatig = "62.8"
            else:
                fatig = "37.2"
            self.txt_TW_fatigue_limit.setText(fatig)
            self.chk_NDT.setChecked(False)
            self.chk_NDT.setEnabled(True)
        elif self.cb_TW_connect.currentText() == "Threaded":
            if self.cb_TW_type.currentText() == "Tapered":
                str_img = "TW_thread_tapered.png"
                self.lb_img.show()
                self.txt_TW_out_dia.setText("")
                self.txt_TW_out_dia.setEnabled(True)
                self.StepInputsHide()
            elif self.cb_TW_type.currentText() == "Straight":
                str_img = "TW_thread_straight.png"
                self.lb_img.show()
                self.txt_TW_out_dia.setEnabled(False)
                self.StepInputsHide()
            elif self.cb_TW_type.currentText() == "Step":
                str_img = "TW_thread_step.png"
                self.lb_img.show()
                self.txt_TW_out_dia.setText("")
                self.txt_TW_out_dia.setEnabled(True)
                self.StepInputsShow()
            self.img_TW_type.setStyleSheet(QString.fromUtf8("background-image: url(:/myresources/images/" + str_img + "); background-repeat:no-repeat; background-position: center;"))
            self.cb_flange_mat.clear()
            self.cb_flange_mat.setEnabled(False)
            self.cb_flange_class.clear()
            self.cb_flange_class.setEnabled(False)
            self.txt_TW_press_limit.setText("")
            self.txt_TW_press_limit.setEnabled(False)
            fatig = "37.2"
            self.txt_TW_fatigue_limit.setText(fatig)
            self.chk_NDT.setChecked(False)
            self.chk_NDT.setEnabled(False)
        else:
            self.txt_TW_out_dia.setText("")
            self.txt_TW_out_dia.setEnabled(True)
            self.StepInputsHide()
            self.img_TW_type.setStyleSheet(QString.fromUtf8(""))
            self.lb_img.hide()
            self.cb_flange_mat.clear()
            self.cb_flange_mat.setEnabled(False)
            self.cb_flange_class.clear()
            self.cb_flange_class.setEnabled(False)
            self.txt_TW_press_limit.setEnabled(True)
            fatig = "37.2"
            self.txt_TW_fatigue_limit.setText(fatig)
            self.chk_NDT.setChecked(False)
            self.chk_NDT.setEnabled(False)
        
    def StepInputsHide(self):
        self.txt_TW_tip_dia.setText("")
        self.txt_TW_tip_dia.show()
        self.cb_TW_step_tip_dia.setCurrentIndex(0)
        self.cb_TW_step_tip_dia.hide()
        
        self.lb_TW_step_fillet_rad.hide()
        self.txt_TW_step_fillet_rad.setText("")
        self.txt_TW_step_fillet_rad.hide()
        self.lb_TW_step_fillet_rad_unit.hide()
        
        self.lb_TW_step_length.hide()
        self.txt_TW_step_length.setText("")
        self.txt_TW_step_length.hide()
        self.lb_TW_step_length_unit.hide()
        
    def StepInputsShow(self):
        self.txt_TW_tip_dia.setText("")
        self.txt_TW_tip_dia.hide()
        self.cb_TW_step_tip_dia.setCurrentIndex(0)
        self.cb_TW_step_tip_dia.show()
        
        self.lb_TW_step_fillet_rad.show()
        self.txt_TW_step_fillet_rad.setText("0")
        self.txt_TW_step_fillet_rad.show()
        self.lb_TW_step_fillet_rad_unit.show()
        
        self.lb_TW_step_length.show()
        self.txt_TW_step_length.setText("")
        self.txt_TW_step_length.show()
        self.lb_TW_step_length_unit.show()

    def GetTWMaterials(self):
        self.materials_xml = self.tree.parse(self.materialdata_file)
        self.materials = self.materials_xml.getiterator("material")
        matlist = QStringList("")
        for material in self.materials:
            matlist.append(material.attrib["name"])
        return matlist

    def OnTWMatCurrentIndexChanged(self):
        self.ClearTWSelectInputs()
        material = self.cb_TW_mat.currentText()
        temp = self.txt_design_temp.text()
        if material != "":
            Eref = self.GetMaterialData(material,  "25",  "elast")
            if Eref and Eref != "":
                Eref = str(float(Eref) / 1000000000)
                self.txt_TW_Eref.setText(Eref)
            else:
                self.txt_TW_Eref.setText("194.8")
                self.ShowMsg("No temperature match found for E<sub>ref</sub> in material data file. Value of 194.8 GPa will be used.",  "blue")
        if material != "" and temp != "":
            if self.CheckTWTemp(material,  temp):
                dens = self.GetMaterialData(material,  temp,  "dens")
                self.txt_TW_dens.setText(dens)
                elast = self.GetMaterialData(material,  temp,  "elast")
                elast = str(float(elast) / 1000000000)
                self.txt_TW_Et.setText(elast)
                stress = self.GetMaterialData(material,  temp,  "stress")
                stress = str(float(stress) / 1000000)
                self.txt_TW_stress_limit.setText(stress)
            else:
                self.ShowMsg("No temperature match found for " + temp + "&#x00B0; C in material data file. Extrapolated data will be used where possible or add new material data.",  "blue")
                dens = self.GetExtrapolatedMaterialData(material,  temp,  "dens")
                self.txt_TW_dens.setText(dens)
                elast = self.GetExtrapolatedMaterialData(material,  temp,  "elast")
                elast = str(float(elast) / 1000000000)
                self.txt_TW_Et.setText(elast)
                stress = self.GetExtrapolatedMaterialData(material,  temp,  "stress")
                stress = str(float(stress) / 1000000)
                self.txt_TW_stress_limit.setText(stress)
        else:
            self.ClearTWSelectInputs()

    def CheckTWTemp(self, matvar, tempvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if int(temp.text) == int(tempvar):
                        return True
                return False

    def GetMaterialData(self, matvar, tempvar, tag):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        value = temp.find(tag)
                        return value.text

    def GetExtrapolatedMaterialData(self, matvar, tempvar, tag):
        try:
            templist = QStringList()
            for material in self.materials:
                if material.attrib["name"] == matvar:
                    temps = material.getiterator("temp")
                    for temp in temps:
                        templist.append(temp.text)
            templist.sort()
            target = int(tempvar)
            x1 = max(int(t) for t in templist if t != '' and int(t) < target)
            x2 = min(int(t) for t in templist if t != '' and int(t) > target)
            y1 = float(self.GetMaterialData(matvar, str(x1), tag))
            y2 = float(self.GetMaterialData(matvar, str(x2), tag))
            x = target
            y = y1 - ((y1 - y2) * (x - x1) / (x2 - x1))
            return str(y)
        except Exception, inst:
            return "0"

    def ClearTWSelectInputs(self):
        self.ShowMsg("",  "blue")
        self.txt_TW_Eref.setText("")
        self.txt_TW_Et.setText("")
        self.txt_TW_dens.setText("")
        self.txt_TW_stress_limit.setText("")

    def GetFlangeMaterials(self):
        self.flanges_xml = self.tree.parse(self.flangedata_file)
        self.flanges = self.flanges_xml.getiterator("material")
        matlist = QStringList("")
        for material in self.flanges:
            matlist.append(material.attrib["name"])
        return matlist

    def OnFlangeMatCurrentIndexChanged(self):
        self.ClearFlangeSelectInputs()
        self.cb_flange_class.clear()
        material = self.cb_flange_mat.currentText()
        press = self.txt_design_press.text()
        temp = self.txt_design_temp.text()
        if material != "" and press != "" and temp != "":
            templist = self.GetFlangeTemp(material,  temp)
            if self.CheckFlangeTemp(material,  temp):
                classlist = self.GetFlangeClass(material,  press,  temp)
                self.cb_flange_class.addItems(classlist)
            elif len(templist) > 1:
                higher_temp = min(int(t) for t in templist if t != '' and int(t) > int(temp))
                classlist = self.GetFlangeClass(material,  press,  str(higher_temp))
                self.cb_flange_class.addItems(classlist)
            else:
                self.ShowMsg("No flange data found for " + temp + "&#x00B0; C in flange data file. Add new flange data.",  "red")
        else:
            self.ClearFlangeSelectInputs()
            self.cb_flange_class.clear()

    def OnFlangeClassCurrentIndexChanged(self):
        material = self.cb_flange_mat.currentText()
        temp = self.txt_design_temp.text()
        fclass = self.cb_flange_class.currentText()
        if material != "" and temp != "" and fclass != "":
            if self.CheckFlangeTemp(material, temp):
                press = self.GetFlangeData(material,  temp,  fclass,  "pressure")
                press = str(float(press) / 1000)
                self.txt_TW_press_limit.setText(press)
            else:
                self.ShowMsg("No temperature match found for " + temp + "&#x00B0; C in flange data file. Extrapolated data will be used where possible or add new flange data.",  "blue")
                press = self.GetExtrapolatedFlangeData(material,  temp,  fclass,  "pressure")
                press = str(float(press) / 1000)
                self.txt_TW_press_limit.setText(press)
        else:
            self.ClearFlangeSelectInputs()

    def CheckFlangeTemp(self, matvar, tempvar):
        for material in self.flanges:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if int(temp.text) == int(tempvar):
                        return True
                return False

    def GetFlangeTemp(self, matvar, tempvar):
        templist = QStringList("")
        for material in self.flanges:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    templist.append(temp.text)
                return (templist)

    def GetFlangeClass(self, matvar, pressvar,  tempvar):
        classlist = QStringList("")
        for material in self.flanges:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        classes = temp.getiterator("class")
                        for fclass in classes:
                            press = fclass.find("pressure")
                            if float(press.text) > float(pressvar) * 1000:
                                classlist.append(fclass.text)
                        return classlist

    def GetFlangeData(self, matvar, tempvar, classvar, tag):
        for material in self.flanges:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        classes = temp.getiterator("class")
                        for fclass in classes:
                            if fclass.text == classvar:
                                press = fclass.find(tag)
                                return press.text

    def GetExtrapolatedFlangeData(self, matvar, tempvar, classvar, tag):
        try:
            templist = QStringList()
            for material in self.materials:
                if material.attrib["name"] == matvar:
                    temps = material.getiterator("temp")
                    for temp in temps:
                        templist.append(temp.text)
            templist.sort()
            target = int(tempvar)
            x1 = max(int(t) for t in templist if t != '' and int(t) < target)
            x2 = min(int(t) for t in templist if t != '' and int(t) > target)
            y1 = float(self.GetFlangeData(matvar, str(x1), classvar, tag))
            y2 = float(self.GetFlangeData(matvar, str(x2), classvar, tag))
            x = target
            y = y1 - ((y1 - y2) * (x - x1) / (x2 - x1))
            return str(y)
        except Exception, inst:
            return "0"

    def ClearFlangeSelectInputs(self):
        self.ShowMsg("",  "blue")
        self.txt_TW_press_limit.setText("")

    def OnDesignTempTextChanged(self):
        self.txt_fluid_temp.setText(self.txt_design_temp.text())
        self.cb_TW_mat.clear()
        self.cb_TW_mat.addItems(self.GetTWMaterials())
        self.cb_flange_mat.clear()
        self.cb_flange_mat.addItems(self.GetFlangeMaterials())

    def OnDesignPressTextChanged(self):
        self.txt_fluid_press.setText(self.txt_design_press.text())
        self.cb_flange_mat.clear()
        self.cb_flange_mat.addItems(self.GetFlangeMaterials())

    def OnTWTipDiaTextChanged(self):
        if self.cb_TW_type.currentText() == "Straight":
            self.txt_TW_out_dia.setText(self.txt_TW_tip_dia.text())
        
    def GetPipeMaterials(self):
        matlist = QStringList("")
        self.pipes_xml = self.tree.parse(self.pipedata_file)
        self.pipes = self.pipes_xml.getiterator("material")
        for material in self.pipes:
            matlist.append(material.attrib["name"])
        return matlist

    def OnPipeMatCurrentIndexChanged(self):
        self.ClearPipeSelectInputs()
        self.cb_pipe_DN.clear()
        material = self.cb_pipe_mat.currentText()
        if material != "":
            dnlist = self.GetPipeDN(material)
            self.cb_pipe_DN.addItems(dnlist)
        else:
            self.ClearPipeSelectInputs()
            self.cb_pipe_DN.clear()

    def GetPipeDN(self,  matvar):
        dnlist = QStringList("")
        for material in self.pipes:
            if material.attrib["name"] == matvar:
                dns = material.getiterator("DN")
                for dn in dns:
                    dnlist.append(dn.text)
                return dnlist

    def OnPipeDNCurrentIndexChanged(self):
        self.cb_pipe_sch.clear()
        material = self.cb_pipe_mat.currentText()
        dn = self.cb_pipe_DN.currentText()
        if material != "" and dn != "":
            schedulelist = self.GetPipeSch(material,  dn)
            self.cb_pipe_sch.addItems(schedulelist)
        else:
            self.cb_pipe_sch.clear()

    def GetPipeSch(self,  matvar,  dnvar):
        schedulelist = QStringList("")
        for material in self.pipes:
            if material.attrib["name"] == matvar:
                dns = material.getiterator("DN")
                for dn in dns:
                    if dn.text == dnvar:
                        schedules = dn.getiterator("schedule")
                        for schedule in schedules:
                            schedulelist.append(schedule.text)
                        return schedulelist

    def OnPipeSchCurrentIndexChanged(self):
        self.ClearPipeSelectInputs()
        material = self.cb_pipe_mat.currentText()
        dn = self.cb_pipe_DN.currentText()
        schedule = self.cb_pipe_sch.currentText()
        if schedule != "":
            od = self.GetPipeData(material,  dn,  schedule,  "OD")
            od = str(float(od) * 1000)
            self.lb_pipe_OD_value.setText(od)
            wt = self.GetPipeData(material,  dn,  schedule,  "WT")
            wt = str(float(wt) * 1000)
            self.lb_pipe_WT_value.setText(wt)
            id = self.GetPipeData(material,  dn,  schedule,  "ID")
            id = str(float(id) * 1000)
            self.txt_pipe_ID.setText(id)
        else:
            self.ClearPipeSelectInputs()

    def GetPipeData(self,  matvar,  dnvar,  schedulevar,  tag):
        for material in self.pipes:
            if material.attrib["name"] == matvar:
                dns = material.getiterator("DN")
                for dn in dns:
                    if dn.text == dnvar:
                        schedules = dn.getiterator("schedule")
                        for schedule in schedules:
                            if schedule.text == schedulevar:
                                value = schedule.find(tag)
                                return value.text

    def ClearPipeSelectInputs(self):
        self.ShowMsg("",  "blue")
        self.lb_pipe_OD_value.setText("")
        self.lb_pipe_WT_value.setText("")
        self.txt_pipe_ID.setText("")

    def Calculate(self):
        self.ShowMsg("",  "blue")
        type = self.cb_TW_type.currentText()
        if type == "Tapered":
            self.tapered_calc = TaperedCalc(self.results_file)
            return self.tapered_calc.Calculate(self.GetData())
        elif type == "Straight":
            self.straight_calc = StraightCalc(self.results_file)
            return self.straight_calc.Calculate(self.GetData())
        elif type == "Step":
            self.step_calc = StepCalc(self.results_file)
            return self.step_calc.Calculate(self.GetData())
        else:
            return "Error"

    def ClearData(self):
        if self.FormClearDlg():
            self.txt_TW_out_dia.setText("")
            self.txt_TW_tip_dia.setText("")
            self.cb_TW_step_tip_dia.setCurrentIndex(0)
            self.txt_TW_bore_dia.setText("")
            self.txt_TW_fillet_rad.setText("")
            self.txt_TW_step_fillet_rad.setText("")
            self.txt_TW_tip_thick.setText("")
            self.txt_TW_length.setText("")
            self.txt_TW_shield_length.setText("")
            self.txt_TW_step_length.setText("")
            
            self.cb_TW_type.setCurrentIndex(0)
            self.cb_TW_connect.setCurrentIndex(0)
            self.cb_TW_install.setCurrentIndex(0)
            self.txt_design_temp.setText("")
            self.txt_design_press.setText("")
            self.chk_NDT.setChecked(False)
            
            self.txt_TW_Eref.setText("")
            self.txt_TW_Et.setText("")
            self.txt_TW_dens.setText("")
            self.txt_TW_stress_limit.setText("")
            self.txt_TW_fatigue_limit.setText("")
            
            self.cb_pipe_mat.setCurrentIndex(0)
            self.txt_pipe_ID.setText("")
            self.chk_TW_stress.setChecked(False)
            
            self.txt_fluid_rate.setText("")
            self.txt_fluid_dens.setText("")
            self.txt_fluid_spec_vol.setText("")
            self.txt_fluid_velos.setText("")
            self.txt_fluid_viscos.setText("")
        
        return True

    def FormClearDlg(self):
        dlg = QMessageBox(self)
        dlg.setIcon(QMessageBox.Warning)
        dlg.setWindowTitle("Clear Form - Cute Thermowell")
        dlg.setText("Clear form without saving data?")
        dlg.addButton(QMessageBox.Ok)
        dlg.addButton(QMessageBox.Cancel)
        dlg.setDefaultButton(QMessageBox.Cancel)
        button = dlg.exec_()
        if (button == QMessageBox.Ok):
            return True
        else:
            return False        

    def GetData(self):
        input_data = InputData()
        
        input_data.version = self.parent.version
        
        if self.txt_TW_out_dia.text() != "":
            input_data.A = float(self.txt_TW_out_dia.text()) / 1000
        else:
            input_data.A = 0.0
        if self.cb_TW_type.currentText() != "Step" and self.txt_TW_tip_dia.text() != "":
            input_data.B = float(self.txt_TW_tip_dia.text()) / 1000
        elif self.cb_TW_type.currentText() == "Step" and self.cb_TW_step_tip_dia.currentText() != "":
            input_data.B = float(self.cb_TW_step_tip_dia.currentText()) / 1000
        else:
            input_data.B = 0.0
        if self.txt_TW_bore_dia.text() != "":
            input_data.d = float(self.txt_TW_bore_dia.text()) / 1000
        else:
            input_data.d = 0.0
        if self.txt_TW_fillet_rad.text() != "":
            input_data.b = float(self.txt_TW_fillet_rad.text()) / 1000
        else:
            input_data.b = 0.0
        if self.txt_TW_step_fillet_rad.text() != "":
            input_data.bs = float(self.txt_TW_step_fillet_rad.text()) / 1000
        else:
            input_data.bs = 0.0
        if self.txt_TW_tip_thick.text() != "":
            input_data.t = float(self.txt_TW_tip_thick.text()) / 1000
        else:
            input_data.t = 0.0
        if self.txt_TW_length.text() != "":
            input_data.L = float(self.txt_TW_length.text()) / 1000
        else:
            input_data.L = 0.0
        if self.txt_TW_shield_length.text() != "":
            input_data.Lo = float(self.txt_TW_shield_length.text()) / 1000
        else:
            input_data.Lo = 0.0
        if self.txt_TW_step_length.text() != "":
            input_data.Ls = float(self.txt_TW_step_length.text()) / 1000
        else:
            input_data.Ls = 0.0
        
        input_data.TW_type = self.cb_TW_type.currentText()
        input_data.TW_connect = self.cb_TW_connect.currentText()
        input_data.TW_install = self.cb_TW_install.currentText()
        if self.txt_design_press.text() != "":
            input_data.P = float(self.txt_design_press.text())
        else:
            input_data.P = 0.0
        if self.txt_design_temp.text() != "":
            input_data.T = float(self.txt_design_temp.text())
        else:
            input_data.T = 0.0
        input_data.TW_mat = self.cb_TW_mat.currentText()
        input_data.flange_mat = self.cb_flange_mat.currentText()
        input_data.flange_class = self.cb_flange_class.currentText()
        input_data.NDT = self.chk_NDT.isChecked()
        
        if self.txt_TW_Eref.text() != "":
            input_data.TW_Eref = float(self.txt_TW_Eref.text()) * 1000000000
        else:
            input_data.TW_Eref = 0.0
        if self.txt_TW_Et.text() != "":
            input_data.TW_Et = float(self.txt_TW_Et.text()) * 1000000000
        else:
            input_data.TW_Et = 0.0
        if self.txt_TW_dens.text() != "":
            input_data.TW_dens = float(self.txt_TW_dens.text())
        else:
            input_data.TW_dens = 0.0
        if self.txt_sensor_dens.text() != "":
            input_data.sensor_dens = float(self.txt_sensor_dens.text())
        else:
            input_data.sensor_dens = 0.0
        if self.txt_TW_stress_limit.text() != "":
            input_data.TW_stress_limit = float(self.txt_TW_stress_limit.text()) * 1000000
        else:
            input_data.TW_stress_limit = 0.0
        if self.txt_TW_fatigue_limit.text() != "":
            input_data.TW_fatigue_limit = float(self.txt_TW_fatigue_limit.text()) * 1000000
        else:
            input_data.TW_fatigue_limit = 0.0
        if self.txt_TW_press_limit.text() != "":
            input_data.TW_press_limit = float(self.txt_TW_press_limit.text()) * 1000
        else:
            input_data.TW_press_limit = 0.0
        input_data.TW_stress = self.chk_TW_stress.isChecked()
        
        input_data.pipe_mat = self.cb_pipe_mat.currentText()
        input_data.pipe_DN = self.cb_pipe_DN.currentText()
        input_data.pipe_sch = self.cb_pipe_sch.currentText()
        if self.lb_pipe_OD.text() != "":
            input_data.pipe_OD = float(self.lb_pipe_OD_value.text()) / 1000
        else:
            input_data.pipe_OD = 0.0
        if self.lb_pipe_WT.text() != "":
            input_data.pipe_WT = float(self.lb_pipe_WT_value.text()) / 1000
        else:
            input_data.pipe_WT = 0.0
        if self.txt_pipe_ID.text() != "":
            input_data.pipe_ID = float(self.txt_pipe_ID.text()) / 1000
        else:
            input_data.pipe_ID = 0.0
        if self.txt_fluid_press.text() != "":
            input_data.fluid_press = float(self.txt_fluid_press.text()) * 1000
        else:
            input_data.fluid_press = 0.0
        if self.txt_fluid_temp.text() != "":
            input_data.fluid_temp = float(self.txt_fluid_temp.text())
        else:
            input_data.fluid_temp = 0.0
        if self.txt_fluid_rate.text() != "":
            input_data.fluid_rate = float(self.txt_fluid_rate.text()) / 3600
        else:
            input_data.fluid_rate = 0.0
        if self.txt_fluid_dens.text() != "":
            input_data.fluid_dens = float(self.txt_fluid_dens.text())
        elif self.txt_fluid_dens.text() == "" and self.txt_fluid_spec_vol.text() != "":
            input_data.fluid_dens = 1 / float(self.txt_fluid_spec_vol.text())
        else:
            input_data.fluid_dens = 0.0
        if self.txt_fluid_spec_vol.text() != "":
            input_data.fluid_spec_vol = float(self.txt_fluid_spec_vol.text())
        elif self.txt_fluid_spec_vol.text() == "" and self.txt_fluid_dens.text() != "":
            input_data.fluid_spec_vol = 1 / float(self.txt_fluid_dens.text())
        else:
            input_data.fluid_spec_vol = 0.0
        if self.txt_fluid_velos.text() != "":
            input_data.fluid_velos = float(self.txt_fluid_velos.text())
        else:
            input_data.fluid_velos = 0.0
        if self.txt_fluid_viscos.text() != "":
            input_data.fluid_viscos = float(self.txt_fluid_viscos.text()) / 1000
        else:
            input_data.fluid_viscos = 0.0

        return input_data
    
    def StartTimer(self):
        self.timer.start()
    
    def TimerHit(self):
        self.parent.lb_msg.setText("")

    def ShowMsg(self, msg,  color):
        self.parent.lb_msg.setText("<font color=\"" + color + "\">" + msg + "</font>")
        self.StartTimer()

    def OnImgTWTypeMousePressed(self):
        str_img = ""
        if self.cb_TW_type.currentText() == "Tapered":
            if self.cb_TW_connect.currentText() == "Flanged":
                str_img = "TW_flange_tapered_large.png"
            elif self.cb_TW_connect.currentText() == "Threaded":
                str_img = "TW_thread_tapered_large.png"
        elif self.cb_TW_type.currentText() == "Straight":
            if self.cb_TW_connect.currentText() == "Flanged":
                str_img = "TW_flange_straight_large.png"
            elif self.cb_TW_connect.currentText() == "Threaded":
                str_img = "TW_thread_straight_large.png"
        elif self.cb_TW_type.currentText() == "Step":
            if self.cb_TW_connect.currentText() == "Flanged":
                str_img = "TW_flange_step_large.png"
            elif self.cb_TW_connect.currentText() == "Threaded":
                str_img = "TW_thread_step_large.png"
        if str_img != "":
            self.parent.img_widget = TWImageWidget(str_img)
            self.parent.img_widget.show()

class InputData(object):
    def __init__(self, parent = None):
        
        version = ""
        
        A = 0.0
        B = 0.0
        d = 0.0
        b = 0.0
        bs = 0.0
        t = 0.0
        L = 0.0
        Lo = 0.0
        Ls = 0.0
        
        TW_type = ""
        TW_connect = ""
        TW_install = ""
        P = 0.0
        T = 0.0
        TW_mat = ""
        flange_mat = ""
        flange_class = ""
        NDT = False
        
        TW_Eref = 0.0
        TW_Et = 0.0
        TW_dens = 0.0
        sensor_dens = 0.0
        TW_stress_limit = 0.0
        TW_fatigue_limit = 0.0
        TW_press_limit = 0.0
        
        pipe_mat = ""
        pipe_DN = ""
        pipe_sch = ""
        pipe_OD = 0.0
        pipe_WT = 0.0
        pipe_ID = 0.0
        TW_stress = False
        
        fluid_press = 0.0
        fluid_temp = 0.0
        fluid_rate = 0.0
        fluid_dens = 0.0
        fluid_spec_vol = 0.0
        fluid_velos = 0.0
        fluid_viscos = 0.0

class ImgLabel(QLabel):
    def __init__(self, *args):
        QLabel.__init__(self, *args)
        
    mousePressedSignal = pyqtSignal()

    def mousePressEvent(self, event):
        if (event.button() == Qt.LeftButton):
            self.mousePressedSignal.emit()
