# -*- coding: utf-8 -*-
"""
Program:
    Cute Thermowell
    (qt-thermowell)
    MainWindow.py

Author:
    Alex Borisov <>

Copyright (c) 2011 Alex Borisov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
import sys
import resources
from FormWidget import FormWidget
from MaterialData import MaterialDataDlg
from FlangeData import FlangeDataDlg
from PipeData import PipeDataDlg
from Results import ResultsDlg
from QtConverter.MainWindow import MainWindow as Converter

class MainWindow(QMainWindow):
    def __init__(self, parent = None):
        QMainWindow.__init__(self)
        
        self.version = "0.7"
        
        self.matdata_dlg = None
        self.flangedata_dlg = None
        self.pipedata_dlg = None
        self.results_win = None
        self.converter_win = None
        self.materialdata_file = QString("")
        self.flangedata_file = QString("")
        self.pipedata_file = QString("")
        self.results_file = QString("")
        self.script_path = sys.path[0]
        self.ReadSettings()
        self.MainUI()

    def ReadSettings(self):
        if sys.platform == "win32":
            settings = QSettings("qt-thermowell.ini", QSettings.IniFormat)
        else:
            settings = QSettings(QDir.homePath() + "/.qt-thermowell/qt-thermowell.conf", QSettings.NativeFormat)
        settings.beginGroup("Materialdata")
        self.materialdata_file = settings.value("materialdata_file", self.script_path + "/data/materialdata.xml").toString()
        settings.endGroup()
        settings.beginGroup("Flangedata")
        self.flangedata_file = settings.value("flangedata_file", self.script_path + "/data/flangedata.xml").toString()
        settings.endGroup()
        settings.beginGroup("Pipedata")
        self.pipedata_file = settings.value("pipedata_file", self.script_path + "/data/pipedata.xml").toString()
        settings.endGroup()
        settings.beginGroup("MainWindow")
        self.results_file = settings.value("results_file", self.script_path + "/calc_results.html").toString()
        settings.endGroup()
    
    def MainUI(self):
        self.win_icon = QIcon(":/myresources/icons/thermowell.ico")
        self.setWindowIcon(self.win_icon)
        self.win_size = QSize(700, 300)
        self.setWindowTitle("Cute Thermowell")
        self.resize(self.win_size)

        self.icon_size = QSize(20, 20)
        self.btn_size = QSize(100, 25)

        self.h_spacer = QSpacerItem(2, 20, QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        self.v_spacer = QSpacerItem(20, 2, QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)

        self.main_widget = QWidget(self)

        self.icon_clear = QIcon(":/myresources/icons/clear.svg")
        self.icon_load = QIcon(":/myresources/icons/load.svg")
        self.icon_save = QIcon(":/myresources/icons/save.svg")
        self.icon_quit = QIcon(":/myresources/icons/quit.svg")
        self.icon_calc = QIcon(":/myresources/icons/calc.svg")
        self.icon_view = QIcon(":/myresources/icons/view.svg")
        self.icon_material = QIcon(":/myresources/icons/material.png")
        self.icon_flange = QIcon(":/myresources/icons/flange.png")
        self.icon_pipe = QIcon(":/myresources/icons/pipe.png")
        self.icon_convert = QIcon(":/myresources/QtConverter/icons/converter.svg")
        self.icon_about = QIcon(":/myresources/icons/about.svg")

        self.action_clear = QAction(self.icon_clear, "&Clear Form", self)
        self.action_load = QAction(self.icon_load, "&Load Form Data", self)
        self.action_save = QAction(self.icon_save, "&Save Form Data", self)
        self.action_quit = QAction(self.icon_quit, "&Quit", self)
        self.action_calc = QAction(self.icon_calc, "&Calculate", self)
        self.action_view = QAction(self.icon_view, "&View Results", self)
        self.action_view.setEnabled(False)
        self.action_matdata = QAction(self.icon_material, "Edit &Material Data", self)
        self.action_flangedata = QAction(self.icon_flange, "Edit &Flange Data", self)
        self.action_pipedata = QAction(self.icon_pipe, "Edit &Pipe Data", self)
        self.action_convert = QAction(self.icon_convert, "Convert &Units", self)
        self.action_about = QAction(self.icon_about, "&About", self)

        self.menu_bar = QMenuBar(self)
        self.menu_bar.Enabled = True
        self.setMenuBar(self.menu_bar)

        self.menu_file = QMenu("&File", self.menu_bar);
        self.menu_tools = QMenu("&Tools", self.menu_bar);
        self.menu_help = QMenu("&Help", self.menu_bar);

        self.menu_bar.addMenu(self.menu_file)
        self.menu_bar.addMenu(self.menu_tools)
        self.menu_bar.addMenu(self.menu_help)

        self.menu_file.addAction(self.action_clear)
        self.menu_file.addAction(self.action_load)
        self.menu_file.addAction(self.action_save)
        self.menu_file.addSeparator()
        self.menu_file.addAction(self.action_quit)

        self.menu_tools.addAction(self.action_calc)
        self.menu_tools.addAction(self.action_view)
        self.menu_tools.addAction(self.action_matdata)
        self.menu_tools.addAction(self.action_flangedata)
        self.menu_tools.addAction(self.action_pipedata)
        self.menu_tools.addSeparator()
        self.menu_tools.addAction(self.action_convert)

        self.menu_help.addAction(self.action_about)

        self.tool_bar = QToolBar(self)
        self.tool_bar.addAction(self.action_clear)
        self.tool_bar.addAction(self.action_load)
        self.tool_bar.addAction(self.action_save)
        self.tool_bar.addSeparator()
        self.tool_bar.addAction(self.action_calc)
        self.tool_bar.addAction(self.action_view)
        self.tool_bar.addAction(self.action_matdata)
        self.tool_bar.addAction(self.action_flangedata)
        self.tool_bar.addAction(self.action_pipedata)
        self.tool_bar.addSeparator()
        self.tool_bar.addAction(self.action_convert)
        self.tool_bar.addSeparator()
        self.tool_bar.addAction(self.action_about)
        self.tool_bar.addSeparator()
        self.tool_bar.addAction(self.action_quit)
        self.addToolBar(Qt.TopToolBarArea, self.tool_bar)

        self.status_bar = QStatusBar(self)
        self.setStatusBar(self.status_bar)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_msg = QLabel(self.main_widget)
        self.lb_msg.setTextFormat(Qt.RichText)
        self.lb_msg.setText("")
        size_policy.setHeightForWidth(self.lb_msg.sizePolicy().hasHeightForWidth())
        self.lb_msg.setSizePolicy(size_policy)

        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.btn_calc = QPushButton(self.icon_calc, "Calculate", self.main_widget)
        self.btn_calc.setIconSize(self.icon_size)
        sizePolicy.setHeightForWidth(self.btn_calc.sizePolicy().hasHeightForWidth())
        self.btn_calc.setSizePolicy(sizePolicy)
        self.btn_calc.setMinimumSize(self.btn_size)
        self.btn_quit = QPushButton(self.icon_quit, "Quit", self.main_widget)
        self.btn_quit.setIconSize(self.icon_size)
        sizePolicy.setHeightForWidth(self.btn_quit.sizePolicy().hasHeightForWidth())
        self.btn_quit.setSizePolicy(sizePolicy)
        self.btn_quit.setMinimumSize(self.btn_size)

        self.hb_button_layout = QHBoxLayout()
        self.hb_button_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.hb_button_layout.addWidget(self.lb_msg)
        self.hb_button_layout.addItem(self.h_spacer)
        self.hb_button_layout.addWidget(self.btn_calc)
        self.hb_button_layout.addWidget(self.btn_quit)
        
        if not self.FindDataFile(self.materialdata_file):
            self.materialdata_file = self.SelectDataFile("material")
                
        if not self.FindDataFile(self.flangedata_file):
            self.flangedata_file = self.SelectDataFile("flange")
                
        if not self.FindDataFile(self.pipedata_file):
            self.pipedata_file = self.SelectDataFile("pipe")

        self.main_layout = QVBoxLayout(self.main_widget)
        self.main_layout.sizeConstraint = QLayout.SetDefaultConstraint
        if self.materialdata_file and self.flangedata_file and self.pipedata_file:
            self.form_widget = FormWidget(self.materialdata_file, self.flangedata_file, self.pipedata_file, self.results_file,  self)
            self.main_layout.addWidget(self.form_widget.main_widget)
        else:
            self.btn_calc.setDisabled(True)
            self.ShowStatus("No data files selected.")
        self.main_layout.addLayout(self.hb_button_layout)

        self.main_widget.setLayout(self.main_layout)
        self.setCentralWidget(self.main_widget)

        self.connect(self.action_clear, SIGNAL("triggered()"), self.OnActionClearTriggered)
        self.connect(self.action_load, SIGNAL("triggered()"), self.OnActionLoadTriggered)
        self.connect(self.action_save, SIGNAL("triggered()"), self.OnActionSaveTriggered)
        self.connect(self.action_quit, SIGNAL("triggered()"), self.Quit)
        self.connect(self.action_calc, SIGNAL("triggered()"), self.OnActionCalcTriggered)
        self.connect(self.action_convert, SIGNAL("triggered()"), self.OnActionConvertTriggered)
        self.connect(self.action_view, SIGNAL("triggered()"), self.OnActionViewTriggered)
        self.connect(self.action_matdata, SIGNAL("triggered()"), self.OnActionMatdataTriggered)
        self.connect(self.action_flangedata, SIGNAL("triggered()"), self.OnActionFlangedataTriggered)
        self.connect(self.action_pipedata, SIGNAL("triggered()"), self.OnActionPipedataTriggered)
        self.connect(self.action_about, SIGNAL("triggered()"), self.OnActionAboutTriggered)
        self.connect(self.btn_calc, SIGNAL("clicked()"), self.OnBtnCalcClicked)
        self.connect(self.btn_quit, SIGNAL("clicked()"), self.Quit)
    
    def FindDataFile(self, file_name):
        file = QFile(file_name)
        if file.exists():
            return True
        else:
            return False
            
    def SelectDataFile(self,  s):
        dlg_title = "Select " + s + " data file - Cute Thermowell"
        dlg_filter = "XML files (*.xml)"
        factors_dir = QDir(QDir.rootPath())
        file_name = QFileDialog.getOpenFileName(self, dlg_title, factors_dir.absolutePath(), dlg_filter)
        file_info = QFileInfo(file_name)
        if file_info.isFile():
            return file_name
        else:
            return None

    def Quit(self):
        self.WriteSettings()
        self.close()

    def WriteSettings(self):
        if sys.platform == "win32":
            settings = QSettings("qt-thermowell.ini", QSettings.IniFormat)
        else:
            settings = QSettings(QDir.homePath() + "/.qt-thermowell/qt-thermowell.conf", QSettings.NativeFormat)
        settings.beginGroup("Materialdata")
        settings.setValue("materialdata_file", self.materialdata_file)
        settings.endGroup()
        settings.beginGroup("Flangedata")
        settings.setValue("flangedata_file", self.flangedata_file)
        settings.endGroup()
        settings.beginGroup("Pipedata")
        settings.setValue("pipedata_file", self.pipedata_file)
        settings.endGroup()
        settings.beginGroup("MainWindow")
        settings.setValue("results_file", self.results_file)
        settings.endGroup()

    def OnActionAboutTriggered(self):
        msg = "Cute Thermowell - Qt-based Thermowell Design Calculator (SI Units)\n\n"
        msg += "Thermowell design calculation program based on latest standard ASME PTC 19.3 TW-2010 and "
        msg += "uses material data from ASME II, ASME VIII, ASME B16.5, ASME B31.1, ASME B31.3, ASME B36.10 and others.\n\n"
        msg += "Version " + self.version + "\n\n"
        msg += unichr(int('00A9', 16)) + " 2010-" + QDateTime().currentDateTime().toString("yyyy") + ", Alex Borisov\n\n"
        msg += "Distributed under GPL ver. 3"
        QMessageBox.about(self, "About - Cute Thermowell", msg)

    def OnBtnCalcClicked(self):
        test = self.form_widget.Calculate()
        if test == "Pass":
            self.OpenResults()
            self.action_view.setEnabled(True)
        elif test == "Fail":
            self.action_view.setEnabled(True)
            QMessageBox.about(self, "Calculation Result - Cute Thermowell", "Calculation failed or parameters outside of Standard scope.")
        elif test == "Error":
            self.action_view.setEnabled(False)
            QMessageBox.warning(self, "Error - Cute Thermowell", "Calculation error occurred: input(s) may be empty or parameters are outside of Standard scope.")
        else:
            self.action_view.setEnabled(False)
            QMessageBox.warning(self, "Error - Cute Thermowell", "Unexpected error occurred.")

    def OnActionClearTriggered(self):
        self.form_widget.ClearData()

    def OnActionLoadTriggered(self):
        self.LoadFormData()

    def OnActionSaveTriggered(self):
        name, ok = QInputDialog.getText(self, "Form Data - Cute Thermowell", "Enter Form Data Name: ")
        if ok and name != "":
                self.SaveFormData(name)
        else:
            self.status_bar.setStyleSheet("color: rgb(255, 0, 0)")
            self.status_bar.showMessage("No form data saved", 5000)

    def OnActionCalcTriggered(self):
        self.OnBtnCalcClicked()

    def OnActionViewTriggered(self):
        self.OpenResults()

    def OpenResults(self):
        if self.FindDataFile(self.results_file):
            self.results_win = ResultsDlg(self.results_file)
            self.results_win.show()
        else:
            self.action_view.setEnabled(False)
            self.status_bar.setStyleSheet("color: rgb(255, 0, 0)")
            self.status_bar.showMessage("No results file found", 5000)

    def OnActionMatdataTriggered(self):
        if self.matdata_dlg:
            self.matdata_dlg.show()
        else:
            self.matdata_dlg = MaterialDataDlg(self.materialdata_file)
            self.matdata_dlg.show()
            self.connect(self.matdata_dlg, SIGNAL("closed()"), self.OnTWDataDlgClosed)

    def OnActionFlangedataTriggered(self):
        if self.flangedata_dlg:
            self.flangedata_dlg.show()
        else:
            self.flangedata_dlg = FlangeDataDlg(self.flangedata_file)
            self.flangedata_dlg.show()
            self.connect(self.flangedata_dlg, SIGNAL("closed()"), self.OnTWDataDlgClosed)

    def OnActionPipedataTriggered(self):
        if self.pipedata_dlg:
            self.pipedata_dlg.show()
        else:
            self.pipedata_dlg = PipeDataDlg(self.pipedata_file)
            self.pipedata_dlg.show()
            self.connect(self.pipedata_dlg, SIGNAL("closed()"), self.OnPipeDataDlgClosed)

    def OnActionConvertTriggered(self):
        if self.converter_win:
            self.converter_win.show()
        else:
            self.converter_win = Converter(self)
            self.converter_win.show()

    def OnTWDataDlgClosed(self):
        self.form_widget.cb_TW_mat.clear()
        self.form_widget.cb_TW_mat.addItems(self.form_widget.GetTWMaterials())
        self.form_widget.cb_flange_mat.clear()
        self.form_widget.cb_flange_mat.addItems(self.form_widget.GetFlangeMaterials())

    def OnPipeDataDlgClosed(self):
        self.form_widget.cb_pipe_mat.clear()
        self.form_widget.cb_pipe_mat.addItems(self.form_widget.GetPipeMaterials())

    def LoadFormData(self):
        groupNames = QStringList()
        if sys.platform == "win32":
            settings = QSettings("form_data.ini", QSettings.IniFormat)
        else:
            settings = QSettings(QDir.homePath() + "/.qt-thermowell/form_data.conf", QSettings.NativeFormat)
        groupNames = settings.childGroups()
        name, ok = QInputDialog.getItem(self, "Form Data - Cute Thermowell", "Select Form Data Name: ", groupNames)
        if ok and not name.isEmpty():
            settings.beginGroup(name)
            index = self.form_widget.cb_TW_type.findText(settings.value("TW_type", "").toString())
            if index != -1:
                self.form_widget.cb_TW_type.setCurrentIndex(index)
            index = self.form_widget.cb_TW_connect.findText(settings.value("TW_connect", "").toString())
            if index != -1:
                self.form_widget.cb_TW_connect.setCurrentIndex(index)
            index = self.form_widget.cb_TW_install.findText(settings.value("TW_install", "").toString())
            if index != -1:
                self.form_widget.cb_TW_install.setCurrentIndex(index)
            self.form_widget.txt_design_press.setText(settings.value("design_press", "").toString())
            self.form_widget.txt_design_temp.setText(settings.value("design_temp", "").toString())
            index = self.form_widget.cb_TW_mat.findText(settings.value("TW_mat", "").toString())
            if index != -1:
                self.form_widget.cb_TW_mat.setCurrentIndex(index)
            index = self.form_widget.cb_flange_mat.findText(settings.value("flange_mat", "").toString())
            if index != -1:
                self.form_widget.cb_flange_mat.setCurrentIndex(index)
            index = self.form_widget.cb_flange_class.findText(settings.value("flange_class", "").toString())
            if index != -1:
                self.form_widget.cb_flange_class.setCurrentIndex(index)
            if settings.value("NDT", "True").toString() == "False":
                self.form_widget.chk_NDT.setChecked(False)
            else:
                self.form_widget.chk_NDT.setChecked(True)
                
            self.form_widget.txt_TW_out_dia.setText(settings.value("TW_out_dia", "").toString())
            self.form_widget.txt_TW_tip_dia.setText(settings.value("TW_tip_dia", "").toString())
            index = self.form_widget.cb_TW_step_tip_dia.findText(settings.value("TW_step_tip_dia", "").toString())
            if index != -1:
                self.form_widget.cb_TW_step_tip_dia.setCurrentIndex(index)
            self.form_widget.txt_TW_bore_dia.setText(settings.value("TW_bore_dia", "").toString())
            self.form_widget.txt_TW_fillet_rad.setText(settings.value("TW_fillet_rad", "").toString())
            self.form_widget.txt_TW_step_fillet_rad.setText(settings.value("TW_step_fillet_rad", "").toString())
            self.form_widget.txt_TW_step_fillet_rad.setText(settings.value("TW_step_fillet_rad", "").toString())
            self.form_widget.txt_TW_tip_thick.setText(settings.value("TW_tip_thick", "").toString())
            self.form_widget.txt_TW_length.setText(settings.value("TW_length", "").toString())
            self.form_widget.txt_TW_shield_length.setText(settings.value("TW_shield_length", "").toString())
            self.form_widget.txt_TW_step_length.setText(settings.value("TW_step_length", "").toString())
            
            index = self.form_widget.cb_pipe_mat.findText(settings.value("pipe_mat", "").toString())
            if index != -1:
                self.form_widget.cb_pipe_mat.setCurrentIndex(index)
            index = self.form_widget.cb_pipe_DN.findText(settings.value("pipe_DN", "").toString())
            if index != -1:
                self.form_widget.cb_pipe_DN.setCurrentIndex(index)
            index = self.form_widget.cb_pipe_sch.findText(settings.value("pipe_sch", "").toString())
            if index != -1:
                self.form_widget.cb_pipe_sch.setCurrentIndex(index)
            if settings.value("TW_stress", "True").toString() == "False":
                self.form_widget.chk_TW_stress.setChecked(False)
            else:
                self.form_widget.chk_TW_stress.setChecked(True)

            self.form_widget.txt_fluid_press.setText(settings.value("fluid_press", "").toString())
            self.form_widget.txt_fluid_temp.setText(settings.value("fluid_temp", "").toString())
            self.form_widget.txt_fluid_rate.setText(settings.value("fluid_rate", "").toString())
            self.form_widget.txt_fluid_dens.setText(settings.value("fluid_dens", "").toString())
            self.form_widget.txt_fluid_spec_vol.setText(settings.value("fluid_spec_vol", "").toString())
            self.form_widget.txt_fluid_velos.setText(settings.value("fluid_velos", "").toString())
            self.form_widget.txt_fluid_viscos.setText(settings.value("fluid_viscos", "").toString())
            settings.endGroup()
            
        else:
            self.status_bar.setStyleSheet("color: rgb(255, 0, 0)")
            self.status_bar.showMessage("No form data loaded", 5000)

    def SaveFormData(self, settingName):
        groupNames = QStringList()
        if sys.platform == "win32":
            settings = QSettings("form_data.ini", QSettings.IniFormat)
        else:
            settings = QSettings(QDir.homePath() + "/.qt-thermowell/form_data.conf", QSettings.NativeFormat)
        settings.beginGroup(settingName)
        settings.setValue("TW_out_dia", self.form_widget.txt_TW_out_dia.text())
        settings.setValue("TW_tip_dia", self.form_widget.txt_TW_tip_dia.text())
        settings.setValue("TW_step_tip_dia", self.form_widget.cb_TW_step_tip_dia.currentText())
        settings.setValue("TW_bore_dia", self.form_widget.txt_TW_bore_dia.text())
        settings.setValue("TW_fillet_rad", self.form_widget.txt_TW_fillet_rad.text())
        settings.setValue("TW_step_fillet_rad", self.form_widget.txt_TW_step_fillet_rad.text())
        settings.setValue("TW_tip_thick", self.form_widget.txt_TW_tip_thick.text())
        settings.setValue("TW_length", self.form_widget.txt_TW_length.text())
        settings.setValue("TW_shield_length", self.form_widget.txt_TW_shield_length.text())
        settings.setValue("TW_step_length", self.form_widget.txt_TW_step_length.text())
        
        settings.setValue("TW_type", self.form_widget.cb_TW_type.currentText())
        settings.setValue("TW_connect", self.form_widget.cb_TW_connect.currentText())
        settings.setValue("TW_install", self.form_widget.cb_TW_install.currentText())
        settings.setValue("design_press", self.form_widget.txt_design_press.text())
        settings.setValue("design_temp", self.form_widget.txt_design_temp.text())
        settings.setValue("TW_mat", self.form_widget.cb_TW_mat.currentText())
        settings.setValue("flange_mat", self.form_widget.cb_flange_mat.currentText())
        settings.setValue("flange_class", self.form_widget.cb_flange_class.currentText())
        if self.form_widget.chk_NDT.isChecked():
            settings.setValue("NDT", "True")
        else:
            settings.setValue("NDT", "False")
        
        settings.setValue("TW_Eref", self.form_widget.txt_TW_Eref.text())
        settings.setValue("TW_Et", self.form_widget.txt_TW_Et.text())
        settings.setValue("TW_dens", self.form_widget.txt_TW_dens.text())
        settings.setValue("sensor_dens", self.form_widget.txt_sensor_dens.text())
        settings.setValue("TW_stress_limit", self.form_widget.txt_TW_stress_limit.text())
        settings.setValue("TW_fatigue_limit", self.form_widget.txt_TW_fatigue_limit.text())
        settings.setValue("TW_press_limit", self.form_widget.txt_TW_press_limit.text())
        
        settings.setValue("pipe_mat", self.form_widget.cb_pipe_mat.currentText())
        settings.setValue("pipe_DN", self.form_widget.cb_pipe_DN.currentText())
        settings.setValue("pipe_sch", self.form_widget.cb_pipe_sch.currentText())
        settings.setValue("pipe_OD", self.form_widget.lb_pipe_OD_value.text())
        settings.setValue("pipe_WT", self.form_widget.lb_pipe_WT_value.text())
        settings.setValue("pipe_ID", self.form_widget.txt_pipe_ID.text())
        if self.form_widget.chk_TW_stress.isChecked():
            settings.setValue("TW_stress", "True")
        else:
            settings.setValue("TW_stress", "False")
        
        settings.setValue("fluid_press", self.form_widget.txt_fluid_press.text())
        settings.setValue("fluid_temp", self.form_widget.txt_fluid_temp.text())
        settings.setValue("fluid_rate", self.form_widget.txt_fluid_rate.text())
        settings.setValue("fluid_dens", self.form_widget.txt_fluid_dens.text())
        settings.setValue("fluid_spec_vol", self.form_widget.txt_fluid_spec_vol.text())
        settings.setValue("fluid_velos", self.form_widget.txt_fluid_velos.text())
        settings.setValue("fluid_viscos", self.form_widget.txt_fluid_viscos.text())
        settings.endGroup()

    def ShowStatus(self, msg):
        if msg == "":
            msg = "Cute Thermowell - Qt-based Thermowell Design Calculator"
        self.status_bar.setStyleSheet("color: rgb(0, 0, 255)")
        self.status_bar.showMessage(msg, 5000)
