# -*- coding: utf-8 -*-
"""
Program:
    Cute Thermowell
    (qt-thermowell)
    MaterialData.py

Author:
    Alex Borisov <>

Copyright (c) 2011 Alex Borisov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from xml.etree.ElementTree import ElementTree, Element, SubElement
import sys
import resources

class MaterialDataDlg(QDialog):
    def __init__(self, materialdata_file,  parent = None):
        QDialog.__init__(self)

        self.parent = parent
        self.materialdata_file = materialdata_file
        self.script_path = sys.path[0]

        self.tree = ElementTree()

        self.MatDataUI()

    def MatDataUI(self):
        self.win_icon = QIcon(":/myresources/icons/material.png")
        self.setWindowIcon(self.win_icon)
        self.win_size = QSize(500, 200)
        self.setWindowTitle("Edit Material Data - Cute Thermowell")
        self.resize(self.win_size)

        self.icon_size = QSize(20, 20)
        self.btn_size = QSize(100, 25)
        self.l_size = QSize(160, 20)
        self.txt_size = QSize(130, 20)
        self.cb_size = QSize(130, 20)
        self.chk_size = QSize(150, 20)
        self.radio_size = QSize(80, 20)
        self.txt_max_length = 10

        self.dbl_validator = QDoubleValidator()
        self.dbl_validator.setNotation(QDoubleValidator.StandardNotation)

        self.h_spacer = QSpacerItem(2, 20, QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        self.v_spacer = QSpacerItem(20, 2, QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)

        self.icon_save = QIcon(":/myresources/icons/save.svg")
        self.icon_cancel = QIcon(":/myresources/icons/cancel.svg")

        self.gb_matdata = QGroupBox("Material Data", self)
        self.gb_matdata.setStyleSheet(QString.fromUtf8("font-size: 8pt;"))
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_new_val = QLabel("New / Edit Values",  self.gb_matdata)
        size_policy.setHeightForWidth(self.lb_new_val.sizePolicy().hasHeightForWidth())
        self.lb_new_val.setAlignment(Qt.AlignHCenter)
        self.lb_new_val.setSizePolicy(size_policy)
        self.lb_new_val.setMinimumSize(self.l_size)
        self.lb_material = QLabel("Material", self.gb_matdata)
        size_policy.setHeightForWidth(self.lb_material.sizePolicy().hasHeightForWidth())
        self.lb_material.setSizePolicy(size_policy)
        self.lb_material.setMinimumSize(self.l_size)
        self.lb_temp = QLabel(self.gb_matdata)
        self.lb_temp.setTextFormat(Qt.RichText)
        self.lb_temp.setText("Temperature (&#x00B0; C)")
        size_policy.setHeightForWidth(self.lb_temp.sizePolicy().hasHeightForWidth())
        self.lb_temp.setSizePolicy(size_policy)
        self.lb_temp.setMinimumSize(self.l_size)
        self.lb_dens = QLabel(self.gb_matdata)
        self.lb_dens.setTextFormat(Qt.RichText)
        self.lb_dens.setText("Density (kg/m<sup>3</sup>)")
        size_policy.setHeightForWidth(self.lb_dens.sizePolicy().hasHeightForWidth())
        self.lb_dens.setSizePolicy(size_policy)
        self.lb_dens.setMinimumSize(self.l_size)
        self.lb_elast = QLabel("Module of Elasticity (GPa)", self.gb_matdata)
        size_policy.setHeightForWidth(self.lb_elast.sizePolicy().hasHeightForWidth())
        self.lb_elast.setSizePolicy(size_policy)
        self.lb_elast.setMinimumSize(self.l_size)
        self.lb_stress = QLabel("Allowable Stress Limit (MPag)", self.gb_matdata)
        size_policy.setHeightForWidth(self.lb_stress.sizePolicy().hasHeightForWidth())
        self.lb_stress.setSizePolicy(size_policy)
        self.lb_stress.setMinimumSize(self.l_size)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.txt_material = QLineEdit("", self.gb_matdata)
        size_policy.setHeightForWidth(self.txt_material.sizePolicy().hasHeightForWidth())
        self.txt_material.setSizePolicy(size_policy)
        self.txt_material.setMinimumSize(self.txt_size)
        self.txt_material.setMaxLength(20)
        self.txt_temp = QLineEdit("", self.gb_matdata)
        size_policy.setHeightForWidth(self.txt_temp.sizePolicy().hasHeightForWidth())
        self.txt_temp.setSizePolicy(size_policy)
        self.txt_temp.setMinimumSize(self.txt_size)
        self.txt_temp.setMaxLength(3)
        self.txt_temp.setValidator(self.dbl_validator)
        self.txt_dens = QLineEdit("", self.gb_matdata)
        size_policy.setHeightForWidth(self.txt_dens.sizePolicy().hasHeightForWidth())
        self.txt_dens.setSizePolicy(size_policy)
        self.txt_dens.setMinimumSize(self.txt_size)
        self.txt_dens.setMaxLength(self.txt_max_length)
        self.txt_dens.setValidator(self.dbl_validator)
        self.txt_elast = QLineEdit("", self.gb_matdata)
        size_policy.setHeightForWidth(self.txt_elast.sizePolicy().hasHeightForWidth())
        self.txt_elast.setSizePolicy(size_policy)
        self.txt_elast.setMinimumSize(self.txt_size)
        self.txt_elast.setMaxLength(self.txt_max_length)
        self.txt_elast.setValidator(self.dbl_validator)
        self.txt_stress = QLineEdit("", self.gb_matdata)
        size_policy.setHeightForWidth(self.txt_stress.sizePolicy().hasHeightForWidth())
        self.txt_stress.setSizePolicy(size_policy)
        self.txt_stress.setMinimumSize(self.txt_size)
        self.txt_stress.setMaxLength(self.txt_max_length)
        self.txt_stress.setValidator(self.dbl_validator)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.cb_material = QComboBox(self.gb_matdata)
        size_policy.setHeightForWidth(self.cb_material.sizePolicy().hasHeightForWidth())
        self.cb_material.setSizePolicy(size_policy)
        self.cb_material.setMinimumSize(self.cb_size)
        self.cb_temp = QComboBox(self.gb_matdata)
        size_policy.setHeightForWidth(self.cb_temp.sizePolicy().hasHeightForWidth())
        self.cb_temp.setSizePolicy(size_policy)
        self.cb_temp.setMinimumSize(self.cb_size)

        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.radio_exist_material = QRadioButton("Existing", self.gb_matdata)
        self.radio_exist_material.setSizePolicy(sizePolicy)
        self.radio_exist_material.setMinimumSize(self.radio_size)
        self.radio_new_material = QRadioButton("New", self.gb_matdata)
        self.radio_new_material.setSizePolicy(sizePolicy)
        self.radio_new_material.setMinimumSize(self.radio_size)
        self.radio_exist_temp = QRadioButton("Existing", self.gb_matdata)
        self.radio_exist_temp.setSizePolicy(sizePolicy)
        self.radio_exist_temp.setMinimumSize(self.radio_size)
        self.radio_new_temp = QRadioButton("New", self.gb_matdata)
        self.radio_new_temp.setSizePolicy(sizePolicy)
        self.radio_new_temp.setMinimumSize(self.radio_size)
        
        self.btngr_material = QButtonGroup()
        self.btngr_material.addButton(self.radio_exist_material)
        self.btngr_material.addButton(self.radio_new_material)
        self.radio_exist_material.setChecked(True)
        self.btngr_temp = QButtonGroup()
        self.btngr_temp.addButton(self.radio_exist_temp)
        self.btngr_temp.addButton(self.radio_new_temp)
        self.radio_exist_temp.setChecked(True)
        
        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.chk_del_material = QCheckBox("Delete Material", self.gb_matdata)
        self.chk_del_material.setSizePolicy(sizePolicy)
        self.chk_del_material.setMinimumSize(self.chk_size)
        self.chk_del_temp = QCheckBox("Delete Temperature", self.gb_matdata)
        self.chk_del_temp.setSizePolicy(sizePolicy)
        self.chk_del_temp.setMinimumSize(self.chk_size)
        
        self.gb_matdata_layout = QGridLayout(self.gb_matdata)
        self.gb_matdata_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gb_matdata_layout.addItem(self.h_spacer, 0, 0, 2, 1)
        self.gb_matdata_layout.addWidget(self.lb_new_val, 0, 2)
        self.gb_matdata_layout.addWidget(self.lb_material, 1, 0)
        self.gb_matdata_layout.addWidget(self.cb_material, 1, 1)
        self.gb_matdata_layout.addWidget(self.txt_material, 1, 2)
        self.gb_matdata_layout.addWidget(self.radio_exist_material,  1, 3)
        self.gb_matdata_layout.addWidget(self.radio_new_material,  1, 4)
        self.gb_matdata_layout.addWidget(self.chk_del_material,  1, 5)
        self.gb_matdata_layout.addWidget(self.lb_temp, 2, 0)
        self.gb_matdata_layout.addWidget(self.cb_temp, 2, 1)
        self.gb_matdata_layout.addWidget(self.txt_temp, 2, 2)
        self.gb_matdata_layout.addWidget(self.radio_exist_temp,  2, 3)
        self.gb_matdata_layout.addWidget(self.radio_new_temp,  2, 4)
        self.gb_matdata_layout.addWidget(self.chk_del_temp,  2, 5)
        self.gb_matdata_layout.addWidget(self.lb_dens, 3, 0)
        self.gb_matdata_layout.addItem(self.h_spacer, 3, 1)
        self.gb_matdata_layout.addWidget(self.txt_dens, 3, 2)
        self.gb_matdata_layout.addItem(self.h_spacer, 3, 3)
        self.gb_matdata_layout.addWidget(self.lb_elast, 4, 0)
        self.gb_matdata_layout.addItem(self.h_spacer, 4, 1)
        self.gb_matdata_layout.addWidget(self.txt_elast, 4, 2)
        self.gb_matdata_layout.addItem(self.h_spacer, 4, 3)
        self.gb_matdata_layout.addWidget(self.lb_stress, 5, 0)
        self.gb_matdata_layout.addItem(self.h_spacer, 5, 1)
        self.gb_matdata_layout.addWidget(self.txt_stress, 5, 2)
        self.gb_matdata_layout.addItem(self.h_spacer, 5, 3)
        self.gb_matdata_layout.addItem(self.h_spacer, 0, 6, 6, 1)
        self.gb_matdata_layout.addItem(self.v_spacer, 6, 0)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_status = QLabel("", self)
        self.lb_status.setTextFormat(Qt.RichText)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.btn_save = QPushButton(self.icon_save, "Save", self)
        size_policy.setHeightForWidth(self.btn_save.sizePolicy().hasHeightForWidth())
        self.btn_save.setSizePolicy(size_policy)
        self.btn_save.setMinimumSize(self.btn_size)
        self.btn_cancel = QPushButton(self.icon_cancel, "Cancel", self)
        size_policy.setHeightForWidth(self.btn_cancel.sizePolicy().hasHeightForWidth())
        self.btn_cancel.setSizePolicy(size_policy)
        self.btn_cancel.setMinimumSize(self.btn_size)

        self.hb_button_layout = QHBoxLayout()
        self.hb_button_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.hb_button_layout.addWidget(self.lb_status)
        self.hb_button_layout.addItem(self.h_spacer)
        self.hb_button_layout.addWidget(self.btn_save)
        self.hb_button_layout.addWidget(self.btn_cancel)
        
        self.main_layout = QVBoxLayout(self)
        self.main_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.main_layout.addWidget(self.gb_matdata)
        self.main_layout.addLayout(self.hb_button_layout)
        
        self.cb_material.addItems(self.GetMaterials())

        self.timer = QTimer(self)
        self.timer.setInterval(5000)
        
        self.connect(self.timer, SIGNAL("timeout()") , self.TimerHit)
        self.connect(self.btn_save, SIGNAL("clicked()"), self.OnBtnSaveClicked)
        self.connect(self.btn_cancel, SIGNAL("clicked()"), self.Close)
        self.connect(self.cb_material, SIGNAL("currentIndexChanged(int)"), self.OnMaterialCurrentIndexChanged)
        self.connect(self.cb_temp, SIGNAL("currentIndexChanged(int)"), self.OnTempCurrentIndexChanged)
        self.connect(self.radio_exist_material, SIGNAL("toggled(bool)"), self.OnExistMaterialToggled)
        self.connect(self.radio_new_material, SIGNAL("toggled(bool)"), self.OnNewMaterialToggled)
        self.connect(self.radio_exist_temp, SIGNAL("toggled(bool)"), self.OnExistTempToggled)
        self.connect(self.radio_new_temp, SIGNAL("toggled(bool)"), self.OnNewTempToggled)

    def GetMaterials(self):
        self.materials_xml = self.tree.parse(self.materialdata_file)
        self.materials = self.materials_xml.getiterator("material")
        matlist = QStringList("")
        for material in self.materials:
            matlist.append(material.attrib["name"])
        return matlist

    def OnMaterialCurrentIndexChanged(self):
        self.cb_temp.clear()
        self.ClearInputs()
        material = self.cb_material.currentText()
        if material != "":
            self.txt_material.setText(material)
            templist = self.GetTemp(material)
            self.cb_temp.addItems(templist)
        else:
            self.ClearInputs()

    def GetTemp(self,  matvar):
        templist = QStringList("")
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    templist.append(temp.text)
                return templist

    def OnTempCurrentIndexChanged(self):
        material = self.cb_material.currentText()
        temp = self.cb_temp.currentText()
        self.txt_temp.setText(temp)
        if temp != "":
            dens = self.GetMaterialData(material,  temp,  "dens")
            self.txt_dens.setText(dens)
            elast = self.GetMaterialData(material,  temp,  "elast")
            elast = str(float(elast) / 1000000000)
            self.txt_elast.setText(elast)
            stress = self.GetMaterialData(material,  temp,  "stress")
            stress = str(float(stress) / 1000000)
            self.txt_stress.setText(stress)
        else:
            self.txt_dens.setText("")
            self.txt_elast.setText("")
            self.txt_stress.setText("")

    def GetMaterialData(self,  matvar,  tempvar,  tag):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        value = temp.find(tag)
                        return value.text

    def OnExistMaterialToggled(self):
        self.lb_material.setText("Material")
        self.cb_material.show()
        self.cb_material.addItem("")
        self.cb_material.addItems(self.GetMaterials())
        self.cb_material.setCurrentIndex(0)
        self.chk_del_material.show()
        self.chk_del_material.setChecked(False)
        self.chk_del_temp.setChecked(False)
        self.radio_exist_temp.show()
        self.radio_exist_temp.setChecked(True)

    def OnNewMaterialToggled(self):
        self.lb_material.setText("New Material")
        self.cb_material.hide()
        self.cb_material.clear()
        self.chk_del_material.hide()
        self.chk_del_material.setChecked(False)
        self.chk_del_temp.setChecked(False)
        self.radio_exist_temp.hide()
        self.radio_new_temp.setChecked(True)
        self.ClearInputs()

    def OnExistTempToggled(self):
        self.lb_temp.setText("Temperature (&#x00B0; C)")
        self.cb_temp.show()
        self.OnMaterialCurrentIndexChanged()
        self.chk_del_temp.show()
        self.chk_del_temp.setChecked(False)
        self.ClearInputs()

    def OnNewTempToggled(self):
        self.lb_temp.setText("New Temperature (&#x00B0; C)")
        self.cb_temp.clear()
        self.cb_temp.hide()
        self.chk_del_temp.hide()
        self.chk_del_temp.setChecked(False)
        self.ClearInputs()
        self.txt_material.setText(self.cb_material.currentText())

    def OnBtnSaveClicked(self):
        try:
            if self.SaveChangesDlg():
                if not self.MakeDataBackup():
                        self.ShowStatus("Could not back up data file", "red")
                material = self.cb_material.currentText()
                new_material = self.txt_material.text()
                temp = self.cb_temp.currentText()
                new_temp = self.txt_temp.text()
                dens = self.txt_dens.text()
                elast = self.txt_elast.text()
                stress = self.txt_stress.text()
                if self.chk_del_material.isChecked() and material != "":
                    if self.DeleteMaterial(material):
                        self.Close()
                    else:
                        self.ShowStatus("Could not delete material and associated data", "red")
                elif self.chk_del_temp.isChecked() and temp != "":
                    if self.DeleteTemp(material,  temp):
                        self.Close()
                    else:
                        self.ShowStatus("Could not delete temperature and associated data", "red")
                elif new_material != "" and new_temp != "" and dens != "" and elast != "" and stress != "":
                    elast = str(float(elast) * 1000000000)
                    stress = str(float(stress) * 1000000)
                    if self.SaveData(material, new_material, temp, new_temp, dens, elast, stress):
                        self.Close()
                    else:
                        self.ShowStatus("Could not save data", "red")
                else:
                    self.ShowStatus("Could not save changes, some fields are empty", "red")
        except Exception, inst:
            QMessageBox.warning(self.parent, "Error - Cute Thermowell", "Unexpected error:\n %s" % (inst))

    def SaveChangesDlg(self):
        msg = "You are about to make changes to material data file.\n"
        msg += "A backup copy of the data file will be created.\n"
        msg += "Press OK to save changes or Cancel to return."
        dlg = QMessageBox(self)
        dlg.setIcon(QMessageBox.Warning)
        dlg.setWindowTitle("Save Changes - Cute Thermowell")
        dlg.setText(msg)
        dlg.addButton(QMessageBox.Ok)
        dlg.addButton(QMessageBox.Cancel)
        dlg.setDefaultButton(QMessageBox.Cancel)
        button = dlg.exec_()
        if (button == QMessageBox.Ok):
            return True
        else:
            return False        

    def MakeDataBackup(self):
        data_file_info = QFileInfo(self.materialdata_file)
        data_file_dir = self.script_path + "/data"
        data_file_base = data_file_info.completeBaseName()
        data_file_ext = data_file_info.completeSuffix()
        backup_file_ext = data_file_info.completeSuffix() + "~"
        data_file_name = data_file_dir + "/" + data_file_base + "." + data_file_ext
        backup_file_name = data_file_dir + "/" + data_file_base + "." + backup_file_ext
        data_file = QFile(data_file_name)
        backup_file = QFile(backup_file_name)
        if self.FileExists(data_file_name):
            if self.FileExists(backup_file_name):
                backup_file.remove()
                data_file.copy(backup_file_name)
                return True
            else:
                data_file.copy(backup_file_name)
                return True
        else:
            return False
        
    def FileExists(self, file_name):
        file = QFile(file_name)
        if (file.exists()):
            return True
        else:
            return False

    def ClearInputs(self):
        self.txt_material.setText("")
        self.txt_temp.setText("")
        self.txt_dens.setText("")
        self.txt_elast.setText("")
        self.txt_stress.setText("")
        self.lb_status.setText("<font color=\"blue\"></font>")

    def Close(self):
        self.close()

    def DeleteMaterial(self,  matvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                self.materials_xml.remove(material)
                self.SaveXML()
                return True

    def DeleteTemp(self,  matvar,  tempvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        material.remove(temp)
                        self.SaveXML()
                        return True

    def SaveData(self, matvar, new_matvar, tempvar, new_tempvar, densvar, elastvar, stressvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        temp.text = unicode(new_tempvar)
                        items = temp.getchildren()
                        items[0].text = unicode(densvar)
                        items[1].text =unicode(elastvar)
                        items[2].text = unicode(stressvar)
                        break
                    elif tempvar == "" and temp.text == new_tempvar:
                        temp.text = unicode(new_tempvar)
                        items = temp.getchildren()
                        items[0].text = unicode(densvar)
                        items[1].text =unicode(elastvar)
                        items[2].text = unicode(stressvar)
                        break
                    elif temps.index(temp) == len(temps) - 1 and temp.text != new_tempvar:
                        new_temp = SubElement(material, "temp")
                        new_temp.text = unicode(new_tempvar)
                        new_dens = SubElement(new_temp, "dens")
                        new_dens.text = unicode(densvar)
                        new_elast = SubElement(new_temp, "elast")
                        new_elast.text = unicode(elastvar)
                        new_stress = SubElement(new_temp, "stress")
                        new_stress.text = unicode(stressvar)
                material.attrib["name"] = unicode(new_matvar)
                self.SaveXML()
                return True
            elif matvar == "" and material.attrib["name"] == new_matvar:
                temps = material.getiterator("temp")
                for temp in temps:
                    if temp.text == tempvar:
                        temp.text = unicode(new_tempvar)
                        items = temp.getchildren()
                        items[0].text = unicode(densvar)
                        items[1].text =unicode(elastvar)
                        items[2].text = unicode(stressvar)
                        break
                    elif tempvar == "" and temp.text == new_tempvar:
                        temp.text = unicode(new_tempvar)
                        items = temp.getchildren()
                        items[0].text = unicode(densvar)
                        items[1].text =unicode(elastvar)
                        items[2].text = unicode(stressvar)
                        break
                    elif temps.index(temp) == len(temps) - 1 and temp.text != new_tempvar:
                        new_temp = SubElement(material, "temp")
                        new_temp.text = unicode(new_tempvar)
                        new_dens = SubElement(new_temp, "dens")
                        new_dens.text = unicode(densvar)
                        new_elast = SubElement(new_temp, "elast")
                        new_elast.text = unicode(elastvar)
                        new_stress = SubElement(new_temp, "stress")
                        new_stress.text = unicode(stressvar)
                material.attrib["name"] = unicode(new_matvar)
                self.SaveXML()
                return True
            elif self.materials.index(material) == len(self.materials) - 1 and material.attrib["name"] != new_matvar:
                new_material = Element("material", name = unicode(new_matvar))
                new_temp = SubElement(new_material, "temp")
                new_temp.text = unicode(new_tempvar)
                new_dens = SubElement(new_temp, "dens")
                new_dens.text = unicode(densvar)
                new_elast = SubElement(new_temp, "elast")
                new_elast.text = unicode(elastvar)
                new_stress = SubElement(new_temp, "stress")
                new_stress.text = unicode(stressvar)
                self.tree.getroot().append(new_material)
                self.SaveXML()
                return True
        return False
        
    def SaveXML(self):
        self.xmlIndent(self.tree.getroot())
        self.tree.write(self.materialdata_file, encoding="utf-8")
    
    """
    def Prettify(self, elem):
        ugly_string = ElementTree.tostring(elem, 'utf-8')
        pretty_string = minidom.parseString(ugly_string)
        return pretty_string.toprettyxml(indent="  ")
    """

    def xmlIndent(self, elem, level=0):
        i = "\n" + level*"  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self.xmlIndent(elem, level+1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i

    def closeEvent(self, event):
        self.ClearInputs()
        self.chk_del_material.setChecked(False)
        self.chk_del_temp.setChecked(False)
        self.radio_exist_material.setChecked(True)
        self.radio_exist_temp.setChecked(True)
        self.cb_material.clear()
        self.cb_material.addItems(self.GetMaterials())
        self.cb_material.setCurrentIndex(0)
        self.emit(SIGNAL("closed()"))

    def StartTimer(self):
        self.timer.start()
    
    def TimerHit(self):
        self.lb_status.setText("")

    def ShowStatus(self, msg,  color):
        self.lb_status.setText("<font color=\"" + color + "\">" + msg + "</font>")
        self.StartTimer()
