# -*- coding: utf-8 -*-
"""
Program:
    Cute Thermowell
    (qt-thermowell)
    PipeData.py

Author:
    Alex Borisov <>

Copyright (c) 2011 Alex Borisov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from xml.etree.ElementTree import ElementTree, Element, SubElement
import sys
import resources

class PipeDataDlg(QDialog):
    def __init__(self, pipedata_file,  parent = None):
        QDialog.__init__(self)

        self.parent = parent
        self.pipedata_file = pipedata_file
        self.script_path = sys.path[0]

        self.tree = ElementTree()

        self.PipeDataUI()

    def PipeDataUI(self):
        self.win_icon = QIcon(":/myresources/icons/pipe.png")
        self.setWindowIcon(self.win_icon)
        self.win_size = QSize(500, 200)
        self.setWindowTitle("Edit Pipe Data - Cute Thermowell")
        self.resize(self.win_size)

        self.icon_size = QSize(20, 20)
        self.btn_size = QSize(100, 25)
        self.l_size = QSize(160, 20)
        self.txt_size = QSize(130, 20)
        self.cb_size = QSize(130, 20)
        self.chk_size = QSize(150, 20)
        self.radio_size = QSize(80, 20)
        self.txt_max_length = 10

        self.dbl_validator = QDoubleValidator()
        self.dbl_validator.setNotation(QDoubleValidator.StandardNotation)

        self.h_spacer = QSpacerItem(2, 20, QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        self.v_spacer = QSpacerItem(20, 2, QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)

        self.icon_save = QIcon(":/myresources/icons/save.svg")
        self.icon_cancel = QIcon(":/myresources/icons/cancel.svg")

        self.gb_pipedata = QGroupBox("Pipe Data", self)
        self.gb_pipedata.setStyleSheet(QString.fromUtf8("font-size: 8pt;"))
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_new_val = QLabel("New / Edit Values",  self.gb_pipedata)
        size_policy.setHeightForWidth(self.lb_new_val.sizePolicy().hasHeightForWidth())
        self.lb_new_val.setAlignment(Qt.AlignHCenter)
        self.lb_new_val.setSizePolicy(size_policy)
        self.lb_new_val.setMinimumSize(self.l_size)
        self.lb_material = QLabel("Material", self.gb_pipedata)
        size_policy.setHeightForWidth(self.lb_material.sizePolicy().hasHeightForWidth())
        self.lb_material.setSizePolicy(size_policy)
        self.lb_material.setMinimumSize(self.l_size)
        self.lb_DN = QLabel("Nominal Diameter (mm)", self.gb_pipedata)
        size_policy.setHeightForWidth(self.lb_DN.sizePolicy().hasHeightForWidth())
        self.lb_DN.setSizePolicy(size_policy)
        self.lb_DN.setMinimumSize(self.l_size)
        self.lb_schedule = QLabel("Schedule",  self.gb_pipedata)
        size_policy.setHeightForWidth(self.lb_schedule.sizePolicy().hasHeightForWidth())
        self.lb_schedule.setSizePolicy(size_policy)
        self.lb_schedule.setMinimumSize(self.l_size)
        self.lb_OD = QLabel("Outside Diameter (mm)", self.gb_pipedata)
        size_policy.setHeightForWidth(self.lb_OD.sizePolicy().hasHeightForWidth())
        self.lb_OD.setSizePolicy(size_policy)
        self.lb_OD.setMinimumSize(self.l_size)
        self.lb_WT = QLabel("Wall Thickness (mm)", self.gb_pipedata)
        size_policy.setHeightForWidth(self.lb_WT.sizePolicy().hasHeightForWidth())
        self.lb_WT.setSizePolicy(size_policy)
        self.lb_WT.setMinimumSize(self.l_size)
        self.lb_ID = QLabel("Inside Diameter (mm)", self.gb_pipedata)
        size_policy.setHeightForWidth(self.lb_ID.sizePolicy().hasHeightForWidth())
        self.lb_ID.setSizePolicy(size_policy)
        self.lb_ID.setMinimumSize(self.l_size)
        self.lb_ID_value = QLabel("", self.gb_pipedata)
        size_policy.setHeightForWidth(self.lb_ID_value.sizePolicy().hasHeightForWidth())
        self.lb_ID_value.setSizePolicy(size_policy)
        self.lb_ID_value.setMinimumSize(self.l_size)
        self.lb_ID_value.setFrameStyle(QFrame.Panel)
        self.lb_ID_value.setLineWidth(1)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.txt_material = QLineEdit("", self.gb_pipedata)
        size_policy.setHeightForWidth(self.txt_material.sizePolicy().hasHeightForWidth())
        self.txt_material.setSizePolicy(size_policy)
        self.txt_material.setMinimumSize(self.txt_size)
        self.txt_material.setMaxLength(20)
        self.txt_DN = QLineEdit("", self.gb_pipedata)
        size_policy.setHeightForWidth(self.txt_DN.sizePolicy().hasHeightForWidth())
        self.txt_DN.setSizePolicy(size_policy)
        self.txt_DN.setMinimumSize(self.txt_size)
        self.txt_DN.setMaxLength(3)
        self.txt_DN.setValidator(self.dbl_validator)
        self.txt_schedule = QLineEdit("", self.gb_pipedata)
        size_policy.setHeightForWidth(self.txt_schedule.sizePolicy().hasHeightForWidth())
        self.txt_schedule.setSizePolicy(size_policy)
        self.txt_schedule.setMinimumSize(self.txt_size)
        self.txt_schedule.setMaxLength(20)
        self.txt_OD = QLineEdit("", self.gb_pipedata)
        size_policy.setHeightForWidth(self.txt_OD.sizePolicy().hasHeightForWidth())
        self.txt_OD.setSizePolicy(size_policy)
        self.txt_OD.setMinimumSize(self.txt_size)
        self.txt_OD.setMaxLength(self.txt_max_length)
        self.txt_OD.setValidator(self.dbl_validator)
        self.txt_WT = QLineEdit("", self.gb_pipedata)
        size_policy.setHeightForWidth(self.txt_WT.sizePolicy().hasHeightForWidth())
        self.txt_WT.setSizePolicy(size_policy)
        self.txt_WT.setMinimumSize(self.txt_size)
        self.txt_WT.setMaxLength(self.txt_max_length)
        self.txt_WT.setValidator(self.dbl_validator)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.cb_material = QComboBox(self.gb_pipedata)
        size_policy.setHeightForWidth(self.cb_material.sizePolicy().hasHeightForWidth())
        self.cb_material.setSizePolicy(size_policy)
        self.cb_material.setMinimumSize(self.cb_size)
        self.cb_DN = QComboBox(self.gb_pipedata)
        size_policy.setHeightForWidth(self.cb_DN.sizePolicy().hasHeightForWidth())
        self.cb_DN.setSizePolicy(size_policy)
        self.cb_DN.setMinimumSize(self.cb_size)
        self.cb_schedule = QComboBox(self.gb_pipedata)
        size_policy.setHeightForWidth(self.cb_schedule.sizePolicy().hasHeightForWidth())
        self.cb_schedule.setSizePolicy(size_policy)
        self.cb_schedule.setMinimumSize(self.cb_size)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.radio_exist_material = QRadioButton("Existing", self.gb_pipedata)
        self.radio_exist_material.setSizePolicy(size_policy)
        self.radio_exist_material.setMinimumSize(self.radio_size)
        self.radio_new_material = QRadioButton("New", self.gb_pipedata)
        self.radio_new_material.setSizePolicy(size_policy)
        self.radio_new_material.setMinimumSize(self.radio_size)
        self.radio_exist_DN = QRadioButton("Existing", self.gb_pipedata)
        self.radio_exist_DN.setSizePolicy(size_policy)
        self.radio_exist_DN.setMinimumSize(self.radio_size)
        self.radio_new_DN = QRadioButton("New", self.gb_pipedata)
        self.radio_new_DN.setSizePolicy(size_policy)
        self.radio_new_DN.setMinimumSize(self.radio_size)
        self.radio_exist_schedule = QRadioButton("Existing", self.gb_pipedata)
        self.radio_exist_schedule.setSizePolicy(size_policy)
        self.radio_exist_schedule.setMinimumSize(self.radio_size)
        self.radio_new_schedule = QRadioButton("New", self.gb_pipedata)
        self.radio_new_schedule.setSizePolicy(size_policy)
        self.radio_new_schedule.setMinimumSize(self.radio_size)
        
        self.btngr_material = QButtonGroup()
        self.btngr_material.addButton(self.radio_exist_material)
        self.btngr_material.addButton(self.radio_new_material)
        self.radio_exist_material.setChecked(True)
        self.btngr_DN = QButtonGroup()
        self.btngr_DN.addButton(self.radio_exist_DN)
        self.btngr_DN.addButton(self.radio_new_DN)
        self.radio_exist_DN.setChecked(True)
        self.btngr_schedule = QButtonGroup()
        self.btngr_schedule.addButton(self.radio_exist_schedule)
        self.btngr_schedule.addButton(self.radio_new_schedule)
        self.radio_exist_schedule.setChecked(True)
       
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.chk_del_material = QCheckBox("Delete Material", self.gb_pipedata)
        self.chk_del_material.setSizePolicy(size_policy)
        self.chk_del_material.setMinimumSize(self.chk_size)
        self.chk_del_DN = QCheckBox("Delete Nominal Diameter", self.gb_pipedata)
        self.chk_del_DN.setSizePolicy(size_policy)
        self.chk_del_DN.setMinimumSize(self.chk_size)
        self.chk_del_schedule = QCheckBox("Delete Schedule", self.gb_pipedata)
        self.chk_del_schedule.setSizePolicy(size_policy)
        self.chk_del_schedule.setMinimumSize(self.chk_size)
        
        self.gb_pipe_data_layout = QGridLayout(self.gb_pipedata)
        self.gb_pipe_data_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gb_pipe_data_layout.addItem(self.h_spacer, 0, 0, 2, 1)
        self.gb_pipe_data_layout.addWidget(self.lb_new_val, 0, 2)
        self.gb_pipe_data_layout.addWidget(self.lb_material, 1, 0)
        self.gb_pipe_data_layout.addWidget(self.cb_material, 1, 1)
        self.gb_pipe_data_layout.addWidget(self.txt_material, 1, 2)
        self.gb_pipe_data_layout.addWidget(self.radio_exist_material,  1, 3)
        self.gb_pipe_data_layout.addWidget(self.radio_new_material,  1, 4)
        self.gb_pipe_data_layout.addWidget(self.chk_del_material,  1, 5)
        self.gb_pipe_data_layout.addWidget(self.lb_DN, 2, 0)
        self.gb_pipe_data_layout.addWidget(self.cb_DN, 2, 1)
        self.gb_pipe_data_layout.addWidget(self.txt_DN, 2, 2)
        self.gb_pipe_data_layout.addWidget(self.radio_exist_DN,  2, 3)
        self.gb_pipe_data_layout.addWidget(self.radio_new_DN,  2, 4)
        self.gb_pipe_data_layout.addWidget(self.chk_del_DN,  2, 5)
        self.gb_pipe_data_layout.addWidget(self.lb_schedule, 3, 0)
        self.gb_pipe_data_layout.addWidget(self.cb_schedule, 3, 1)
        self.gb_pipe_data_layout.addWidget(self.txt_schedule, 3, 2)
        self.gb_pipe_data_layout.addWidget(self.radio_exist_schedule,  3, 3)
        self.gb_pipe_data_layout.addWidget(self.radio_new_schedule,  3, 4)
        self.gb_pipe_data_layout.addWidget(self.chk_del_schedule,  3, 5)
        self.gb_pipe_data_layout.addWidget(self.lb_OD, 4, 0)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 4, 1)
        self.gb_pipe_data_layout.addWidget(self.txt_OD, 4, 2)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 4, 3)
        self.gb_pipe_data_layout.addWidget(self.lb_WT, 5, 0)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 5, 1)
        self.gb_pipe_data_layout.addWidget(self.txt_WT, 5, 2)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 5, 3)
        self.gb_pipe_data_layout.addWidget(self.lb_ID, 6, 0)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 6, 1)
        self.gb_pipe_data_layout.addWidget(self.lb_ID_value, 6, 2)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 6, 3)
        self.gb_pipe_data_layout.addItem(self.h_spacer, 0, 6, 7, 1)
        self.gb_pipe_data_layout.addItem(self.v_spacer, 7, 0)

        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_status = QLabel("", self)
        self.lb_status.setTextFormat(Qt.RichText)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.btn_save = QPushButton(self.icon_save, "Save", self)
        size_policy.setHeightForWidth(self.btn_save.sizePolicy().hasHeightForWidth())
        self.btn_save.setSizePolicy(size_policy)
        self.btn_save.setMinimumSize(self.btn_size)
        self.btn_cancel = QPushButton(self.icon_cancel, "Cancel", self)
        size_policy.setHeightForWidth(self.btn_cancel.sizePolicy().hasHeightForWidth())
        self.btn_cancel.setSizePolicy(size_policy)
        self.btn_cancel.setMinimumSize(self.btn_size)

        self.hb_button_layout = QHBoxLayout()
        self.hb_button_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.hb_button_layout.addWidget(self.lb_status)
        self.hb_button_layout.addItem(self.h_spacer)
        self.hb_button_layout.addWidget(self.btn_save)
        self.hb_button_layout.addWidget(self.btn_cancel)
        
        self.main_layout = QVBoxLayout(self)
        self.main_layout.sizeConstraint = QLayout.SetDefaultConstraint
        self.main_layout.addWidget(self.gb_pipedata)
        self.main_layout.addLayout(self.hb_button_layout)
        
        self.cb_material.addItems(self.GetMaterials())

        self.timer = QTimer(self)
        self.timer.setInterval(5000)
        
        self.connect(self.timer, SIGNAL("timeout()") , self.TimerHit)
        self.connect(self.btn_save, SIGNAL("clicked()"), self.OnBtnSaveClicked)
        self.connect(self.btn_cancel, SIGNAL("clicked()"), self.Close)
        self.connect(self.cb_material, SIGNAL("currentIndexChanged(int)"), self.OnMaterialCurrentIndexChanged)
        self.connect(self.cb_DN, SIGNAL("currentIndexChanged(int)"), self.OnDNCurrentIndexChanged)
        self.connect(self.cb_schedule, SIGNAL("currentIndexChanged(int)"), self.OnScheduleCurrentIndexChanged)
        self.connect(self.radio_exist_material, SIGNAL("toggled(bool)"), self.OnExistMaterialToggled)
        self.connect(self.radio_new_material, SIGNAL("toggled(bool)"), self.OnNewMaterialToggled)
        self.connect(self.radio_exist_DN, SIGNAL("toggled(bool)"), self.OnExistDNToggled)
        self.connect(self.radio_new_DN, SIGNAL("toggled(bool)"), self.OnNewDNToggled)
        self.connect(self.radio_exist_schedule, SIGNAL("toggled(bool)"), self.OnExistScheduleToggled)
        self.connect(self.radio_new_schedule, SIGNAL("toggled(bool)"), self.OnNewScheduleToggled)
        self.connect(self.txt_OD, SIGNAL("textChanged(QString)"), self.SetID)
        self.connect(self.txt_WT, SIGNAL("textChanged(QString)"), self.SetID)

    def GetMaterials(self):
        matlist = QStringList("")
        self.pipes_xml = self.tree.parse(self.pipedata_file)
        self.materials = self.pipes_xml.getiterator("material")
        for material in self.materials:
            matlist.append(material.attrib["name"])
        return matlist

    def OnMaterialCurrentIndexChanged(self):
        self.cb_DN.clear()
        self.ClearInputs()
        material = self.cb_material.currentText()
        if material != "":
            self.txt_material.setText(material)
            dnlist = self.GetDN(material)
            self.cb_DN.addItems(dnlist)
        else:
            self.ClearInputs()

    def GetDN(self,  matvar):
        dnlist = QStringList("")
        for material in self.materials:
            if material.attrib["name"] == matvar:
                dns = material.getiterator("DN")
                for dn in dns:
                    dnlist.append(dn.text)
                return dnlist

    def OnDNCurrentIndexChanged(self):
        self.cb_schedule.clear()
        material = self.cb_material.currentText()
        dn = self.cb_DN.currentText()
        self.txt_DN.setText(dn)
        if material != "" and dn != "":
            schedulelist = self.GetSchedule(material,  dn)
            self.cb_schedule.addItems(schedulelist)
        else:
            self.cb_schedule.clear()

    def GetSchedule(self,  matvar,  dnvar):
        schedulelist = QStringList("")
        for material in self.materials:
            if material.attrib["name"] == matvar:
                dns = material.getiterator("DN")
                for dn in dns:
                    if dn.text == dnvar:
                        schedules = dn.getiterator("schedule")
                        for schedule in schedules:
                            schedulelist.append(schedule.text)
                        return schedulelist

    def OnScheduleCurrentIndexChanged(self):
        material = self.cb_material.currentText()
        dn = self.cb_DN.currentText()
        schedule = self.cb_schedule.currentText()
        self.txt_schedule.setText(schedule)
        if schedule != "":
            od = self.GetPipeData(material,  dn,  schedule,  "OD")
            od = str(float(od) * 1000)
            self.txt_OD.setText(od)
            wt = self.GetPipeData(material,  dn,  schedule,  "WT")
            wt = str(float(wt) * 1000)
            self.txt_WT.setText(wt)
            id = str(float(od) - 2 * float(wt))
            self.lb_ID_value.setText(id)
        else:
            self.txt_OD.setText("")
            self.txt_WT.setText("")
            self.lb_ID_value.setText("")

    def GetPipeData(self,  matvar,  dnvar,  schedulevar,  tag):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                dns = material.getiterator("DN")
                for dn in dns:
                    if dn.text == dnvar:
                        schedules = dn.getiterator("schedule")
                        for schedule in schedules:
                            if schedule.text == schedulevar:
                                value = schedule.find(tag)
                                return value.text

    def OnExistMaterialToggled(self):
        self.lb_material.setText("Material")
        self.cb_material.show()
        self.cb_material.addItem("")
        self.cb_material.addItems(self.GetMaterials())
        self.cb_material.setCurrentIndex(0)
        self.chk_del_material.show()
        self.chk_del_material.setChecked(False)
        self.chk_del_DN.setChecked(False)
        self.radio_exist_DN.show()
        self.radio_exist_DN.setChecked(True)

    def OnNewMaterialToggled(self):
        self.lb_material.setText("New Material")
        self.cb_material.hide()
        self.cb_material.clear()
        self.chk_del_material.hide()
        self.chk_del_material.setChecked(False)
        self.chk_del_DN.setChecked(False)
        self.radio_exist_DN.hide()
        self.radio_new_DN.setChecked(True)
        self.ClearInputs()

    def OnExistDNToggled(self):
        self.lb_DN.setText("Nominal Diameter (mm)")
        self.cb_DN.show()
        self.OnMaterialCurrentIndexChanged()
        self.chk_del_DN.show()
        self.chk_del_DN.setChecked(False)
        self.chk_del_schedule.setChecked(False)
        self.radio_exist_schedule.show()
        self.radio_exist_schedule.setChecked(True)

    def OnNewDNToggled(self):
        self.lb_DN.setText("New Nominal Diameter (mm)")
        self.cb_DN.clear()
        self.cb_DN.hide()
        self.chk_del_DN.hide()
        self.chk_del_DN.setChecked(False)
        self.chk_del_schedule.setChecked(False)
        self.radio_exist_schedule.hide()
        self.radio_new_schedule.setChecked(True)

    def OnExistScheduleToggled(self):
        self.lb_schedule.setText("Schedule")
        self.cb_schedule.show()
        self.OnDNCurrentIndexChanged()
        self.chk_del_schedule.show()
        self.chk_del_schedule.setChecked(False)

    def OnNewScheduleToggled(self):
        self.lb_schedule.setText("New Schedule")
        self.cb_schedule.clear()
        self.cb_schedule.hide()
        self.chk_del_schedule.hide()
        self.chk_del_schedule.setChecked(False)

    def OnBtnSaveClicked(self):
        try:
            if self.SaveChangesDlg():
                if not self.MakeDataBackup():
                        self.ShowStatus("Could not back up data file", "red")
                material = self.cb_material.currentText()
                new_material = self.txt_material.text()
                dn = self.cb_DN.currentText()
                new_dn = self.txt_DN.text()
                sch = self.cb_schedule.currentText()
                new_sch = self.txt_schedule.text()
                od = self.txt_OD.text()
                wt = self.txt_WT.text()
                id = self.lb_ID_value.text()
                if self.chk_del_material.isChecked() and material != "":
                    if self.DeleteMaterial(material):
                        self.Close()
                    else:
                        self.ShowStatus("Could not delete material and associated data", "red")
                elif self.chk_del_DN.isChecked() and dn != "":
                    if self.DeleteDN(material,  dn):
                        self.Close()
                    else:
                        self.ShowStatus("Could not delete nominal diameter and associated data", "red")
                elif self.chk_del_schedule.isChecked() and sch != "":
                    if self.DeleteSchedule(material,  dn,  sch):
                        self.Close()
                    else:
                        self.ShowStatus("Could not delete schedule and associated data", "red")
                elif new_material != "" and new_dn != "" and new_sch != "" and od != "" and wt != "":
                    od = str(float(od) / 1000)
                    wt = str(float(wt) / 1000)
                    id = str(float(id) / 1000)
                    if self.SaveData(material, new_material, dn, new_dn, sch, new_sch, od,  wt,  id):
                        self.Close()
                    else:
                        self.ShowStatus("Could not save data", "red")
                else:
                    self.ShowStatus("Could not save changes, some fields are empty", "red")
        except Exception, inst:
            QMessageBox.warning(self.parent, "Error - Cute Thermowell", "Unexpected error:\n %s" % (inst))

    def SaveChangesDlg(self):
        msg = "You are about to make changes to pipe data file.\n"
        msg += "A backup copy of the data file will be created.\n"
        msg += "Press OK to save changes or Cancel to return."
        dlg = QMessageBox(self)
        dlg.setIcon(QMessageBox.Warning)
        dlg.setWindowTitle("Save Changes - Cute Thermowell")
        dlg.setText(msg)
        dlg.addButton(QMessageBox.Ok)
        dlg.addButton(QMessageBox.Cancel)
        dlg.setDefaultButton(QMessageBox.Cancel)
        button = dlg.exec_()
        if (button == QMessageBox.Ok):
            return True
        else:
            return False        

    def MakeDataBackup(self):
        data_file_info = QFileInfo(self.pipedata_file)
        data_file_dir = self.script_path + "/data"
        QMessageBox.about(self.parent,  "",  data_file_dir)
        data_file_base = data_file_info.completeBaseName()
        data_file_ext = data_file_info.completeSuffix()
        backup_file_ext = data_file_info.completeSuffix() + "~"
        data_file_name = data_file_dir + "/" + data_file_base + "." + data_file_ext
        backup_file_name = data_file_dir + "/" + data_file_base + "." + backup_file_ext
        data_file = QFile(data_file_name)
        backup_file = QFile(backup_file_name)
        if self.FileExists(data_file_name):
            if self.FileExists(backup_file_name):
                backup_file.remove()
                data_file.copy(backup_file_name)
                return True
            else:
                data_file.copy(backup_file_name)
                return True
        else:
            return False
        
    def FileExists(self, file_name):
        file = QFile(file_name)
        if (file.exists()):
            return True
        else:
            return False

    def ClearInputs(self):
        self.txt_material.setText("")
        self.txt_DN.setText("")
        self.txt_schedule.setText("")
        self.txt_OD.setText("")
        self.txt_WT.setText("")
        self.lb_ID_value.setText("")
        self.lb_status.setText("<font color=\"blue\"></font>")

    def SetID(self):
        od = self.txt_OD.text()
        wt = self.txt_WT.text()
        if od != "" and wt != "":
            id = str(float(od) - 2 * float(wt))
            self.lb_ID_value.setText(id)
        else:
            self.lb_ID_value.setText("")

    def Close(self):
        self.close()

    def DeleteMaterial(self,  matvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                self.pipes_xml.remove(material)
                self.SaveXML()
                return True

    def DeleteDN(self,  matvar,  dnvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                dns = material.getiterator("DN")
                for dn in dns:
                    if dn.text == dnvar:
                        material.remove(dn)
                        self.SaveXML()
                        return True

    def DeleteSchedule(self,  matvar,  dnvar,  schvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                dns = material.getiterator("DN")
                for dn in dns:
                    if dn.text == dnvar:
                        schs = dn.getiterator("schedule")
                        for sch in schs:
                            if sch.text == schvar:
                                dn.remove(sch)
                                self.SaveXML()
                                return True

    def SaveData(self, matvar, new_matvar, dnvar, new_dnvar, schvar, new_schvar, odvar,  wtvar,  idvar):
        for material in self.materials:
            if material.attrib["name"] == matvar:
                dns = material.getiterator("DN")
                for dn in dns:
                    if dn.text == dnvar:
                        schs = dn.getiterator("schedule")
                        for sch in schs:
                            if sch.text == schvar:
                                sch.text = unicode(new_schvar)
                                items = sch.getchildren()
                                items[0].text = unicode(odvar)
                                items[1].text = unicode(wtvar)
                                items[2].text = unicode(idvar)
                                break
                            elif schvar == "" and sch.text == new_schvar:
                                sch.text = unicode(new_schvar)
                                items = sch.getchildren()
                                items[0].text = unicode(odvar)
                                items[1].text = unicode(wtvar)
                                items[2].text = unicode(idvar)
                                break
                            elif schs.index(sch) == len(schs) - 1 and sch.text != new_schvar:
                                new_sch = SubElement(dn, "schedule")
                                new_sch.text = unicode(new_schvar)
                                new_od = SubElement(new_sch, "OD")
                                new_od.text = unicode(odvar)
                                new_wt = SubElement(new_sch, "WT")
                                new_wt.text = unicode(wtvar)
                                new_id = SubElement(new_sch, "ID")
                                new_id.text = unicode(idvar)
                        dn.text = unicode(new_dnvar)
                        material.attrib["name"] = unicode(new_matvar)
                        self.SaveXML()
                        return True
                    elif dnvar == "" and dn.text == new_dnvar:
                        schs = temp.getiterator("schedule")
                        for sch in schs:
                            if sch.text == schvar:
                                sch.text = unicode(new_schvar)
                                items = sch.getchildren()
                                items[0].text = unicode(odvar)
                                items[1].text = unicode(wtvar)
                                items[2].text = unicode(idvar)
                                break
                            elif schvar == "" and sch.text == new_schvar:
                                sch.text = unicode(new_schvar)
                                items = sch.getchildren()
                                items[0].text = unicode(odvar)
                                items[1].text = unicode(wtvar)
                                items[2].text = unicode(idvar)
                                break
                            elif schs.index(sch) == len(schs) - 1 and sch.text != new_schvar:
                                new_sch = SubElement(dn, "schedule")
                                new_sch.text = unicode(new_schvar)
                                new_od = SubElement(new_sch, "OD")
                                new_od.text = unicode(odvar)
                                new_wt = SubElement(new_sch, "WT")
                                new_wt.text = unicode(wtvar)
                                new_id = SubElement(new_sch, "ID")
                                new_id.text = unicode(idvar)
                        dn.text = unicode(new_dnvar)
                        material.attrib["name"] = unicode(new_matvar)
                        self.SaveXML()
                        return True
                    elif dns.index(dn) == len(dns) - 1 and dn.text != new_dnvar:
                        new_dn = SubElement(material, "DN")
                        new_dn.text = unicode(new_dnvar)
                        new_sch = SubElement(new_dn, "schedule")
                        new_sch.text = unicode(new_schvar)
                        new_od = SubElement(new_sch, "OD")
                        new_od.text = unicode(odvar)
                        new_wt = SubElement(new_sch, "WT")
                        new_wt.text = unicode(wtvar)
                        new_id = SubElement(new_sch, "ID")
                        new_id.text = unicode(idvar)
                material.attrib["name"] = unicode(new_matvar)
                self.SaveXML()
                return True
            elif matvar == "" and material.attrib["name"] == new_matvar:
                dns = material.getiterator("DN")
                for dn in dns:
                    if dn.text == dnvar:
                        schs = dn.getiterator("schedule")
                        for sch in schs:
                            if sch.text == schvar:
                                sch.text = unicode(new_schvar)
                                items = sch.getchildren()
                                items[0].text = unicode(odvar)
                                items[1].text = unicode(wtvar)
                                items[2].text = unicode(idvar)
                                break
                            elif schvar == "" and sch.text == new_schvar:
                                sch.text = unicode(new_schvar)
                                items = sch.getchildren()
                                items[0].text = unicode(odvar)
                                items[1].text = unicode(wtvar)
                                items[2].text = unicode(idvar)
                                break
                            elif schs.index(sch) == len(schs) - 1 and sch.text != new_schvar:
                                new_sch = SubElement(dn, "schedule")
                                new_sch.text = unicode(new_schvar)
                                new_od = SubElement(new_sch, "OD")
                                new_od.text = unicode(odvar)
                                new_wt = SubElement(new_sch, "WT")
                                new_wt.text = unicode(wtvar)
                                new_id = SubElement(new_sch, "ID")
                                new_id.text = unicode(idvar)
                        dn.text = unicode(new_dnvar)
                        material.attrib["name"] = unicode(new_matvar)
                        self.SaveXML()
                        return True
                    elif dnvar == "" and dn.text == new_dnvar:
                        schs = temp.getiterator("schedule")
                        for sch in schs:
                            if sch.text == schvar:
                                sch.text = unicode(new_schvar)
                                items = sch.getchildren()
                                items[0].text = unicode(odvar)
                                items[1].text = unicode(wtvar)
                                items[2].text = unicode(idvar)
                                break
                            elif schvar == "" and sch.text == new_schvar:
                                sch.text = unicode(new_schvar)
                                items = sch.getchildren()
                                items[0].text = unicode(odvar)
                                items[1].text = unicode(wtvar)
                                items[2].text = unicode(idvar)
                                break
                            elif schs.index(sch) == len(schs) - 1 and sch.text != new_schvar:
                                new_sch = SubElement(dn, "schedule")
                                new_sch.text = unicode(new_schvar)
                                new_od = SubElement(new_sch, "OD")
                                new_od.text = unicode(odvar)
                                new_wt = SubElement(new_sch, "WT")
                                new_wt.text = unicode(wtvar)
                                new_id = SubElement(new_sch, "ID")
                                new_id.text = unicode(idvar)
                        dn.text = unicode(new_dnvar)
                        material.attrib["name"] = unicode(new_matvar)
                        self.SaveXML()
                        return True
                    elif dns.index(dn) == len(dns) - 1 and dn.text != new_dnvar:
                        new_dn = SubElement(material, "DN")
                        new_dn.text = unicode(new_dnvar)
                        new_sch = SubElement(new_dn, "schedule")
                        new_sch.text = unicode(new_schvar)
                        new_od = SubElement(new_sch, "OD")
                        new_od.text = unicode(odvar)
                        new_wt = SubElement(new_sch, "WT")
                        new_wt.text = unicode(wtvar)
                        new_id = SubElement(new_sch, "ID")
                        new_id.text = unicode(idvar)
                material.attrib["name"] = unicode(new_matvar)
                self.SaveXML()
                return True
            elif self.materials.index(material) == len(self.materials) - 1 and material.attrib["name"] != new_matvar:
                new_material = Element("material", name = unicode(new_matvar))
                new_dn = SubElement(new_material, "DN")
                new_dn.text = unicode(new_dnvar)
                new_sch = SubElement(new_dn, "schedule")
                new_sch.text = unicode(new_schvar)
                new_od = SubElement(new_sch, "OD")
                new_od.text = unicode(odvar)
                new_wt = SubElement(new_sch, "WT")
                new_wt.text = unicode(wtvar)
                new_id = SubElement(new_sch, "ID")
                new_id.text = unicode(idvar)
                self.tree.getroot().append(new_material)
                self.SaveXML()
                return True
        return False
        
    def SaveXML(self):
        self.xmlIndent(self.tree.getroot())
        self.tree.write(self.pipedata_file, encoding="utf-8")
    
    """
    def Prettify(self, elem):
        ugly_string = ElementTree.tostring(elem, 'utf-8')
        pretty_string = minidom.parseString(ugly_string)
        return pretty_string.toprettyxml(indent="  ")
    """

    def xmlIndent(self, elem, level=0):
        i = "\n" + level*"  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self.xmlIndent(elem, level+1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i

    def closeEvent(self, event):
        self.ClearInputs()
        self.chk_del_material.setChecked(False)
        self.chk_del_DN.setChecked(False)
        self.chk_del_schedule.setChecked(False)
        self.radio_exist_material.setChecked(True)
        self.radio_exist_DN.setChecked(True)
        self.radio_exist_schedule.setChecked(True)
        self.cb_material.clear()
        self.cb_material.addItems(self.GetMaterials())
        self.cb_material.setCurrentIndex(0)
        self.emit(SIGNAL("closed()"))

    def StartTimer(self):
        self.timer.start()
    
    def TimerHit(self):
        self.lb_status.setText("")

    def ShowStatus(self, msg,  color):
        self.lb_status.setText("<font color=\"" + color + "\">" + msg + "</font>")
        self.StartTimer()
