# -*- coding: utf-8 -*-
"""
Program:
    Cute Converter
    (qt-converter)
    Converter.py

Author:
    Alex Borisov <>

Copyright (c) 2010 Alex Borisov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from decimal import Decimal
from xml.etree.ElementTree import ElementTree, Element, SubElement
#from xml.dom import minidom

class Converter(object):
    def __init__(self, xmlFile = None, parent = None):
        self.parent=parent
        self.xmlFile = xmlFile
        self.tree = ElementTree()

    def GetProperties(self):
        self.xmlDoc = self.tree.parse(self.xmlFile)
        self.lstProperty = self.xmlDoc.getiterator("property")
        propertyList = QStringList("")
        for property in self.lstProperty:
            propertyList.append(property.attrib["name"])
        return propertyList

    def GetUnits(self, strProperty):
        for property in self.lstProperty:
            if property.attrib["name"] == strProperty:
                lstUnits = property.getiterator("unitname")
                unitList = QStringList()
                for unit in lstUnits:
                    unitList.append(unit.text)
                return unitList

    def Convert(self, strProperty, strFrom, strTo, strInput):
        if strProperty == "Temperature":
            return self.ConvertTemp(strFrom, strTo, strInput)
        else:
            xmlDataFrom = self.GetXMLData(strProperty, strFrom)
            xmlDataTo = self.GetXMLData(strProperty, strTo)
            xmlFromUnit = xmlDataFrom[0].text
            xmlFromFactor = xmlDataFrom[1].text
            xmlFromE = xmlDataFrom[2].text
            xmlFromUnicode = xmlDataFrom[3].text
            xmlToUnit = xmlDataTo[0].text
            xmlToFactor = xmlDataTo[1].text
            xmlToE = xmlDataTo[2].text
            xmlToUnicode = xmlDataTo[3].text
            fromFactor = Decimal(xmlFromFactor) * Decimal(pow(10,Decimal(xmlFromE)))
            toFactor = Decimal(xmlToFactor) * Decimal(pow(10,Decimal(xmlToE)))
            result = Decimal(str(strInput)) * fromFactor / toFactor
            return str(result)

    def ConvertTemp(self, strFrom, strTo, strInput):
        input = float(str(strInput))
        output = 0

        if strFrom == "kelvin":
            input *= 1
        elif strFrom == "degree Celsius":
            input += 273.15
        elif strFrom == "degree centigrade":
            input += 273.15
        elif strFrom == "degree Fahrenheit":
            input = (input + 459.67) / 1.8
        elif strFrom == "degree Rankine":
            input /= 1.8

        if strTo == "kelvin":
            output = input
        elif strTo == "degree Celsius":
            output = input - 273.15
        elif strTo == "degree centigrade":
            output = input - 273.15
        elif strTo == "degree Fahrenheit":
            output = (input  * 1.8) - 459.67
        elif strTo == "degree Rankine":
            output = input * 1.8

        return str(output)

    def UnicodeToString(self, strProperty, strUnit):
        for property in self.lstProperty:
            if property.attrib["name"] == strProperty:
                lstUnits = property.getiterator("unit")
                for units in lstUnits:
                    unit = units.getchildren()
                    if unit[0].text == strUnit:
                        return unit[3].text

    def GetXMLData(self, strProperty, strUnit):
        for property in self.lstProperty:
            if property.attrib["name"] == strProperty:
                lstUnits = property.getiterator("unit")
                for units in lstUnits:
                    unit = units.getchildren()
                    if unit[0].text == strUnit:
                        return unit

    def SetDecimals(self, strValue, decimals):
        rndValue = round(Decimal(str(strValue)), decimals)
        return str(rndValue)

    def StatusString(self, strProperty, strFrom, strTo, strInput, strOutput):
        if strProperty == "Temperature":
            return strProperty
        else:
            xmlDataFrom = self.GetXMLData(strProperty, strFrom)
            xmlDataTo = self.GetXMLData(strProperty, strTo)
            fromFactor = xmlDataFrom[1].text
            toFactor = xmlDataTo[1].text
            fromE = xmlDataFrom[2].text
            toE = xmlDataTo[2].text
            factor = round(Decimal(fromFactor) / Decimal(toFactor), 6)
            E = Decimal(fromE) - Decimal(toE)
            statusString = strProperty + ": 1 " + strInput + " = " + str(factor) + "E" + str(E) + " " + strOutput
            return statusString

    def DeleteProperty(self, strProperty):
        for property in self.lstProperty:
            if property.attrib["name"] == strProperty:
                self.xmlDoc.remove(property)
                self.SaveXML()
                return True

    def DeleteUnit(self, strProperty, strUnit):
        for property in self.lstProperty:
            if property.attrib["name"] == strProperty:
                lstUnits = property.getiterator("unit")
                for units in lstUnits:
                    unit = units.getchildren()
                    if unit[0].text == strUnit:
                        property.remove(units)
                        self.SaveXML()
                        return True
        
    def SaveUnits(self, strProperty, strNewProperty, strFromUnit, strNewFromUnit, strToUnit, strNewToUnit, strNewFromUnicode, strNewToUnicode, strNewToFactor, strNewPower):
        for property in self.lstProperty:
            if property.attrib["name"] == strProperty:
                lstUnits = property.getiterator("unit")
                for units in lstUnits:
                    toUnit = units.getchildren()
                    if toUnit[0].text == strToUnit:
                        toUnit[0].text = strNewToUnit
                        toUnitFactor = toUnit[1].text
                        toUnitE = toUnit[2].text
                        toUnit[3].text = strNewToUnicode
                        break
                    elif lstUnits.index(units) == len(lstUnits) - 1 and toUnit[0].text != strToUnit:
                        return False
                lstUnits = property.getiterator("unit")
                for units in lstUnits:
                    fromUnit = units.getchildren()
                    if fromUnit[0].text == strFromUnit:
                        fromUnit[0].text = strNewFromUnit
                        fromUnit[1].text = str(Decimal(toUnitFactor) * Decimal(strNewToFactor))
                        fromUnit[2].text = str(Decimal(toUnitE) + Decimal(strNewPower))
                        fromUnit[3].text = strNewFromUnicode
                        break
                    elif lstUnits.index(units) == len(lstUnits) - 1 and fromUnit[0].text != strFromUnit:
                        newFromUnit = SubElement(property, "unit")
                        newFromUnitname = SubElement(newFromUnit, "unitname")
                        newFromUnitname.text = strNewFromUnit
                        newFromFactor = SubElement(newFromUnit, "factor")
                        newFromFactor.text = str(Decimal(toUnitFactor) * Decimal(strNewToFactor))
                        newFromE = SubElement(newFromUnit, "E")
                        newFromE.text = str(Decimal(toUnitE) + Decimal(strNewPower))
                        newFromUnicode = SubElement(newFromUnit, "unicode")
                        newFromUnicode.text = strNewFromUnicode
                property.attrib["name"] = strNewProperty
                self.SaveXML()
                return True
            elif self.lstProperty.index(property) == len(self.lstProperty) - 1:
                newProperty = Element("property", name = strNewProperty)
                newToUnit = SubElement(newProperty, "unit")
                newToUnitname = SubElement(newToUnit, "unitname")
                newToUnitname.text = strNewToUnit
                newToFactor = SubElement(newToUnit, "factor")
                newToFactor.text = "1"
                newToE = SubElement(newToUnit, "E")
                newToE.text = "0"
                newToUnicode = SubElement(newToUnit, "unicode")
                newToUnicode.text = strNewToUnicode
                newFromUnit = SubElement(newProperty, "unit")
                newFromUnitname = SubElement(newFromUnit, "unitname")
                newFromUnitname.text = strNewFromUnit
                newFromFactor = SubElement(newFromUnit, "factor")
                newFromFactor.text = str(Decimal(newToFactor.text) * Decimal(strNewToFactor))
                newFromE = SubElement(newFromUnit, "E")
                newFromE.text = str(Decimal(newToE.text) + Decimal(strNewPower))
                newFromUnicode = SubElement(newFromUnit, "unicode")
                newFromUnicode.text = strNewFromUnicode
                self.tree.getroot().append(newProperty)
                self.SaveXML()
                return True
        return False

    def SaveXML(self):
        self.xmlIndent(self.tree.getroot())
        self.tree.write(self.xmlFile, encoding="utf-8")

    def xmlIndent(self, elem, level=0):
        i = "\n" + level*"  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self.xmlIndent(elem, level+1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i
