# -*- coding: utf-8 -*-
"""
Program:
    Cute Converter
    (qt-converter)
    CustomUnits.py

Author:
    Alex Borisov <>

Copyright (c) 2010 Alex Borisov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from Converter import Converter
from UnicodeSymbols import UnicodeSymbols
from decimal import Decimal
from xml.etree.ElementTree import ElementTree
import sys
import resources

class CustomWindow(QDialog):
    def __init__(self, convert = None, parent = None):
        QDialog.__init__(self)

        self.convert = convert
        self.usymbols = None
        self.script_path = sys.path[0]
        self.CustomUI()

    def CustomUI(self):
        self.winIcon = QIcon(":/myresources/icons/converter.ico")
        self.setWindowIcon(self.winIcon)
        self.winSize = QSize(300, 100)
        self.setWindowTitle("Custom Units - Cute Converter")
        self.resize(self.winSize)

        self.dbl_validator = QDoubleValidator()
        self.dbl_validator.setNotation(QDoubleValidator.StandardNotation)

        self.hSpacer = QSpacerItem(2, 20, QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        self.vVpacer = QSpacerItem(20, 2, QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)

        self.lSize = QSize(200, 25)
        self.txtSize = QSize(240, 25)
        self.btnSize = QSize(100, 25)
        self.cbSize = QSize(240, 25)
        self.radioSize = QSize(140, 25)
        self.chkSize = QSize(200, 25)

        self.iconSave = QIcon(":/myresources/icons/save.svg")
        self.iconCancel = QIcon(":/myresources/icons/cancel.svg")

        self.gbProperty = QGroupBox("", self)
        self.gbUnits = QGroupBox("", self)
        self.gbResult = QGroupBox("", self)
        
        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.lDummy = QLabel("", self.gbUnits)
        sizePolicy.setHeightForWidth(self.lDummy.sizePolicy().hasHeightForWidth())
        self.lDummy.setSizePolicy(sizePolicy)
        self.lDummy.setMinimumWidth(94)
        self.lUnitFrom = QLabel("From Unit", self.gbUnits)
        sizePolicy.setHeightForWidth(self.lUnitFrom.sizePolicy().hasHeightForWidth())
        self.lUnitFrom.setSizePolicy(sizePolicy)
        self.lUnitFrom.setMinimumWidth(94)
        self.lUnitTo = QLabel("To Unit", self.gbUnits)
        sizePolicy.setHeightForWidth(self.lUnitTo.sizePolicy().hasHeightForWidth())
        self.lUnitTo.setSizePolicy(sizePolicy)
        self.lUnitTo.setMinimumWidth(94)
        self.lProperty = QLabel("Property", self.gbResult)
        sizePolicy.setHeightForWidth(self.lProperty.sizePolicy().hasHeightForWidth())
        self.lProperty.setSizePolicy(sizePolicy)
        self.lProperty.setAlignment(Qt.AlignHCenter)
        self.lFromUnit = QLabel("From Unit", self.gbResult)
        sizePolicy.setHeightForWidth(self.lFromUnit.sizePolicy().hasHeightForWidth())
        self.lFromUnit.setSizePolicy(sizePolicy)
        self.lFromUnit.setAlignment(Qt.AlignHCenter)
        self.lEqual = QLabel("=", self.gbResult)
        sizePolicy.setHeightForWidth(self.lEqual.sizePolicy().hasHeightForWidth())
        self.lEqual.setSizePolicy(sizePolicy)
        self.lEqual.setAlignment(Qt.AlignHCenter)
        self.lEqual.setMaximumWidth(20)
        self.lToFactor = QLabel("To Factor", self.gbResult)
        sizePolicy.setHeightForWidth(self.lToFactor.sizePolicy().hasHeightForWidth())
        self.lToFactor.setSizePolicy(sizePolicy)
        self.lToFactor.setAlignment(Qt.AlignHCenter)
        self.lToFactor.setMaximumWidth(100)
        self.lE = QLabel(unichr(int('00d7', 16)) + "E", self.gbResult)
        sizePolicy.setHeightForWidth(self.lE.sizePolicy().hasHeightForWidth())
        self.lE.setSizePolicy(sizePolicy)
        self.lE.setAlignment(Qt.AlignHCenter)
        self.lE.setMaximumWidth(20)
        self.lPower = QLabel("Power", self.gbResult)
        sizePolicy.setHeightForWidth(self.lPower.sizePolicy().hasHeightForWidth())
        self.lPower.setSizePolicy(sizePolicy)
        self.lPower.setAlignment(Qt.AlignHCenter)
        self.lPower.setMaximumWidth(50)
        self.lToUnit = QLabel("To Unit", self.gbResult)
        sizePolicy.setHeightForWidth(self.lToUnit.sizePolicy().hasHeightForWidth())
        self.lToUnit.setSizePolicy(sizePolicy)
        self.lToUnit.setAlignment(Qt.AlignHCenter)
        self.lFromUnicode = QLabel("Units", self.gbResult)
        sizePolicy.setHeightForWidth(self.lFromUnicode.sizePolicy().hasHeightForWidth())
        self.lFromUnicode.setSizePolicy(sizePolicy)
        self.lFromUnicode.setAlignment(Qt.AlignRight)
        self.lToUnicode = QLabel("Units", self.gbResult)
        sizePolicy.setHeightForWidth(self.lToUnicode.sizePolicy().hasHeightForWidth())
        self.lToUnicode.setSizePolicy(sizePolicy)
        self.lToUnicode.setAlignment(Qt.AlignRight)

        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.txtProperty = QLineEdit("", self.gbResult)
        sizePolicy.setHeightForWidth(self.txtProperty.sizePolicy().hasHeightForWidth())
        self.txtProperty.setSizePolicy(sizePolicy)
        self.txtProperty.setMinimumSize(self.txtSize)
        self.txtFromUnit = QLineEdit("", self.gbResult)
        sizePolicy.setHeightForWidth(self.txtFromUnit.sizePolicy().hasHeightForWidth())
        self.txtFromUnit.setSizePolicy(sizePolicy)
        self.txtFromUnit.setMinimumSize(self.txtSize)
        self.txtToFactor = QLineEdit("", self.gbResult)
        sizePolicy.setHeightForWidth(self.txtToFactor.sizePolicy().hasHeightForWidth())
        self.txtToFactor.setSizePolicy(sizePolicy)
        self.txtToFactor.setMaximumWidth(100)
        self.txtToFactor.setMaxLength(8)
        self.txtToFactor.setValidator(self.dbl_validator)
        self.txtPower = QLineEdit("", self.gbResult)
        sizePolicy.setHeightForWidth(self.txtPower.sizePolicy().hasHeightForWidth())
        self.txtPower.setSizePolicy(sizePolicy)
        self.txtPower.setMaximumWidth(50)
        self.txtPower.setMaxLength(3)
        self.txtPower.setValidator(self.dbl_validator)
        self.txtToUnit = QLineEdit("", self.gbResult)
        sizePolicy.setHeightForWidth(self.txtToUnit.sizePolicy().hasHeightForWidth())
        self.txtToUnit.setSizePolicy(sizePolicy)
        self.txtToUnit.setMinimumSize(self.txtSize)
        self.txtFromUnicode = QLineEdit("", self.gbResult)
        sizePolicy.setHeightForWidth(self.txtFromUnicode.sizePolicy().hasHeightForWidth())
        self.txtFromUnicode.setSizePolicy(sizePolicy)
        self.txtFromUnicode.setMinimumSize(self.txtSize)
        self.txtToUnicode = QLineEdit("", self.gbResult)
        sizePolicy.setHeightForWidth(self.txtToUnicode.sizePolicy().hasHeightForWidth())
        self.txtToUnicode.setSizePolicy(sizePolicy)
        self.txtToUnicode.setMinimumSize(self.txtSize)

        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.cbProperty = QComboBox(self.gbProperty)
        sizePolicy.setHeightForWidth(self.cbProperty.sizePolicy().hasHeightForWidth())
        self.cbProperty.setSizePolicy(sizePolicy)
        self.cbProperty.setMinimumSize(self.cbSize)
        self.cbFrom = QComboBox(self.gbUnits)
        sizePolicy.setHeightForWidth(self.cbFrom.sizePolicy().hasHeightForWidth())
        self.cbFrom.setSizePolicy(sizePolicy)
        self.cbFrom.setMinimumSize(self.cbSize)
        self.cbTo = QComboBox(self.gbUnits)
        sizePolicy.setHeightForWidth(self.cbTo.sizePolicy().hasHeightForWidth())
        self.cbTo.setSizePolicy(sizePolicy)
        self.cbTo.setMinimumSize(self.cbSize)

        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.radioExistProperty = QRadioButton("Existing Property", self.gbProperty)
        self.radioExistProperty.setSizePolicy(sizePolicy)
        self.radioExistProperty.setMinimumSize(self.radioSize)
        self.radioExistProperty.setChecked(True)
        self.radioNewProperty = QRadioButton("New Property", self.gbProperty)
        self.radioNewProperty.setSizePolicy(sizePolicy)
        self.radioNewProperty.setMinimumSize(self.radioSize)
        self.radioExistUnits = QRadioButton("Existing Units", self.gbUnits)
        self.radioExistUnits.setSizePolicy(sizePolicy)
        self.radioExistUnits.setMinimumSize(self.radioSize)
        self.radioExistUnits.setChecked(True)
        self.radioNewUnits = QRadioButton("New Units", self.gbUnits)
        self.radioNewUnits.setSizePolicy(sizePolicy)
        self.radioNewUnits.setMinimumSize(self.radioSize)
        
        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.chkDelProperty = QCheckBox("Delete Property", self.gbProperty)
        self.chkDelProperty.setSizePolicy(sizePolicy)
        self.chkDelProperty.setMinimumSize(self.chkSize)
        self.chkDelFromUnit = QCheckBox("Delete Unit", self.gbUnits)
        self.chkDelFromUnit.setSizePolicy(sizePolicy)
        self.chkDelFromUnit.setMinimumSize(self.chkSize)
        self.chkDelToUnit = QCheckBox("Delete Unit", self.gbUnits)
        self.chkDelToUnit.setSizePolicy(sizePolicy)
        self.chkDelToUnit.setMinimumSize(self.chkSize)
        
        size_policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        
        self.lb_status = QLabel("", self)
        self.lb_status.setTextFormat(Qt.RichText)
        
        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.btnSave = QPushButton(self.iconSave, "Save", self)
        sizePolicy.setHeightForWidth(self.btnSave.sizePolicy().hasHeightForWidth())
        self.btnSave.setSizePolicy(sizePolicy)
        self.btnSave.setMinimumSize(self.btnSize)
        self.btnCancel = QPushButton(self.iconCancel, "Cancel", self)
        sizePolicy.setHeightForWidth(self.btnCancel.sizePolicy().hasHeightForWidth())
        self.btnCancel.setSizePolicy(sizePolicy)
        self.btnCancel.setMinimumSize(self.btnSize)
        self.btnFromUnicode = QPushButton(unichr(int('03A9', 16)), self.gbResult)
        self.btnFromUnicode.setMaximumWidth(20)
        self.btnToUnicode = QPushButton(unichr(int('03A9', 16)), self.gbResult)
        self.btnToUnicode.setMaximumWidth(20)

        self.gbPropertyLayout = QGridLayout(self.gbProperty)
        self.gbPropertyLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gbPropertyLayout.addWidget(self.radioExistProperty, 0, 0)
        self.gbPropertyLayout.addWidget(self.lDummy, 0, 1)
        self.gbPropertyLayout.addWidget(self.cbProperty, 0, 2)
        self.gbPropertyLayout.addWidget(self.chkDelProperty, 0, 3)
        self.gbPropertyLayout.addWidget(self.radioNewProperty, 1, 0)
        self.gbPropertyLayout.addItem(self.hSpacer, 0, 4)

        self.gbUnitsLayout = QGridLayout(self.gbUnits)
        self.gbUnitsLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gbUnitsLayout.addWidget(self.radioExistUnits, 0, 0)
        self.gbUnitsLayout.addWidget(self.lUnitFrom, 0, 1)
        self.gbUnitsLayout.addWidget(self.cbFrom, 0, 2)
        self.gbUnitsLayout.addWidget(self.chkDelFromUnit, 0, 3)
        self.gbUnitsLayout.addWidget(self.radioNewUnits, 1, 0)
        self.gbUnitsLayout.addWidget(self.lUnitTo, 1, 1)
        self.gbUnitsLayout.addWidget(self.cbTo, 1, 2)
        self.gbUnitsLayout.addWidget(self.chkDelToUnit, 1, 3)
        self.gbUnitsLayout.addItem(self.hSpacer, 0, 4)

        self.gbResultLayout = QGridLayout(self.gbResult)
        self.gbResultLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gbResultLayout.addWidget(self.lProperty, 0, 0)
        self.gbResultLayout.addWidget(self.txtProperty, 1, 0)
        self.gbResultLayout.addWidget(self.lFromUnicode, 2, 0)
        self.gbResultLayout.addWidget(self.lFromUnit, 0, 1)
        self.gbResultLayout.addWidget(self.txtFromUnit, 1, 1)
        self.gbResultLayout.addWidget(self.txtFromUnicode, 2, 1)
        self.gbResultLayout.addWidget(self.btnFromUnicode, 2, 2)
        self.gbResultLayout.addWidget(self.lEqual, 1, 2)
        self.gbResultLayout.addWidget(self.lToFactor, 0, 3)
        self.gbResultLayout.addWidget(self.txtToFactor, 1, 3)
        self.gbResultLayout.addWidget(self.lToUnicode, 2, 3, 1, 3)
        self.gbResultLayout.addWidget(self.lE, 1, 4)
        self.gbResultLayout.addWidget(self.lPower, 0, 5)
        self.gbResultLayout.addWidget(self.txtPower, 1, 5)
        self.gbResultLayout.addWidget(self.lToUnit, 0, 6)
        self.gbResultLayout.addWidget(self.txtToUnit, 1, 6)
        self.gbResultLayout.addWidget(self.txtToUnicode, 2, 6)
        self.gbResultLayout.addWidget(self.btnToUnicode, 2, 7)

        self.hbButtonsLayout = QHBoxLayout()
        self.hbButtonsLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.hbButtonsLayout.addWidget(self.lb_status)
        self.hbButtonsLayout.addItem(self.hSpacer)
        self.hbButtonsLayout.addWidget(self.btnSave)
        self.hbButtonsLayout.addWidget(self.btnCancel)
        
        self.mainLayout = QVBoxLayout(self)
        self.mainLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.mainLayout.addWidget(self.gbProperty)
        self.mainLayout.addWidget(self.gbUnits)
        self.mainLayout.addWidget(self.gbResult)
        self.mainLayout.addLayout(self.hbButtonsLayout)

        self.cbProperty.addItems(self.convert.GetProperties())

        self.connect(self.cbProperty, SIGNAL("currentIndexChanged(int)"), self.OnPropertyCurrentIndexChanged)
        self.connect(self.cbFrom, SIGNAL("currentIndexChanged(int)"), self.OnFromCurrentIndexChanged)
        self.connect(self.cbTo, SIGNAL("currentIndexChanged(int)"), self.OnToCurrentIndexChanged)
        self.connect(self.radioExistProperty, SIGNAL("toggled(bool)"), self.OnExistPropertyToggled)
        self.connect(self.radioNewProperty, SIGNAL("toggled(bool)"), self.OnNewPropertyToggled)
        self.connect(self.radioExistUnits, SIGNAL("toggled(bool)"), self.OnExistUnitsToggled)
        self.connect(self.radioNewUnits, SIGNAL("toggled(bool)"), self.OnNewUnitsToggled)
        self.connect(self.txtProperty, SIGNAL("textChanged(QString)"), self.PrepareStatus)
        self.connect(self.txtFromUnit, SIGNAL("textChanged(QString)"), self.PrepareStatus)
        self.connect(self.txtFromUnicode, SIGNAL("textChanged(QString)"), self.PrepareStatus)
        self.connect(self.txtToUnit, SIGNAL("textChanged(QString)"), self.PrepareStatus)
        self.connect(self.txtToUnicode, SIGNAL("textChanged(QString)"), self.PrepareStatus)
        self.connect(self.txtToFactor, SIGNAL("textChanged(QString)"), self.PrepareStatus)
        self.connect(self.txtPower, SIGNAL("textChanged(QString)"), self.PrepareStatus)
        self.connect(self.btnSave, SIGNAL("clicked()"), self.OnBtnSaveClicked)
        self.connect(self.btnCancel, SIGNAL("clicked()"), self.Close)
        self.connect(self.btnFromUnicode, SIGNAL("clicked()"), self.OnBtnFromUnicodeClicked)
        self.connect(self.btnToUnicode, SIGNAL("clicked()"), self.OnBtnToUnicodeClicked)

    def Close(self):
        self.close()

    def OnBtnFromUnicodeClicked(self):
        if not self.usymbols:
            self.usymbols = UnicodeSymbols()
            self.usymbols.setModal(True)
        self.usymbols.show()
        self.usymbols.SetObj(self.txtFromUnicode)

    def OnBtnToUnicodeClicked(self):
        if not self.usymbols:
            self.usymbols = UnicodeSymbols()
            self.usymbols.setModal(True)
        self.usymbols.show()
        self.usymbols.SetObj(self.txtToUnicode)

    def OnPropertyCurrentIndexChanged(self):
        self.cbFrom.clear()
        self.cbTo.clear()
        property = self.cbProperty.currentText()
        units = QStringList()
        self.txtProperty.setText(property)
        if property !="" and not self.radioNewUnits.isChecked():
            units = self.convert.GetUnits(property)
            self.cbFrom.addItems(units)
            self.cbTo.addItems(units)
        elif property !="" and self.radioNewUnits.isChecked():
            units = self.convert.GetUnits(property)
            self.cbFrom.addItem("New")
            self.cbTo.addItems(units)
        else:
            self.ClearInputs()

    def OnExistPropertyToggled(self):
        self.cbProperty.show()
        self.chkDelProperty.show()
        self.cbProperty.setCurrentIndex(0)
        self.radioExistUnits.show()
        self.radioExistUnits.setChecked(True)
        self.chkDelProperty.setChecked(False)
        self.ClearInputs()

    def OnNewPropertyToggled(self):
        self.cbProperty.hide()
        self.chkDelProperty.hide()
        self.radioExistUnits.hide()
        self.txtProperty.setFocus()
        self.radioNewUnits.setChecked(True)
        self.cbTo.hide()
        self.lUnitTo.hide()
        self.chkDelProperty.setChecked(False)
        self.ClearInputs()

    def OnExistUnitsToggled(self):
        self.lUnitFrom.show()
        self.cbFrom.show()
        self.lUnitTo.show()
        self.cbTo.show()
        self.chkDelFromUnit.show()
        self.chkDelToUnit.show()
        self.chkDelFromUnit.setChecked(False)
        self.chkDelToUnit.setChecked(False)
        self.ClearInputs()

    def OnNewUnitsToggled(self):
        self.lUnitFrom.hide()
        self.cbFrom.hide()
        self.chkDelFromUnit.hide()
        self.chkDelToUnit.hide()
        self.chkDelFromUnit.setChecked(False)
        self.chkDelToUnit.setChecked(False)
        self.ClearInputs()
        self.cbProperty.setCurrentIndex(0)

    def OnFromCurrentIndexChanged(self):
        property = self.cbProperty.currentText()
        fromUnit = self.cbFrom.currentText()
        toUnit = self.cbTo.currentText()
        if self.radioNewUnits.isChecked() and property != "" and fromUnit != "" and toUnit != "":
            self.OnNewUnitChanged()
        elif property != "" and fromUnit != "" and toUnit != "":
            self.OnUnitChanged()

    def OnToCurrentIndexChanged(self):
        property = self.cbProperty.currentText()
        fromUnit = self.cbFrom.currentText()
        toUnit = self.cbTo.currentText()
        if self.radioNewUnits.isChecked() and property != "" and fromUnit != "" and toUnit != "":
            self.OnNewUnitChanged()
        elif property != "" and fromUnit != "" and toUnit != "":
            self.OnUnitChanged()

    def OnUnitChanged(self):
        property = self.cbProperty.currentText()
        fromUnit = self.cbFrom.currentText()
        toUnit = self.cbTo.currentText()
        fromFactor = self.convert.GetXMLData(property, fromUnit)[1].text
        toFactor = self.convert.GetXMLData(property, toUnit)[1].text
        fromE = self.convert.GetXMLData(property, fromUnit)[2].text
        toE = self.convert.GetXMLData(property, toUnit)[2].text
        fromUnicode = self.convert.GetXMLData(property, fromUnit)[3].text
        toUnicode = self.convert.GetXMLData(property, toUnit)[3].text
        factor = round(Decimal(fromFactor) / Decimal(toFactor), 6)
        power = Decimal(fromE) - Decimal(toE)
        self.txtFromUnit.setText(fromUnit)
        self.txtFromUnicode.setText(fromUnicode)
        self.txtToUnit.setText(toUnit)
        self.txtToUnicode.setText(toUnicode)
        self.txtToFactor.setText(str(factor))
        self.txtPower.setText(str(power))
        self.PrepareStatus()

    def OnNewUnitChanged(self):
        property = self.cbProperty.currentText()
        fromUnit = self.cbFrom.currentText()
        toUnit = self.cbTo.currentText()
        toUnicode = self.convert.GetXMLData(property, toUnit)[3].text
        self.txtFromUnit.setText(fromUnit)
        self.txtFromUnicode.setText("")
        self.txtToUnit.setText(toUnit)
        self.txtToUnicode.setText(toUnicode)
        self.txtToFactor.setText("1")
        self.txtPower.setText("0")
        self.PrepareStatus()

    def OnBtnSaveClicked(self):
        if self.SaveChangesDlg():
            if not self.MakeDataBackup():
                    self.ShowStatus("Could not back up data file", "red")
            property = str(self.cbProperty.currentText())
            newProperty = str(self.txtProperty.text())
            fromUnit = str(self.cbFrom.currentText())
            newFromUnit = str(self.txtFromUnit.text())
            toUnit = str(self.cbTo.currentText())
            newToUnit = str(self.txtToUnit.text())
            newFromUnicode = unicode(self.txtFromUnicode.text())
            newToUnicode = unicode(self.txtToUnicode.text())
            newToFactor = str(self.txtToFactor.text())
            newPower = str(self.txtPower.text())
            if self.chkDelProperty.isChecked() and property != "":
                if self.convert.DeleteProperty(property):
                    self.Close()
                else:
                    self.ShowStatus("Could not delete property", "red")
            elif self.chkDelFromUnit.isChecked() and fromUnit != "" and self.chkDelToUnit.isChecked() and toUnit != "":
                if self.convert.DeleteUnit(property, fromUnit) and self.convert.DeleteUnit(property, toUnit):
                    self.Close()
                else:
                    self.ShowStatus("Could not delete units", "red")
            elif self.chkDelFromUnit.isChecked() and fromUnit != "":
                if self.convert.DeleteUnit(property, fromUnit):
                    self.Close()
                else:
                    self.ShowStatus("Could not delete units", "red")
            elif self.chkDelToUnit.isChecked() and toUnit != "":
                if self.convert.DeleteUnit(property, toUnit):
                    self.Close()
                else:
                    self.ShowStatus("Could not delete unit", "red")
            elif newProperty != "" and newFromUnit != "" and newToUnit != "" and newFromUnicode != "" and newToUnicode != "" and newToFactor != "" and newPower != "":
                if self.convert.SaveUnits(property, newProperty, fromUnit, newFromUnit, toUnit, newToUnit, newFromUnicode, newToUnicode, newToFactor, newPower):
                    self.Close()
                else:
                    self.ShowStatus("Could not save changes", "red")
            else:
                self.ShowStatus("Could not save changes, some fields are empty", "red")

    def PrepareStatus(self):
        property = self.txtProperty.text()
        fromUnit = self.txtFromUnit.text()
        toUnit = self.txtToUnit.text()
        fromUnicode = self.UnicodeToString(self.txtFromUnicode.text())
        toUnicode = self.UnicodeToString(self.txtToUnicode.text())
        factor = self.txtToFactor.text()
        power = self.txtPower.text()
        statusString = property + ": 1 " + fromUnit + " (" + fromUnicode + ") = " + factor + "E" + power + " " + toUnit + " (" + toUnicode + ")"
        self.ShowStatus(statusString, "blue")

    def UnicodeToString(self, uString):
        return uString
        """
        sep = ','
        string = ""
        for uWord in str(uString).split(sep):
            if len(uWord) == 4:
                string += unichr(int(uWord, 16))
        return string
        """

    def ClearInputs(self):
        self.txtProperty.setText("")
        self.txtFromUnit.setText("")
        self.txtFromUnicode.setText("")
        self.txtToUnit.setText("")
        self.txtToUnicode.setText("")
        self.txtToFactor.setText("1")
        self.txtPower.setText("0")

    def SaveChangesDlg(self):
        msg = "You are about to make changes to units data file.\n"
        msg += "A backup copy of the data file will be created.\n"
        msg += "Press OK to save changes or Cancel to return."
        dlg = QMessageBox(self)
        dlg.setIcon(QMessageBox.Warning)
        dlg.setWindowTitle("Save Changes - Cute Converter")
        dlg.setText(msg)
        dlg.addButton(QMessageBox.Ok)
        dlg.addButton(QMessageBox.Cancel)
        dlg.setDefaultButton(QMessageBox.Cancel)
        button = dlg.exec_()
        if (button == QMessageBox.Ok):
            return True
        else:
            return False        

    def MakeDataBackup(self):
        data_file_info = QFileInfo(self.convert.xmlFile)
        data_file_dir = self.script_path + "/data"
        data_file_base = data_file_info.completeBaseName()
        data_file_ext = data_file_info.completeSuffix()
        backup_file_ext = data_file_info.completeSuffix() + "~"
        data_file_name = data_file_dir + "/" + data_file_base + "." + data_file_ext
        backup_file_name = data_file_dir + "/" + data_file_base + "." + backup_file_ext
        data_file = QFile(data_file_name)
        backup_file = QFile(backup_file_name)
        if self.FileExists(data_file_name):
            if self.FileExists(backup_file_name):
                backup_file.remove()
                data_file.copy(backup_file_name)
                return True
            else:
                data_file.copy(backup_file_name)
                return True
        else:
            return False
        
    def FileExists(self, file_name):
        file = QFile(file_name)
        if (file.exists()):
            return True
        else:
            return False

    def closeEvent(self, event):
        self.ClearInputs()
        self.chkDelProperty.setChecked(False)
        self.chkDelFromUnit.setChecked(False)
        self.chkDelToUnit.setChecked(False)
        self.radioExistProperty.setChecked(True)
        self.radioExistUnits.setChecked(True)
        self.cbProperty.clear()
        self.cbProperty.addItems(self.convert.GetProperties())
        self.emit(SIGNAL("closed()"))
        
    def ShowStatus(self, msg,  color):
        self.lb_status.setText("<font color=\"" + color + "\">" + msg + "</font>")
