# -*- coding: utf-8 -*-
"""
Program:
    Cute Converter
    (qt-converter)
    MainWindow.py

Author:
    Alex Borisov <>

Copyright (c) 2010 Alex Borisov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from Converter import Converter
from CustomUnits import CustomWindow
import sys
import resources

class MainWindow(QMainWindow):
    def __init__(self, parent = None):
        QMainWindow.__init__(self)

        self.version = "2.0.2"
        
        self.custom_win = None
        self.convert = None
        self.factors_file = QString("")
        self.script_path = sys.path[0]
        self.ReadSettings()
        
        self.MainUI()

    def ReadSettings(self):
        if sys.platform == "win32":
            settings = QSettings("PyQt-converter.ini", QSettings.IniFormat)
        else:
            settings = QSettings(QDir.homePath() + "/.qt-converter/qt-converter.conf", QSettings.NativeFormat)
        settings.beginGroup("MainWindow")
        self.factors_file = settings.value("factors_file", self.script_path + "/data/factors.xml").toString()
        settings.endGroup()
    
    def MainUI(self):
        self.winIcon = QIcon(":/myresources/icons/converter.ico")
        self.setWindowIcon(self.winIcon)
        self.winSize = QSize(300, 200)
        self.setWindowTitle("Cute Converter")
        self.resize(self.winSize)

        self.dbl_validator = QDoubleValidator()
        self.dbl_validator.setNotation(QDoubleValidator.StandardNotation)

        self.hSpacer = QSpacerItem(2, 20, QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        self.vSpacer = QSpacerItem(20, 2, QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)

        self.mainWidget = QWidget(self)

        self.lSize = QSize(100, 25)
        self.txtSize = QSize(160, 25)
        self.cbSize = QSize(240, 25)
        self.spinSize = QSize(50, 25)

        self.iconQuit = QIcon(":/myresources/icons/quit.svg")
        self.iconFindFactors = QIcon(":/myresources/icons/search.svg")
        self.iconCheckFactors = QIcon(":/myresources/icons/check.svg")
        self.iconCustomFactors = QIcon(":/myresources/icons/custom.svg")
        self.iconAbout = QIcon(":/myresources/icons/about.svg")

        self.actionQuit = QAction(self.iconQuit, "&Quit", self)
        self.actionFindFactors = QAction(self.iconFindFactors, "&Find Factors", self)
        self.actionCheckFactors = QAction(self.iconCheckFactors, "&Check Factors", self)
        self.actionCustomFactors = QAction(self.iconCustomFactors, "C&ustom Factors", self)
        self.actionAbout = QAction(self.iconAbout, "&About", self)

        self.menuBar = QMenuBar(self)
        self.menuBar.Enabled = True
        self.setMenuBar(self.menuBar)

        self.menuFile = QMenu("&File",self.menuBar);
        self.menuTools = QMenu("&Tools", self.menuBar);
        self.menuHelp = QMenu("&Help", self.menuBar);

        self.menuBar.addMenu(self.menuFile)
        self.menuBar.addMenu(self.menuTools)
        self.menuBar.addMenu(self.menuHelp)

        self.menuFile.addAction(self.actionQuit)
        self.menuTools.addAction(self.actionFindFactors)
        self.menuTools.addAction(self.actionCheckFactors)
        self.menuTools.addAction(self.actionCustomFactors)
        self.menuHelp.addAction(self.actionAbout)

        self.toolBar = QToolBar(self)
        self.toolBar.addAction(self.actionFindFactors)
        self.toolBar.addAction(self.actionCheckFactors)
        self.toolBar.addAction(self.actionCustomFactors)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionAbout)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionQuit)
        self.addToolBar(Qt.TopToolBarArea, self.toolBar)

        self.statusBar = QStatusBar(self)
        self.setStatusBar(self.statusBar)

        self.gbProperty = QGroupBox("Property", self.mainWidget)
        self.gbUnits = QGroupBox("Units", self.mainWidget)
        self.gbResult = QGroupBox("Result", self.mainWidget)
        
        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.lProperty = QLabel("Property", self.gbProperty)
        sizePolicy.setHeightForWidth(self.lProperty.sizePolicy().hasHeightForWidth())
        self.lProperty.setSizePolicy(sizePolicy)
        self.lProperty.setMinimumSize(self.lSize)
        self.lFrom = QLabel("From", self.gbUnits)
        sizePolicy.setHeightForWidth(self.lFrom.sizePolicy().hasHeightForWidth())
        self.lFrom.setSizePolicy(sizePolicy)
        self.lFrom.setMinimumSize(self.lSize)
        self.lTo = QLabel("To", self.gbUnits)
        sizePolicy.setHeightForWidth(self.lTo.sizePolicy().hasHeightForWidth())
        self.lTo.setSizePolicy(sizePolicy)
        self.lTo.setMinimumSize(self.lSize)
        self.lInput = QLabel("Input", self.gbResult)
        sizePolicy.setHeightForWidth(self.lInput.sizePolicy().hasHeightForWidth())
        self.lInput.setSizePolicy(sizePolicy)
        self.lInput.setMinimumSize(self.lSize)
        self.lOutput = QLabel("Output", self.gbResult)
        sizePolicy.setHeightForWidth(self.lOutput.sizePolicy().hasHeightForWidth())
        self.lOutput.setSizePolicy(sizePolicy)
        self.lOutput.setMinimumSize(self.lSize)
        self.txtOutput = QLabel("", self.gbResult)
        sizePolicy.setHeightForWidth(self.txtOutput.sizePolicy().hasHeightForWidth())
        self.txtOutput.setSizePolicy(sizePolicy)
        self.txtOutput.setMinimumSize(self.lSize)
        self.txtOutput.setFrameStyle(QFrame.Panel)
        self.txtOutput.setLineWidth(1)
        self.lDecimals = QLabel("Decimals", self.gbResult)
        sizePolicy.setHeightForWidth(self.lDecimals.sizePolicy().hasHeightForWidth())
        self.lDecimals.setSizePolicy(sizePolicy)
        self.lDecimals.setMinimumSize(self.lSize)
        self.lInputUnit = QLabel("", self.gbResult)
        sizePolicy.setHeightForWidth(self.lInputUnit.sizePolicy().hasHeightForWidth())
        self.lInputUnit.setSizePolicy(sizePolicy)
        self.lInputUnit.setMinimumSize(self.lSize)
        self.lOutputUnit = QLabel("", self.gbResult)
        sizePolicy.setHeightForWidth(self.lOutputUnit.sizePolicy().hasHeightForWidth())
        self.lOutputUnit.setSizePolicy(sizePolicy)
        self.lOutputUnit.setMinimumSize(self.lSize)

        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.txtInput = QLineEdit("1", self.gbResult)
        sizePolicy.setHeightForWidth(self.txtInput.sizePolicy().hasHeightForWidth())
        self.txtInput.setSizePolicy(sizePolicy)
        self.txtInput.setMinimumSize(self.txtSize)
        self.txtInput.setMaxLength(21)
        self.txtInput.setValidator(self.dbl_validator)
        self.txtInput.selectAll()
        self.txtInput.setFocus()

        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.cbProperty = QComboBox(self.gbProperty)
        sizePolicy.setHeightForWidth(self.cbProperty.sizePolicy().hasHeightForWidth())
        self.cbProperty.setSizePolicy(sizePolicy)
        self.cbProperty.setMinimumSize(self.cbSize)
        self.cbFrom = QComboBox(self.gbUnits)
        sizePolicy.setHeightForWidth(self.cbFrom.sizePolicy().hasHeightForWidth())
        self.cbFrom.setSizePolicy(sizePolicy)
        self.cbFrom.setMinimumSize(self.cbSize)
        self.cbTo = QComboBox(self.gbUnits)
        sizePolicy.setHeightForWidth(self.cbTo.sizePolicy().hasHeightForWidth())
        self.cbTo.setSizePolicy(sizePolicy)
        self.cbTo.setMinimumSize(self.cbSize)

        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.spinDecimals = QSpinBox(self.gbResult)
        self.spinDecimals.setSizePolicy(sizePolicy)
        self.spinDecimals.setMinimumSize(self.spinSize)
        self.spinDecimals.setRange(0, 10)
        self.spinDecimals.setValue(2)
        
        self.gbPropertyLayout = QGridLayout(self.gbProperty)
        self.gbPropertyLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gbPropertyLayout.addWidget(self.lProperty, 0, 0)
        self.gbPropertyLayout.addWidget(self.cbProperty, 0, 1)
        self.gbPropertyLayout.addItem(self.hSpacer, 0, 2)

        self.gbUnitsLayout = QGridLayout(self.gbUnits)
        self.gbUnitsLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gbUnitsLayout.addWidget(self.lFrom, 0, 0)
        self.gbUnitsLayout.addWidget(self.cbFrom, 0, 1)
        self.gbUnitsLayout.addItem(self.hSpacer, 0, 2)
        self.gbUnitsLayout.addWidget(self.lTo, 1, 0)
        self.gbUnitsLayout.addWidget(self.cbTo, 1, 1)
        self.gbUnitsLayout.addItem(self.hSpacer, 1, 2)

        self.gbResultLayout = QGridLayout(self.gbResult)
        self.gbResultLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.gbResultLayout.addWidget(self.lInput, 0, 0)
        self.gbResultLayout.addWidget(self.txtInput, 0, 1)
        self.gbResultLayout.addWidget(self.lInputUnit, 0, 2)
        self.gbResultLayout.addWidget(self.lOutput, 1, 0)
        self.gbResultLayout.addWidget(self.txtOutput, 1, 1)
        self.gbResultLayout.addWidget(self.lOutputUnit, 1, 2)
        self.gbResultLayout.addWidget(self.lDecimals, 2, 0)
        self.gbResultLayout.addWidget(self.spinDecimals, 2, 1)
        self.gbResultLayout.addItem(self.hSpacer, 2, 2)

        self.mainLayout = QVBoxLayout(self.mainWidget)
        self.mainLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.mainLayout.addWidget(self.gbProperty)
        self.mainLayout.addWidget(self.gbUnits)
        self.mainLayout.addWidget(self.gbResult)

        self.mainWidget.setLayout(self.mainLayout)
        self.setCentralWidget(self.mainWidget)

        if self.CheckFactors():
            self.convert = Converter(self.factors_file,  self)
            self.cbProperty.addItems(self.convert.GetProperties())

        self.connect(self.actionQuit, SIGNAL("triggered()"), self.Quit)
        self.connect(self.actionAbout, SIGNAL("triggered()"), self.OnAboutTriggered)
        self.connect(self.actionFindFactors, SIGNAL("triggered()"), self.OnFindFactorsTriggered)
        self.connect(self.actionCheckFactors, SIGNAL("triggered()"), self.OnCheckFactorsTriggered)
        self.connect(self.actionCustomFactors, SIGNAL("triggered()"), self.OnCustomFactorsTriggered)
        self.connect(self.cbProperty, SIGNAL("currentIndexChanged(int)"), self.OnPropertyCurrentIndexChanged)
        self.connect(self.cbFrom, SIGNAL("currentIndexChanged(int)"), self.OnFromCurrentIndexChanged)
        self.connect(self.cbTo, SIGNAL("currentIndexChanged(int)"), self.OnToCurrentIndexChanged)
        self.connect(self.txtInput, SIGNAL("textChanged(QString)"), self.OnInputTextChanged)
        self.connect(self.spinDecimals, SIGNAL("valueChanged(int)"), self.OnDecimalsValueChanged)

    def Quit(self):
        self.WriteSettings()
        self.close()

    def WriteSettings(self):
        if sys.platform == "win32":
            settings = QSettings("qt-converter.ini", QSettings.IniFormat)
        else:
            settings = QSettings(QDir.homePath() + "/.qt-converter/qt-converter.conf", QSettings.NativeFormat)
        settings.beginGroup("MainWindow")
        settings.setValue("factors_file", self.factors_file)
        settings.endGroup()

    def OnAboutTriggered(self):
        msg = "Cute Converter - Qt-based International System of Units Converter\n\n"
        msg += "Version " + self.version + "\n\n"
        msg += unichr(int('00A9', 16)) + " 2010-" + QDateTime().currentDateTime().toString("yyyy") + ", Alex Borisov\n\n"
        msg += "Distributed under GPL ver. 3"
        QMessageBox.about(self, "About - Cute Converter", msg)

    def OnCheckFactorsTriggered(self):
        self.CheckFactors()

    def OnFindFactorsTriggered(self):
        self.FindFactors()
    
    def OnCustomFactorsTriggered(self):
        if self.custom_win:
            self.custom_win.show()
        else:
            self.custom_win= CustomWindow(self.convert,  self)
            self.custom_win.show()
            self.connect(self.custom_win, SIGNAL("closed()"), self.OnCustomWinClosed)
    
    def OnCustomWinClosed(self):
        self.cbProperty.clear()
        self.cbProperty.addItems(self.convert.GetProperties())

    def CheckFactors(self):
        error = "Default "
        error = error + "unit factors file could not be found on system.\n"
        error = error + "If unit factors file is installed on your system,\nyou can manually find and select it from menu."
        file = QFile(self.factors_file)

        if file.exists():
            self.statusBar.setStyleSheet("color: rgb(0, 0, 255)")
            self.statusBar.showMessage("Welcome to Cute Converter. Unit factors file is found", 5000)
            self.cbProperty.setDisabled(False)
            self.cbFrom.setDisabled(False)
            self.cbTo.setDisabled(False)
            self.txtInput.setDisabled(False)
            self.txtOutput.setDisabled(False)
            self.spinDecimals.setDisabled(False)
            return True
        else:
            self.statusBar.setStyleSheet("color: rgb(255, 0, 0)")
            self.statusBar.showMessage("Cute Converter cannot function without unit factors file", 5000)
            self.cbProperty.setDisabled(True)
            self.cbFrom.setDisabled(True)
            self.cbTo.setDisabled(True)
            self.txtInput.setDisabled(True)
            self.txtOutput.setDisabled(True)
            self.spinDecimals.setDisabled(True)
            QMessageBox.warning(self, "Warning - Cute Converter", error)
            return False

    def FindFactors(self):
        fDlgTitle = "Select unit factors file - Cute Converter"
        fDlgFilter = "XML files (*.xml)"
        factors_dir = QDir(QDir.rootPath())
        file_name = QFileDialog.getOpenFileName(self, fDlgTitle, factors_dir.absolutePath(), fDlgFilter)
        factors_info = QFileInfo(file_name)
        if factors_info.isFile():
            self.factors_file = file_name
            self.cbProperty.setDisabled(False)
            self.cbFrom.setDisabled(False)
            self.cbTo.setDisabled(False)
            self.txtInput.setDisabled(False)
            self.txtOutput.setDisabled(False)
            self.spinDecimals.setDisabled(False)
            self.convert = Converter(self.factors_file,  self)
            self.cbProperty.clear()
            self.cbProperty.addItems(self.convert.GetProperties())
        else:
            self.statusBar.setStyleSheet("color: rgb(255, 0, 0)")
            self.statusBar.showMessage("No file selected", 5000)

    def OnPropertyCurrentIndexChanged(self):
        self.cbFrom.clear()
        self.cbTo.clear()
        self.statusBar.showMessage("")
        property = self.cbProperty.currentText()
        if property != "":
            units = self.convert.GetUnits(property)
            self.cbFrom.addItems(units)
            self.cbTo.addItems(units)

    def OnFromCurrentIndexChanged(self):
        property = self.cbProperty.currentText()
        fromUnit = self.cbFrom.currentText()
        toUnit = self.cbTo.currentText()
        input = self.txtInput.text()
        decimals = self.spinDecimals.value()
        if property != "" and fromUnit != "" and toUnit != "" and input != "":
            output = self.convert.Convert(property, fromUnit, toUnit, input)
            inputUnit = self.convert.UnicodeToString(property, fromUnit)
            outputUnit = self.convert.UnicodeToString(property, toUnit)
            self.txtOutput.setText(self.convert.SetDecimals(output, decimals))
            self.lInputUnit.setText(inputUnit)
            self.lOutputUnit.setText(outputUnit)
            self.ShowStatus()
        self.txtInput.selectAll()
        self.txtInput.setFocus()

    def OnToCurrentIndexChanged(self):
        property = self.cbProperty.currentText()
        fromUnit = self.cbFrom.currentText()
        toUnit = self.cbTo.currentText()
        input = self.txtInput.text()
        decimals = self.spinDecimals.value()
        if property != "" and fromUnit != "" and toUnit != "" and input != "":
            output = self.convert.Convert(property, fromUnit, toUnit, input)
            inputUnit = self.convert.UnicodeToString(property, fromUnit)
            outputUnit = self.convert.UnicodeToString(property, toUnit)
            self.txtOutput.setText(self.convert.SetDecimals(output, decimals))
            self.lInputUnit.setText(inputUnit)
            self.lOutputUnit.setText(outputUnit)
            self.ShowStatus()
        self.txtInput.selectAll()
        self.txtInput.setFocus()

    def OnInputTextChanged(self):
        property = self.cbProperty.currentText()
        fromUnit = self.cbFrom.currentText()
        toUnit = self.cbTo.currentText()
        input = self.txtInput.text()
        decimals = self.spinDecimals.value()
        if property != "" and fromUnit != "" and toUnit != "" and input != "":
            output = self.convert.Convert(property, fromUnit, toUnit, input)
            inputUnit = self.convert.UnicodeToString(property, fromUnit)
            outputUnit = self.convert.UnicodeToString(property, toUnit)
            self.txtOutput.setText(self.convert.SetDecimals(output, decimals))
            self.lInputUnit.setText(inputUnit)
            self.lOutputUnit.setText(outputUnit)
            self.ShowStatus()

    def OnDecimalsValueChanged(self):
        self.OnInputTextChanged()
        decimals = self.spinDecimals.value()
        value = self.txtOutput.text()
        self.txtOutput.setText(self.convert.SetDecimals(value, decimals))

    def ShowStatus(self):
        property = self.cbProperty.currentText()
        fromUnit = self.cbFrom.currentText()
        fromInput = self.lInputUnit.text()
        toUnit = self.cbTo.currentText()
        toOutput = self.lOutputUnit.text()
        self.statusBar.setStyleSheet("color: rgb(0, 0, 255)")
        self.statusBar.showMessage(self.convert.StatusString(property, fromUnit, toUnit, fromInput, toOutput))
