# -*- coding: utf-8 -*-
"""
Program:
    Cute Converter
    (qt-converter)
    UnicodeConverter.py

Author:
    Alex Borisov <>

Copyright (c) 2010 Alex Borisov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from CharacterMap import CharacterWidget

class UnicodeSymbols(QDialog):
    def __init__(self, parent = None):
        QDialog.__init__(self)

        self.UnicodeSymbolsUI()
        self._obj = None

    def UnicodeSymbolsUI(self):
        self.winIcon = QIcon(":/myresources/icons/converter.ico")
        self.setWindowIcon(self.winIcon)
        self.winSize = QSize(560, 500)
        self.setWindowTitle("Unicode Symbols - Cute Converter")
        self.resize(self.winSize)

        self.hSpacer = QSpacerItem(20, 20, QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        self.vSpacer = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)

        self.txtSize = QSize(200, 25)
        self.btnSize = QSize(35, 30)

        self.iconClear = QIcon(":/myresources/icons/clear.svg")
        self.iconSave = QIcon(":/myresources/icons/save.svg")
        self.iconCancel = QIcon(":/myresources/icons/cancel.svg")

        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.txtUnicode = QLineEdit("", self)
        self.txtUnicode.setReadOnly(True)
        sizePolicy.setHeightForWidth(self.txtUnicode.sizePolicy().hasHeightForWidth())
        self.txtUnicode.setSizePolicy(sizePolicy)
        self.txtUnicode.setMinimumSize(self.txtSize)

        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        
        self.btnClear = QPushButton(self.iconClear, "Clear", self)
        sizePolicy.setHeightForWidth(self.btnClear.sizePolicy().hasHeightForWidth())
        self.btnClear.setSizePolicy(sizePolicy)
        self.btnClear.setMinimumSize(self.btnSize)
        self.btnSave = QPushButton(self.iconSave, "Save", self)
        sizePolicy.setHeightForWidth(self.btnSave.sizePolicy().hasHeightForWidth())
        self.btnSave.setSizePolicy(sizePolicy)
        self.btnSave.setMinimumSize(self.btnSize)
        self.btnCancel = QPushButton(self.iconCancel, "Cancel", self)
        sizePolicy.setHeightForWidth(self.btnCancel.sizePolicy().hasHeightForWidth())
        self.btnCancel.setSizePolicy(sizePolicy)
        self.btnCancel.setMinimumSize(self.btnSize)

        self.scrollArea = QScrollArea()
        self.characterWidget = CharacterWidget()
        self.scrollArea.setWidget(self.characterWidget)

        self.clipboard = QApplication.clipboard()

        self.hbButtonsLayout = QHBoxLayout()
        self.hbButtonsLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.hbButtonsLayout.addItem(self.hSpacer)
        self.hbButtonsLayout.addWidget(self.btnClear)
        self.hbButtonsLayout.addItem(self.hSpacer)
        self.hbButtonsLayout.addWidget(self.btnSave)
        self.hbButtonsLayout.addWidget(self.btnCancel)
        self.hbButtonsLayout.addItem(self.hSpacer)
        
        self.mainLayout = QVBoxLayout(self)
        self.mainLayout.sizeConstraint = QLayout.SetDefaultConstraint
        self.mainLayout.addWidget(self.txtUnicode)
        self.mainLayout.addWidget(self.scrollArea)
        self.mainLayout.addItem(self.vSpacer)
        self.mainLayout.addLayout(self.hbButtonsLayout)

        self.characterWidget.characterSelected.connect(self.insertCharacter)
        self.connect(self.btnClear, SIGNAL("clicked()"), self.OnBtnClearClicked)
        self.connect(self.btnSave, SIGNAL("clicked()"), self.OnBtnSaveClicked)
        self.connect(self.btnCancel, SIGNAL("clicked()"), self.OnBtnCancelClicked)

    def OnBtnSaveClicked(self):
        self._obj.setText(self.txtUnicode.text())
        self.UpdateClipboard()
        self.Close()

    def OnBtnCancelClicked(self):
        self.Close()

    def OnBtnClearClicked(self):
        self.txtUnicode.clear()

    def Close(self):
        self.txtUnicode.clear()
        self.close()

    def insertCharacter(self, character):
        self.txtUnicode.insert(character)

    def UpdateClipboard(self):
        self.clipboard.setText(self.txtUnicode.text(), QClipboard.Clipboard)
        self.clipboard.setText(self.txtUnicode.text(), QClipboard.Selection)

    def SetObj(self, obj):
        self._obj = obj
        self.txtUnicode.setText(self._obj.text())
